/*> driver.h <*/
/* This file provides the description of the memory and I/O worlds for
 * the VLSI ARM600 VY86PID board.
 *
 * $Revision: 0.1 $
 *   $Author: jsmith $
 *     $Date: 941103 $
 *
 * Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
 *
 * "@(#)driver.h	1.3	96/06/07 VLSI"
 */
/*---------------------------------------------------------------------------*/

#ifndef __cdemon_driver_h
#define __cdemon_driver_h

/*---------------------------------------------------------------------------*/

#define SYSID   "VPID"  /* 4character system identification value */

/*---------------------------------------------------------------------------*/
/* The following define the number of IRQ and FIQ sources that are
 * available through C-Demon for this architecture. These values are
 * used to limit the C-Demon vector processing code.
 */
#define NUMIRQS (8)
#define NUMFIQS (8)

/*---------------------------------------------------------------------------*/

#define IOBase          0x02000000

#ifdef PID3_REV2
#define SER_IOBASE	IOBase
/*
 * The ethernet addressible space is starting at 0x40020 to
 * 0x4003f.  Don't want to change all the register offsets.
 * So, we use 0x3ffe0 as ethernet base.
 */
#define ETHER_IOBASE	(IOBase + 0x40020)
#else
#define SER_IOBASE	IOBase
#define ETHER_IOBASE	(IOBase + 0x40)
#endif

/*
 *Serial Port Registers
 *---------------------
 *offsets from the base
 */
#define RBR             0x20    /* Receive Buffer Reg.                  R/O*/
#define THR             0x20    /* Transmitter Holding Reg.             W/O*/

#define IER             0x24    /* Interrupt Enable Reg.                R/W*/

#define IIR             0x28    /* Interrupt Identificatio Reg.         R/O*/
#define FCR             0x28    /* FIFO Control Reg.                    W/O*/

#define LCR             0x2C    /* Line Control Reg.                    R/W*/
#define MCR             0x30    /* Modem Control Reg.                   R/W*/

#define LSR             0x34    /* Line Status Reg.                     R/W*/
#define MSR             0x38    /* Modem Status Reg.                    R/W*/
#define SCR             0x3C    /* Scratch Reg.                         R/W*/

#define DLL             0x20
#define DLM             0x24

/*----------------------------
 *Line Control Register (LCR)
 *----------------------------
 */
#define DLAccess        0x80    /* 0=Access Rx buffer,1=Div. latch access*/
#define ForceBreak      0x40    /* 0=break disable, 1=break enable*/
#define StickParity     0x20    /* 0=disable, 1=enable*/
#define EvenParity      0x10    /* 0=odd, 1=even*/
#define ParityEnable    0x8     /* 0=disable, 1=enable*/
#define StopSelect      0x4     /* 0=1 stop bit, 1=2 stop bits*/
#define WordLength      0x3     /* 3=8 bits, 2=7 bits, 1=6 bits, 00=5 bits*/
/*
 *value for LCR :
 *- word length                 -> 8 bits
 *- stop bits                   -> 1
 *- Parity                              -> disabled
 *- Parity select               -> odd
 *- stick parity                -> disabled
 *- Break Control               -> Enabled
 *- Div latch access    -> off
 */
#define LCRvalue        WordLength

/*-----------------------------
 *Divisor Latch Register (DLR)
 *-----------------------------
 *clock = 3.6864 MHz OR
 *clock = 1.8432 MHz
 *
 * NOTE: file XTAL.EXE (from XTAL.BAS) was used to calculate the values
 *
 *My calculations show that you divide the clock by 16 and divide it by the
 *desired baud rate to get the MAGIC number assigned to the following. GIC
 */

#if 0 /* 3.6864MHz crystal  */
#define SYSINFO " (3.6864MHz serial crystal)"
#define BR9600   (24)   /*   9600 baud */
#define BR19200  (12)   /*  19200 baud */
#define BR38400   (6)   /*  34000 baud */
#define BR57600   (4)   /*  57600 baud */
#define BR115200  (2)   /* 115200 baud */
#define BRLOCAL   (0)   /* full speed */
#else /* 1.8432MHz crystal */
#define SYSINFO " (1.8432MHz serial crystal)"
#define BR9600    (12)  /*   9600 baud */
#define BR19200    (6)  /*  19200 baud */
#define BR38400    (3)  /*  34000 baud */
#define BR57600    (2)  /*  57600 baud */
#define BR115200   (0)  /* 115200 baud */
#define BRLOCAL    (0)  /* full speed  for Apple Local Talk (115.2K baud) */
#endif

#define DLLvalue        (BR9600 % 256) /* default is 9600 baud */
#define DLMvalue        (BR9600 / 256)

/*--------------------------------
 *Interrupt Enable Register (IER)
 *--------------------------------
 */
#define EnModemStatusInt        0x8     /* 1=enable modem status interrupt*/
#define EnRxLineStatusInt       0x4     /* 1=enables Rx line status interrupt*/
#define EnTxRegEmptyInt         0x2     /* 1=enables Tx holding reg empty interrupt*/
#define EnRxDataAvailInt        0x1     /* 1=enables Rx data available interrupt*/

/*
 *value for IER :
 *- enable INT for Rx data available
 *- enable INT for Rx line status
 */
/*#define       IERvalue                        EnRxDataAvailInt+EnRxLineStatusInt+EnTxRegEmptyInt*/
#define IERvalue                0

/*----------------------------------------
 *Interrupt Identification Register (IIR)
 *----------------------------------------
 */
#define ModemStatus             (0x0 << 1)
#define TxHoldingRegEmpty       (0x1 << 1)
#define RxDataAvailable         (0x2 << 1)
#define RxLineStatus            (0x3 << 1)
#define IntTypeMask             (0x3 << 1)      /* mask to keep b[2:1] only*/
#define InterruptPending        0x1

/*---------------------------
 *Line Status Register (LSR)
 *---------------------------
 */
#define FifoError               0x80
#define TxEmpty                 0x40
#define TxHoldEmpty             0x20
#define RxBreakDetectError      0x10
#define RxFramingError          0x08
#define RxParityError           0x04
#define RxOverrunError          0x02
#define RxDataReady             0x01
/*
 * FIFO Control Register (FCR)
 */
#define RMode                   0x8
#define TxFIFOclear             0x4
#define RxFIFOclear             0x2
#define TxRxFIFOen              0x1

/*-----------------------------
 *Modem Control Register (MCR)
 *-----------------------------
 */
#define LoopBackEnable          0x10
#define MCRInterruptEnable      0x8
#define RequestToSend           0x2
#define DataTransmReady         0x1

/*#define MCRvalue              (RequestToSend | DataTransmReady | MCRInterruptEnable) */
#define MCRvalue                (RequestToSend | DataTransmReady)

/*----------------------
 *Modem Status Register
 *----------------------
 */
#define DataSetReady            0x20
#define DeltaDSR                0x2

/*-------------------------------
 *16C551 Parallel Port Registers
 *-------------------------------
 *offsets from the IOBase
 */
#define PDRW                    0x60    /* Parallel Data Read & Write Reg.*/
#define PSR                     0x64    /* Parallel Status Reg.*/
#define PCR                     0x68    /* Parallel Control Reg.*/
#define GPIO                    0x6C    /* General Purpope I/O Reg. (LEDs)*/
/*----------------
 *LED definitions
 *----------------
 */
#ifdef PID3_REV2
#define LEDport                 0x400020
#else
#define LEDport                 GPIO
#endif

#define LED1                    0x1
#define LED2                    0x2
#define LED3                    0x4
#define LED4                    0x8
#define LEDall                  0xF
/*------------------
 *INTC PGA Registers
 *------------------
 */                             /* Offset from IOBase*/
#define IRQS                    0x00    /* IRQ status */
#define IRQRST                  0x00    /* IRQ reset */
#define IRQM                    0x04    /* IRQ mask */
#define FIQS                    0x08    /* FIQ status */
#define FIQM                    0x0C    /* FIQ mask */
/*
 *I R Q   S t a t u s   a n d   E n a b l e   B i t s  (IRQS and IRQM)
 */
#define SerialPortIT            0x1     /* IRQ0*/
#define TimerIT                 0x2     /* IRQ1*/
#define ParallelPortIT          0x4     /* IRQ2*/
#define ExpSlotIRQ3             0x8     /* IRQ3*/
#define ExpSlotIRQ4             0x10    /* IRQ4*/
#define ExpSlotIRQ5             0x20    /* IRQ5*/
#define ExpSlotIRQ6             0x40    /* IRQ6*/
#define Panic                   0x80    /* Panic*/

/*****************************************************************************/

#define SVCMode                 0x13    /* stay in Supervisor mode*/

#define BIT31                   0x80000000
#define BIT30                   0x40000000

/*---------------------------------------------------------------------------*/

#endif /* __cdemon_driver_h */

/*---------------------------------------------------------------------------*/
/*> EOF driver.h <*/
