        TTL     VY86PID architecture specific description file     > driver_h.s
	; "@(#)driver_h.s	1.4	96/06/07 VLSI"
        ; ---------------------------------------------------------------------
        ; This assembler header file describes the memory and I/O
        ; worlds for the VLSI ARM600 VY86PID (Platform Independant
        ; Development) board.
        ;
        ; $Revision: 0.1 $
        ;   $Author: jsmith $
        ;     $Date: 941030 $
        ;
        ; Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
        ; ---------------------------------------------------------------------

        [   ((:LNOT: :DEF: VY86PID) :LOR: (:LNOT: VY86PID))
        ASSERT  (1 = 0) ; VY86PID architecture description included in non-VY86PID build
        ]

ARMVSN  EQU     0x600

        GBLS    BOARD
BOARD   SETS    "VY86CPID"

NUMIRQS EQU 8
NUMFIQS EQU 8

        ; ---------------------------------------------------------------------

AreaSize                EQU     0x01000000
RAMBase                 EQU     0x00000000
IOBase                  EQU     0x02000000
ROMBase                 EQU     0x03000000
ExpBase                 EQU     0x01000000
SectionsPerArea         EQU     AreaSize / 0x100000

WaitValue   EQU     20  ; 0.20 seconds = (20 x 10)mS

	[ (PID3_REV2)
SER_IOBASE	EQU	IOBase
ETHER_IOBASE	EQU	(IOBase + 0x40020)
	|
SER_IOBASE	EQU	IOBase
ETHER_IOBASE	EQU	(IOBase + 0x40)
	]

        ; ---------------------------------------------------------------------

;following are for VY86PID
;
IRQS            EQU     0x00    ;IRQ Status register
FIQS            EQU     0x08    ;FIQ Status
IRQM            EQU     0x04
FIQM            EQU     0x0C
;

        ; The following should be in the PID600 MMU description file:

; MMU constants (These should be in a separate MMU header file)

MMUCP                   CP      15

MMUIDReg                CN      0
MMUControlReg           CN      1
MMUPageTableBaseReg     CN      2
MMUDomainAccessReg      CN      3
MMUFlushTLBReg          CN      5
MMUPurgeTLBReg          CN      6
MMUFlushIDCReg          CN      7

PageTable1Size          EQU     4 * 4 * 1024 ; one per megabyte

MMU                     EQU     1 ; 0 is off, 1 is on
AlignFault              EQU     0 ; 0 is off, 2 is on
Cache                   EQU     4 ; 0 is off, 4 is on
WriteBuffer             EQU     8 ; 0 is off, 8 is on
Config32Bit             EQU     &10 + &20 ; mmu, idc, wb off, 32 bit prog/data, little endian
EnableMMU32             EQU     Config32Bit + MMU + AlignFault + Cache + WriteBuffer

        ; INTC and UART registers:
IRQRST                  EQU     0x00
UART_IER                EQU     0x24
UART_MCR                EQU     0x30

        ; Bitmasks and values for INTC and UART registers:
Panic                   EQU     0x80
EnRxDataAvailInt        EQU     0x01
EnRxLineStatusInt       EQU     0x04
MCRRTS                  EQU     0x02

; e91c92 resgisters
EBANKSEL	EQU	0x1c	; The Bank Select Register (Read/Write)
EIST		EQU	0x18	; Interrupt Status Register (Read Only)
EACK		EQU	0x18	; Interrupt Acknowledge Register (Write)
ERCV_INTM	EQU	0x0100	; Receive Interrupt Mask

        ; ---------------------------------------------------------------------
        ; On the VY86PID there are 8 possible IRQ sources:
        MACRO
$label  IRQVECTORS
$label  ; Declare interrupt handlers:
        IMPORT  SerialINT
        IMPORT  TimerINT
        IMPORT  PanicINT
	[ (SMC91C92)
	IMPORT  ENET_ISR
	]
        ;       handler         ; vecnum    bitmask     description
        DCD     SerialINT       ; 0x20      0x01        Serial Port 1 interrupt
        DCD     TimerINT        ; 0x21      0x02        Timer interrupt
        DCD     IgnoreIRQ       ; 0x22      0x04        Parallel Port interrupt
	[ (SMC91C92)
        DCD     ENET_ISR        ; 0x23      0x08        IRQ3 (Ethernet)
	|
        DCD     IgnoreIRQ       ; 0x23      0x08        IRQ3 (Ethernet)
	]
        DCD     IgnoreIRQ       ; 0x24      0x10        IRQ4 (Exp. Slot IRQ4)
        DCD     IgnoreIRQ       ; 0x25      0x20        IRQ5 (Exp. Slot IRQ5)
        DCD     IgnoreIRQ       ; 0x26      0x40        IRQ6 (Exp. Slot IRQ6)
        DCD     PanicINT        ; 0x27      0x80        PANIC (NMI)
        MEND

        ; ---------------------------------------------------------------------

        MACRO
$label  FIQVECTORS
$label  ;       handler     ; vecnum    bitmask     description
        DCD     IgnoreFIQ   ; 0x10      0x01        Serial Port 1 Transmit DMA Request
        DCD     IgnoreFIQ   ; 0x11      0x02        Serial Port 1 Receive DMA Request
        DCD     IgnoreFIQ   ; 0x12      0x04        DRQ4 Expansion Slot
        DCD     IgnoreFIQ   ; 0x13      0x08        NMI Logic Analyzer Port
        DCD     IgnoreFIQ   ; 0x14      0x10        DMA done ch 0
        DCD     IgnoreFIQ   ; 0x15      0x20        DMA done ch 1
        DCD     IgnoreFIQ   ; 0x16      0x40        DMA done ch 2
        DCD     IgnoreFIQ   ; 0x17      0x80        DMA done ch 3
        MEND

        ; ---------------------------------------------------------------------

        MACRO
$label  stopCTRLC   $io,$vv
$label  ; Ensure that the reception of a user-interrupt event does not
        ; interfere with the RDP communications that MAY be performed
        ; by and of the C-Demon handlers:
        ;MOV     $io,#SER_IOBASE
        LDR     $io,=SER_IOBASE
        LDRB    $vv,[$io,#UART_IER]
        BIC     $vv,$vv,#(EnRxDataAvailInt :OR: EnRxLineStatusInt)
        STRB    $vv,[$io,#UART_IER]

	[ (SMC91C92)
	; disable e91c92
	mov	$vv, #0x02		; Prepare for Bank 2
	str	$vv, [$io, #EBANKSEL]	; Change to Bank 2
	ldr	$vv, [$io, #EIST]	; Read the Interrupt Status Register
	bic	$vv, $vv, #ERCV_INTM
	str	$vv, [$io, #EACK]
	]
 
        MEND

        MACRO
$label  allowCTRLC  $io,$vv
$label  ; Enable the Rx interrupt:
        ;MOV     $io,#SER_IOBASE
        LDR     $io,=SER_IOBASE
        LDRB    $vv,[$io,#UART_IER]
        ORR     $vv,$vv,#(EnRxDataAvailInt :OR: EnRxLineStatusInt)
        STRB    $vv,[$io,#UART_IER]
        ;
        ; Similarly we need to ensure that the host can transmit
        ; characters:
        LDRB    $vv,[$io,#UART_MCR]   ; get current MCR status
        ORR     $vv,$vv,#MCRRTS       ; set RTS flag
        STRB    $vv,[$io,#UART_MCR]   ; raise RTS (allowing armsd to transmit)

	[ (SMC91C92)
	; enable e91c92
	mov	$vv, #0x02		; Prepare for Bank 2
	str	$vv, [$io, #EBANKSEL]	; Change to Bank 2
	ldr	$vv, [$io, #EIST]	; Read the Interrupt Status Register
	orr	$vv, $vv, #ERCV_INTM
	str	$vv, [$io, #EACK]
	]

        MEND

        MACRO
$label  dropRTS $io,$vv
$label
        ;MOV     $io,#SER_IOBASE           ; reference the UART hardware
        LDR     $io,=SER_IOBASE           ; reference the UART hardware
        LDRB    $vv,[$io,#UART_MCR]   ; get current state
        BIC     $vv,$vv,#MCRRTS       ; clear RTS flag
        STRB    $vv,[$io,#UART_MCR]   ; drop RTS
        MEND

        ; ---------------------------------------------------------------------
        ; IRQ and FIQ support macros:

        MACRO
$label  getIRQsource    $src,$tmp
$label  ; Load the currently active IRQ sources into $src:
        MOV     $tmp,#IOBase            ; get I/O hardware address
        LDRB    $src,[$tmp,#IRQS]       ; get the active sources
        MEND

        MACRO
$label  decodeIRQ       $vec,$w1
$label  ; in: $vec contains the currently active IRQ sources
        ; This macro decodes the active IRQ source bitmask into a bit
        ; position value:
        MOV     $w1,$vec,LSL #(32 - NUMIRQS)
        MOV     $vec,#NUMIRQS       ; start bit number
01      SUB     $vec,$vec,#1        ; decrement bit position
        MOVS    $w1,$w1,ASL #1      ; shift top bit into C
        BCC     %BT01               ; and loop around
        ; exit: $vec should be the vector table slot for the active
        ; IRQ bit. We need to check that the vector number is
        ; LESS than NUMIRQS, otherwise we could end up branching
        ; through a slot outside the table.
        MEND

        MACRO
$label  getFIQsource    $src,$tmp
$label
        MOV     $tmp,#IOBase            ; reference I/O hardware
        LDRB    $src,[$tmp,#FIQS]       ; get the active sources
        MEND

        MACRO
$label  decodeFIQ       $vec,$w1,$w2
$label  ; We are only interested in the lo8bits:
        MOV     $w1,$vec,LSL #(32 - NUMFIQS)
        MOV     $vec,#NUMFIQS       ; start bit number
01      SUB     $vec,$vec,#1        ; decrement bit position
        MOVS    $w1,$w1,ASL #1      ; shift top bit into C
        BCC     %BT01               ; and loop around
        MEND

        ; This macro is just a scheme a method of entering some target
        ; architecture specific interrupt support code into the
        ; "handler.s" source:
        MACRO
$label  PRIVATEINTERRUPTS
$label  ; This macro should only be instantiated once!
        EXPORT  PanicINT
PanicINT
        ; VY86PID breakpoint generator
        ; in:   interrupt mode; IRQ and FIQ disabled as required;
        ;       r13 = FD stack
        ;       r14 = (return address + 4)
        STMFD   sp!,{r0-r7,lr}          ; create some work registers
        ;
        MOV     r0,#IOBase              ; reference the I/O hardware
        MOV     r1,#Panic               ; the required bit
        STRB    r1,[r0,#IRQRST]         ; clear the interrupt
        ;
        dropRTS r0,r1                   ; drop RTS to stop host transmitting
        ;
        ; The "BP_StopInt" call expects to be called with the
        ; following stack state: {r0-r7,lr}
        LDR     pc,=BP_StopInt                  ; simulate a breakpoint
        MEND

        ; --------------------------------------------------------------------- 
        MACRO
$label  STARTUPCODE
        ; Force the code to start executing from the hi-memory ROM
        ; image. This means that we can then safely map out the ROM
        ; from the ARM vector space. The following instruction forces
        ; the PC to be loaded from a literal pool. The value loaded is
        ; the address of the HiReset symbol suitably relocated. This
        ; means when we "armlink" the object created we can specify
        ; the ROM address, and the code will then be correct.
$label  LDR     pc,=gotoROM     ; update PC with address within ROM
gotoROM ; We are now executing from the real ROM location.
        MOV     v4,#0x00000000  ; PowerOnReset
        MOV     v5,#0x00000000  ; RAM size not yet known
        STR     v4,[v4,-v4]     ; ensure RAM mapped to zero on the PID
        ;
        LDR     a2,=IOBase      ; make sure interrupts are masked off
        MOV     a1,#0x00        ; the masks are active high
        STRB    a1,[a2,#IRQM]   ; ensure no IRQs enabled
        STRB    a1,[a2,#FIQM]   ; ensure no FIQs enabled
        MEND

        ; --------------------------------------------------------------------- 

        MACRO
$label  INITMMU $tmp
$label  ; Setup the ARM6x0 MMU with a simple flat page table:
        MCR     MMUCP,0,$tmp,MMUFlushTLBReg,c0  ; Flush the TLB
        MCR     MMUCP,0,$tmp,MMUFlushIDCReg,c0  ; Flush the cache
        LDR     $tmp,=0xFFFFFFFF
        MCR     MMUCP,0,$tmp,MMUDomainAccessReg,c0 ; set domains
        ; The level 1 page table can be found on the next 16K boundary
        ; after the end of the ROM code:
        LDR     $tmp,=ROMEnd            ; reference holding variable
        LDR     $tmp,[$tmp,#0x00]       ; load address of end of ROM code
        BIC     $tmp,$tmp,#0x3F00
        BIC     $tmp,$tmp,#0x00FF       ; mask off at 16K boundary
        ADD     $tmp,$tmp,#0x4000       ; bump up to next 16K border
        ; Reference the page table, and initialise the MMU:
        MCR     MMUCP,0,$tmp,MMUPageTableBaseReg,c0 ; Level 1 page table
        MOV     $tmp,#EnableMMU32                   ; Default MMU configuration
        MCR     MMUCP,0,$tmp,MMUControlReg,c0       ; Ensure MMU enabled
        MEND

        ; --------------------------------------------------------------------- 

        MACRO
$label  INITTIMER $w1,$w2
$label  ; No special timer initialisation needed for the VY86PID.
        MEND

        ; --------------------------------------------------------------------- 
        END     ; EOF driver_h.s
