/*
 * C-Demon SMC9192 ethernet header
 *
 * Copyright (c) 1995, VLSI Technology Inc. All Rights Reserved
 *
 * "@(#)e9192.h	1.2	96/06/07 VLSI"
 */

#ifdef VY86PID

/*
 * I R Q   S t a t u s   a n d   E n a b l e   B i t s  (IRQS and IRQM)
 */
#define SerialPortIT	0x1	/* IRQ0*/
#define TimerIT		0x2	/* IRQ1*/
#define ParallelPortIT	0x4	/* IRQ2*/
#define EtherIT		0x8	/* IRQ3*/
#define ExpSlotIRQ4	0x10	/* IRQ4*/
#define ExpSlotIRQ5	0x20	/* IRQ5*/
#define ExpSlotIRQ6	0x40	/* IRQ6*/
#define Panic		0x80	/* Panic*/

/*
 * The SMC 92C9x Register Definitions
 */

#define BankSel		0x1c	/* The Bank Select Register (Read/Write) */
#define Bank0		0x00
#define Bank1		0x01
#define Bank2		0x02
#define Bank3		0x03

/*
 * Register Bank #0 Definitions
 */

#define TCR	0x00		/* Transmit Control Register (Read/Write) */
#define EPHSR	0x04		/* EPH Status Register (Read Only) */
#define RCR	0x08		/* Receive Control Register (Read/Write) */
#define ECR	0x0c		/* Counter Register (Read Only) */
#define MIR	0x10		/* Memory Information Register (Read Only) */
#define EMCR	0x14		/* Memory Configuration Register (Read/Write) */

/*
 * Register Bank #1 Definitions
 */

#define CR	0x00		/* Configuration Register (Read/Write) */
#define BAR	0x04 		/* Base Address Register (Read/Write) */
#define IAR1	0x08 		/* Hi Individual Address Register #1 (R/W) */
#define IAR2	0x09 		/* Lo Individual Address Register #2 (R/W) */
#define IAR3	0x0c 		/* Hi Individual Address Register #3 (R/W) */
#define IAR4	0x0d 		/* Lo Individual Address Register #4 (R/W) */
#define IAR5	0x10		/* Hi Individual Address Register #5 (R/W) */
#define IAR6	0x11		/* Lo Individual Address Register #6 (R/W) */
#define GPR	0x14 		/* General Purpose Register (Read/Write) */
#define CTR	0x18 		/* Control Register (Read/Write) */

/*
 * Register Bank #2 Definitions
 * NOTE: The Interrupt Mask Register occupies the upper byte of the
 *	halfword read from or written to the IST or ACK Registers
 */

#define MMUCR	0x00		/* MMU Command Register (Read/Write) */
#define PNR	0x04		/* Lo Packet Number Register (Read/Write) */
#define ARR	0x05		/* Hi Allocation Result Register (Read Only) */
#define FIFO	0x08		/* FIFO Ports Register (Read Only) */
#define PTR	0x0c		/* Pointer Register (Read/Write) */
#define DATAR	0x10		/* Data Register (Read/Write) */
#define IST	0x18		/* Interrupt Status Register (Read Only) */
#define ACK	0x18		/* Interrupt Acknowledge Register (Write) */
#define IMSK	0x19		/* Interrupt mask register, R/W */

/*
 * Register Bank #3 Definitions
 */

#define MT0	0x00		/* Lo MultiCast Table #0 Register (R/W) */
#define MT1	0x01		/* Hi MultiCast Table #1 Register (R/W) */
#define MT2	0x04		/* Lo MultiCast Table #2 Register (R/W) */
#define MT3	0x05		/* Hi MultiCast Table #3 Register (R/W) */
#define MT4	0x08		/* Lo MultiCast Table #4 Register (R/W) */
#define MT5	0x09		/* Hi MultiCast Table #5 Register (R/W) */
#define MT6	0x0c		/* Lo MultiCast Table #6 Register (R/W) */
#define MT7	0x0d		/* Hi MultiCast Table #7 Register (R/W) */

/*
 * Bit Masks for the Transmit Control Register (Bank #0/TCR)
 */

#define PAD_EN	0x0080		/* Enable padding to 64 bytes */
#define TXENA	0x0001		/* Transmit Enable */

/*
 * Bit Masks for the Receive Control Register (Bank #0/RCR)
 */

#define EPH_RST		0x8000	/* Software Reset */
#define STRIP_CRC	0x0200	/* Strip Incoming CRCs    */
#define RXEN		0x0100	/* Enables the Receiver */
#define PRMS		0x0002	/* Promisuous mode */
#define ALMUL		0x0004	/* Enables Multicast Receive  */

/*
 * Bit Masks for the Configuration Register (Bank #1/CR)
 */

#define NO_WAIT_ST	0x1000	/* High = No Additional Wait States */
#define AUI_SELECT	0x0100	/* High = AUI/Coax, Low = 10BaseT */
#define Bit_16		0x0080	/* High = 16 Bit System Bus */
#define DIS_LINK	0x0040	/* High = Disable Link Test */
#define RAM_SIZE	0x0020	/* High = 32Kx8, Low = 8Kx8 */
#define AL_ONE		0x0010	/* Always a one */

/*
 * Bits for the Control Register (Bank #1/CTR)
 */

#define AUTO_RELE	0x0800	/* auto release tx memory */
#define TE_ENABLE	0x0020	/* High = Enable Transmit Error Interrupt */
#define EPROM_SEL	0x0004	/* Selects EEProm for Read/Write */
#define EPROM_LOAD	0x0002	/* High = Read EEProm */
#define EPROM_STORE	0x0001	/* High = Write EEProm */

/*
 * Bits for the MMU Control Register (Bank #2/MMUCR)
 */

#define MMU_BUSY	0x0001	/* MMU Busy */
#define MMU_ALLOC	0x0020	/* Allocate  TX Memory */
#define MMU_RESET	0x0040	/* MMU Reset Command */
#define MMU_REMV	0x0060	/* Remove Frame from Top of RX FIFO */
#define MMU_RMVRLS	0x0080	/* Remove & Release Frame */
#define MMU_RLSFRM	0x00A0	/* Release Specific Frame */
#define MMU_ENQUE	0x00C0	/* Enqueue a Frmae to TX FIFO */
#define MMU_TXRESET	0x00E0	/* Reset both TX FIFOs */


/*
 * Bits for the Interrupt and Mask Register (Bank #2/MSK)
 */

#define RCV_INT		0x01	/* Receive Interrupt */
#define TX_INT		0x02	/* Transmit Interrupt */
#define TX_EMPTY	0x04	/* Transmit Empty Interrupt */
#define ALOC_INT	0x08	/* Allocation Interrupt */
#define RCV_OVR		0x10	/* Receive Overrun Interrupt */
#define EPH_INT		0x20	/* EPH Interrupt */

#define RCV_INTM	0x0100	/* Receive Interrupt Mask */
#define TX_INTM		0x0200	/* Transmit Interrupt Mask */
#define TX_EMPTYM	0x0400	/* Transmit Empty Interrupt Mask */
#define ALOC_INTM	0x0800	/* Allocation Interrupt Mask */
#define RCV_OVRM	0x1000	/* Receive Overrun Interrupt Mask */
#define EPH_INTM	0x2000 	/* EPH Interrupt Mask */


/*
 * Bits for the Receive Frame Status Word
 */

#define ALGN_ERR	0x8000	/* Frame had Alignment Error */
#define BROD_CAST	0x4000	/* Frame was Broadcast */
#define BAD_CRC		0x2000	/* Frame had Bad CRC */
#define ODD_FRM		0x1000	/* Rec'd Frame had Odd Number of Bytes */
#define TOO_LNG		0x0800	/* Rec'd Frame longer than 1518 bytes */
#define TOO_SHORT	0x0400	/* Rec'd Frame shorter than 64 bytes */

#endif	/* VY86PID */
