	TTL     C-Demon ethernet ISR support

;******************************************************************************
;*                                                                            *
;*   PixelPress ROM Operating System Version 1.0                              *
;*   Ethernet 802.2/802.3 Driver for SMC 91C9X Controller                     *
;*   Copyright (C) 1994 - ACME Mining & Software Development, Inc.            *
;*   Copyright (C) 1994 - David C. Prue, CCP  All rights reserved worldwide.  *
;*                                                                            *
;*             ACME Mining & Software Development, Inc.                       *
;*             Glen Burnie, Maryland 21061  USA                               *
;*             Voice:  800-318-5796   (From U. S. Only)                       *
;*                     410-768-8978   (International)                         *
;*             Fax:    410-768-8153                                           *
;*                                                                            *
;*   Use, duplication, or disclosure of this software by the U. S. Government *
;*   is subject to the restricted rights applicable to computer software      *
;*   under FAR 52.227-19 and DFARS 252.227-7013.                              *
;*                                                                            *
;******************************************************************************
; needs to be made APCS compliant and integrated into the DEMON environment
;

	;
	; 8/1/95, integrated into the C-DEMON
	;	-spock
	;
	; Copyright (c) 1995, VLSI Technology Inc. All Rights Reserved.
	;
	; "@(#)e9192.s	1.5	96/06/07 VLSI"
	;
	INCLUDE	global_h.s
	INCLUDE $DRIVER/driver_h.s
	INCLUDE e9192_h.s

	EXPORT	ENET_Init
	EXPORT	ENET_ISR
	EXPORT	ENET_TX
	EXPORT	rx_packet
	EXPORT	enet_disable
	EXPORT	enet_enable

	IMPORT	BP_StopInt

	IMPORT	Our_Addr
	IMPORT	Dest_Addr 
	IMPORT	Src_Addr

        KEEP

	AREA |C-Demon$$ENET|, CODE, READONLY

;********************************************************************
;
;   Initialize the SMC91C9x Controller
;
;	void ENET_Init(void)0x20
;
;********************************************************************
ENET_Init
	stmdb	sp!, {a1-a4, v3, lr}       ; Save Registers

;   In discussions with SMC, it turns out that the chip may not be ready yet
;   if we just powered up.  One of the indicators of this situation is
;   an attempt to read any register gives you a 0x03.
;   In order to detect this situation, we chose to write a harmless value
;   into the TCR Register, and try to read it back.  It is important to note
;   that this register is in Bank 0; remember, we can't count on the Bank
;   Select working either until the chip comes up fully!  Since Bank 0 is
;   the power up default, we should hit TCR even if the chip comes ready
;   after the bank select but before the write.

	ldr	v3, =ETHER_IOBASE 		; IO Base for the SMC Chip
01
	mov	a1, #Bank0		; Select Bank 0
	str     a1, [v3, #BankSel]	; Do it

	mov	a1, #0			; Get a Harmless Value
	str	a1, [v3, #TCR] 		; Write it out
	ldr	a1, [v3, #TCR]		; Read it Back
	movs	a1, a1 			; Set the Flags
	bne	%B01			; Loop until Ready


	;   Now do a Software Reset and configure the beast!

	mov	a1, #EPH_RST		; Software Reset
	str	a1, [v3, #RCR]		; Assert it
	mov	a1, #0 			; Now Clear
	str	a1, [v3, #RCR]       	; De-Assert it

	;
	; The available memory is set in MIR and is preset to 0x1212
	; We reserve 256 bytes for transmit in case that incoming
	; packets used up all the memory.
	;
	mov	a1, #0x01		; 1 x 256 bytes
	str	a1, [v3, #EMCR]

	mov	a1, #PAD_EN | TXENA	; Pad to 64 Bytes & Enable Transmit
	str	a1, [v3, #TCR]       	; Do it
    
	mov	a1, #STRIP_CRC | RXEN	; Strip Incoming CRCs & Enable Receive
;	orr     a1, a1, #PRMS		; enable promicuous mode for test
	str	a1, [v3, #RCR]		; Do it

	mov	a1, #Bank1		; Request Bank 1
	str	a1, [v3, #BankSel]	; Select it
    
;    mov     a1, #NO_WAIT_ST+AUI_SELECT	; No Wait States, Non-10BaseT
	mov	a1, #NO_WAIT_ST		; No Wait States, 10BaseT
	orr	a1, a1, #Bit_16		; 16 Bit I/O
	orr	a1, a1, #RAM_SIZE+AL_ONE	; 32K x 16 RAM, Always One
	str	a1, [v3, #CR]		; Do it

	;
	; Set transmit auto release.
	;
	;mov	a1, #TE_ENABLE+AUTO_RELE ; Transmit Error Interrupt Enable
	mov	a1, #AUTO_RELE
	str	a1, [v3, #CTR]		; Set it

	;   Now we must write our Ethernet Address into the chip.  
	;   We also need to place the address into any packets we send out.
	;   Our_Addr    DCB     M,Y,0,A,D,D   typical 
	;   The byte order for IAR should be
	;	Bits 15:8	Bits 7:0
	;	1		0
	;	3		2
	;	5		4

	mov     a2, #IAR1              	; Beginning of the Address Registers
	ldr     a3, =Our_Addr		; Ram Location to keep it
	mov	a4, #3			; 16 bit words in the Address
02	ldrb	a1, [a3], #1		; Get a Byte of our add (low)
	ldrb	v1, [a3], #1		; Get a Byte of our add (high)
	orr	a1, a1, v1, LSL #8	; combine into word (high,low)
	str	a1, [v3, r1]		; Place word, Increment Memory Pointer
	add	a2, a2, #4		; Bump I/O Port Pointer
	subs	a4, a4, #1		; Decrement Counter
	bne	%B02			; Loop until done...

	mov	a1, #Bank2		; Bank 2 is Next
	str	a1, [v3, #BankSel]	; Select it

03	ldr	a1, [v3, #MMUCR]	; Read the MMU Control Register
	ands	a1, a1, #MMU_BUSY	; Is it Busy?
	bne     %B03			; Loop til no

	mov	a1, #MMU_RESET		; Get a Reset Command
	str	a1, [v3, #MMUCR]	; Reset the MMU

04	ldr	a1, [v3, #MMUCR]	; Get the Status
	ands	a1, a1, #MMU_BUSY	; Wait for not busy
	bne     %B04

	ldr	a1, [v3, #IST]		; Read the Interrupt Status Register
	and	a1, a1, #0xFF		; Clear the Mask Portion
;	orr	a1, a1, #TX_INTM+RCV_INTM+EPH_INTM ; Interrupts to Enable
	orr	a1, a1, #RCV_INTM	; Interrupts to Enable
	str	a1, [v3, #ACK]		; Issue it

	; enable bits in the IRQM register
	mov	a1,#(TimerIT+EtherIT)	;enable timer and irq3
	ldr	v3, =IOBase
	str	a1,[v3,#IRQM]		;doit
 
	ldmia   sp!, {a1-a4, v3, pc}	; Restore Registers & Return


;********************************************************************
;
;   Interrupt Service Routine
;
;********************************************************************

;   This routine is called from the main ISR module in the OS.  It could
;   just as easily be called directly from the IRQ Vector in low RAM. 
;   Note that since this routine is executed at IRQ time, and a foreground
;   task may have called the Transmit or Receive routines, we must save
;   the state of the chip, and set it back before we return to foreground.
; if Geary's C-Demon then save r0-r4 (used by irq handler) retur as if a bl
; user r0-r12 will be restored by handler and irq return 
; if Jamie - save all used registers and return with subs pc, lr, #4

ENET_ISR
	stmdb	sp!,{a1-a4,v1-v4,lr}	; Save the lower Registers

	ldr	a1, =ETHER_IOBASE		; a1 -> IO Base Address

	ldr	v2, [a1, #BankSel]	; v2 = Foreground Bank Register
	mov	v1, #Bank2              ; Prepare for Bank 2
	str	v1, [a1, #BankSel]	; Change to Bank 2
	ldr	v3, [a1, #PTR]		; v3 = Foreground Pointer Register
	ldr	v4, [a1, #PNR]		; v4 = Foreground Packet Number Register
	ldr	v1, [a1, #IST]		; Get the Interrupt Status
	and	v1, v1, v1, lsr #8	; Mask only enabled interrupts

	tst	v1, #(RCV_INT)		; Got a Receive Interrupt?

	;
	; a1 is the input parameter to rx_packet
	;
	blne	rx_packet		; If so, handle it

	;
	; The tx interrupt goes off only when tx has been sent out
	; to network.
	;
	;tst     v1, #(TX_INT)		; Got a Transmit Interrupt?
	;blne    TX_ISR			; If so, handle it

	mov	v1, #Bank2              ; Prepare for Bank 2
	str	v1, [a1, #BankSel]	; Change to Bank 2
	str	v4, [a1, #PNR]		; Replace the Foreground Pkt Number
	str	v3, [a1, #PTR]		; Replace the Foreground Pointer
	str	v2, [a1, #BankSel]	; Replace the Foreground Bank Register

	;
	; If rxflag set, this packet is for us.
	; Return to C-Demon control, otherwise return to user.
	;
	ldr	a2, =rxflag
	ldr	a3, [a2]
	movs	a3, a3
	beq	%F01

	ldr	pc,=BP_StopInt

01
	ldmia	sp!,{a1-a4,v1-v4,lr}	; & Return to main ISR handler
	subs	pc, lr, #4	; returning from ISR is 2 instructions ahead


;********************************************************************
;
;   Transmit Packet Interrupt Service Routine
;
;********************************************************************

;	a1 -> IO Base
;	Must Delay 600ns between write to PTR Register &
;		Read from or Write to DATA Register.
;	Must Delay 300ns between DATA Read/Write & DATA Read/Write.
;
;TX_ISR
;	stmdb	sp!, {a1,v1,v2,lr}		; Save Return Address
;	mov	v1, #Bank2              ; Prepare for Bank 2
;	str	v1, [a1, #BankSel]	; Change to Bank 2
;	ldr	v1, [a1, #FIFO]	; Get the Packet Number
;	and	v1, v1, #0x1F	; Mask it out
;	str	v1, [a1, #PNR]	; Set up the TX FIFO
;    
;	ldr	v1, =0x6000 	; Start the Pointer at 0, Auto Increment
;	str	v1, [a1, #PTR]	; Set it
;	bl	delay600		; Wait 600ns
;	ldr	v1, [a1, #DATAR]	; Get the Status Word from the Buffer
;    
;01	ldr	v2, [a1, #MMUCR]	; Get the MMU Status
;	ands	v2, v2, #MMU_BUSY	; Is it Busy?
;	bne	%B01 		; wait 'til free
;
;	mov	v2, #MMU_RLSFRM	; Release a Specific Packet
;	str	v2, [a1, #MMUCR]	; Release it
;
;02	ldr	v2, [a1, #MMUCR]	; Get the MMU Status
;	ands	v2, v2, #MMU_BUSY	; Is it Busy?
;	bne	%B02		; wait 'til free
;    
;	ldr	v2, [a1, #IST]	; Get the Interrupt Status Register
;	and	v2, v2, #0xFF00	; Don't want to change the Enable Mask
;	orr	v2, v2, #(TX_INT)	; Acknowledge the TX Interrupt
;	str	v2, [a1, #ACK]	; Assert it
;
;	ldmia	sp!,{a1,v1,v2,pc}	; & Return

    
;********************************************************************
;
;   Receive Packet Interrupt Service Routine
;
;********************************************************************
;	a1 contains the IO base address.
;
;	Must Delay 600ns between write to PTR Register &
;		Read from or Write to DATA Register.
;	Must Delay 300ns between DATA Read/Write & DATA Read/Write.

rx_packet
	stmdb	sp!, {a1-a4,v1-v4,lr}		; Save the Return Address

	ldr	a2, =rxflag
	mov	a3, #0
	str	a3, [a2]

	ldr	a1, =ETHER_IOBASE		; a1 -> IO Base Address

	mov	v1, #Bank2              ; Prepare for Bank 2
	str	v1, [a1, #BankSel]	; Change to Bank 2

	;
	; disable interrupt.
	;
	ldr	v1, [a1, #IST]		; Read the Interrupt Status Register
	bic	v1, v1, #RCV_INTM
	str	v1, [a1, #ACK]		; Issue it

rx_more
	; Read FIFO port to clear RCV_INT bit
	ldr	v1, [a1, #FIFO]

	;
	; Keep receiving in the loop until all packets are
	; being serviced.
	;
	ldr	v1, =0xE000	; Pointer to RCV + Auto Inc. + READ + 0000
	str	v1, [a1, #PTR]	; Set the Pointer Register
	bl	delay600	; We must delay here
	ldr	v1, [a1, #DATAR]	; Get the Status Word
	tst	v1, #BAD_CRC+TOO_LNG+TOO_SHORT+ALGN_ERR
	bne	done		; If Error, then Exit without saving
	ldr	v1, [a1, #DATAR]	; Get the byte count Word

	bic	v1,v1,#0xFF000000
	bic	v1,v1,#0xFF0000
	mov	v1,v1,LSR #1	; now have 16 bit word count in R4

	mov	v4, #3		; We will move 3 halfwords (6 bytes)
	ldr	v3, =Dest_Addr	; local Dest Address Storage
11	ldr	v2, [a1, #DATAR]	; Get a Halfword
	strb	v2, [v3], #1	; Put lsb in RAM
	mov	v2, v2, lsr #8 	; Get other byte
	strb	v2, [v3], #1	; Put MSB in RAM
	subs	v4, v4, #1		; Decrement Count
	bne	%B11 		; Loop til done
	sub	v1,v1, #3	; word dec by 3

	mov	v4, #3		; We will move 3 halfwords (6 bytes)
	ldr	v3, =Src_Addr	; Source Address Storage
01	ldr	v2, [a1, #DATAR]	; Get a Halfword
	strb	v2, [v3], #1	; Put lsb in RAM
	mov	v2, v2, lsr #8 	; Get other byte
	strb	v2, [v3], #1	; Put MSB in RAM
	subs	v4, v4, #1	; Decrement Count
	bne	%B01 		; Loop til done
	sub	v1,v1, #3	; word dec by 3

	ldr	v1, [a1, #DATAR]	; Get the Length in R4

	mov	v3, v1, lsl #24	; Swap lower halfword hi & low bytes
	mov	v1, v1, lsl #16
	orr	v1, v3, v1, lsr #8
	mov	v1, v1, lsr #16	; R4 now contains the packet length
	cmp	v1, #0x600	; If >= 0x600 it is Bluebook Ethernet
	bge	done		; If it is, then ignore it (NOT)

	; Ignore the packet if no data
	;tst	v1, #0x0a00
	;beq	done

	; For the next few compares, we do a logical shift left 16 bits, so that
	; if any random bits are present in the upper halfword, they aren't
	; used in the compares.

	;
	; For our network, we use LLC1 header to connect to the host machine.
	; Th LLC U/I has 3 bytes, 0xAAAA03. -spock
	;

	ldr	v2, [a1, #DATAR]	; Get the SAP Bytes
	ldr	v3, =0xAAAA0000	; Make sure SAP indicates a SNAP is present
	cmp	v3, v2, lsl #16 	; Does it?
	bne	done                    ; If not, then Exit

	;
	; Identify if this frame uses our protocol!
	; The protocol ID should be 5 bytes after the frame control byte.
	;	030077aa7777
	;
	ldr	v2, [a1, #DATAR]	; Get Frame Type & 1st byte of SNAP
	ldr	v2, [a1, #DATAR]	; Get the Next 2 bytes of SNAP
	ldr	v3, =0x77aa0000         ; OUI for us??
	cmp	v3, v2, lsl #16         ; Compare it
	bne	done                    ; If not, then Exit

	ldr	v2, [a1, #DATAR]	; Get the Last two bytes of SNAP
	ldr	v3, =0x77770000		; Ether type for us ??
	cmp	v3, v2, lsl #16		; Is it?
	bne	done			; If not, then Exit

	;
	; v1 contains the packet length from the network.
	;
	sub	v1, v1, #10     ; Adjust Length to Copy

	;
	; Adjust receive buffer to the starting position.
	;
	ldr	v2, =RecvBuf
	ldr	v4, =rdp_in
	ldr	v3, [v4]
	add	v2, v2, v3
	bl      blast                   ; Copy it to ram (extra)

	;
	; Save packet size + rdp_in to rdp_in.
	;
	add	v3, v3, v1
	str	v3, [v4]

	;
	; Indicate a packet is received.
	;
	ldr	a2, =rxflag
	mov	a3, #1
	str	a3, [a2]

done
	ldr	v2, [a1, #MMUCR]  	; Get MMU Control Register
	ands	v2, v2, #MMU_BUSY	; Wait for Not Busy
	bne	done

	mov	v1, #MMU_RMVRLS		; Release Packet Command
	str	v1, [a1, #MMUCR]	; Do it

04	ldr	v2, [a1, #MMUCR]	; Get MMU Control Register
	ands	v2, v2, #MMU_BUSY	; Wait for Not Busy
	bne	%B04
05
	ldr	v2, [a1, #IST]		; Get the Interrupt Status Register
	; this might hang ourself!
	;tst	v2, #RCV_INT		; more packets to receive?
	;bne	rx_more
	orr	v2, v2, #(RCV_INTM)	; Enable the RX Interrupt
	str	v2, [a1, #ACK]		; Assert it

	ldmia    sp!,{a1-a4,v1-v4,pc}	; and back to main ENET_ISR.....


;********************************************************************
;
;   Transmit a Packet
;
;   Enters with a1 -> Buffer, a2 = Length, a3 -> Destination Address
;
;********************************************************************

ENET_TX
	stmdb	sp!,{a1-a4,v3,v4,v6,r14}

	ldr	v3,=ETHER_IOBASE
	;
	; The AUTO_RELE is on.  The memory will be released after
	; the packet has been transmitted.
	; We save an ISR for TX_EMPTY.  We do check it here and
	; acknowledge it.
	;
	mov	v4, #Bank2              ; Prepare for Bank 2
	str	v4, [v3, #BankSel]	; Change to Bank 2
	ldr	v4, [v3, #IST]	; Get the Interrupt Status Register
	tst	v4, #(TX_EMPTY)
	beq	tx_ok

	orr	v4, v4, #(TX_EMPTY)	; Acknowledge the TX Interrupt
	str	v4, [v3, #ACK]	; Assert it

	;
	; Last tx is done.
	;
	;
	; We should check if MMU is available?
	;
tx_ok
	mov	v4,a1
	mov	v6,a2

	mov	a1,#Bank0
	str	a1,[v3,#BankSel]

	mov	a1,#(PAD_EN | TXENA)	; Pad Enable - Transmit Enable
	str	a1,[v3,#TCR]
    
	mov	a1,#Bank2
	str	a1,[v3,#BankSel]
    
	add	a1,v6,#18		;6=dest add, 6=source add
					;2=tx status, 2=Length, 2=control
	mov	a1,a1 ,LSR #8		; get the length
	orr	a1,a1,#MMU_ALLOC	; 0x20 tx allocate
	str	a1,[v3,#MMUCR]		; Request a Buffer ((L+18)/256+1)*256

tx_mmu
	ldr	a1,[v3,#IST]		; Wait for MMU Completion
	ands	a1,a1,#0x8		; allocate done
	beq	tx_mmu

	ldr	a1,[v3,#ARR]		; Get the Error Flag
	mov	a1,a1 ,LSR #8
	and	a1,a1,#0xFF
	ands	a2,a1,#0x80			; Check it
	bne	tx_mmu			; If Failure, try again

	str	a1,[v3,#PNR]		; Set the TX FIFO

	mov	a1,#0x4000		; Set up the Buffer Pointer
	str	a1,[v3,#PTR]
	bl	delay600

	mov	a1,#0			; Write the Status Byte
	str	a1,[v3,#DATAR]
	bl	delay300

	add	a1,v6,#18 			; For Packet Info
	str	a1,[v3,#DATAR]

	mov	a2,#3			; Words in Address
tx_dest
	ldrb	a1,[a3],#1
	ldrb	a4,[a3],#1
	orr	a1,a1,a4 ,LSL #8
	str	a1,[v3,#DATAR]		; Write the Destination Address
	subs	a2,a2,#1
	bne	tx_dest

	;
	; Write the src address.
	;
	mov	a2,#3			; Words in Address
	ldr	a3, =Our_Addr
tx_src
	ldrb	a1,[a3],#1
	ldrb	a4,[a3],#1
	orr	a1,a1,a4 ,LSL #8
	str	a1,[v3,#DATAR]
	subs	a2,a2,#1
	bne	tx_src

	mov	a4,v6, LSR #1		; number of words
tx_data
	ldrb	a1,[v4],#1		; Get a Byte
	ldrb	a3,[v4],#1
	orr	a1,a1,a3 ,LSL #8
	str	a1,[v3,#DATAR]		; Write it
	subs	a4,a4,#1               	; Decrement Count
	bne	tx_data                  	; Loop 'til done

	tst	v6,#1			; is number of bytes odd?
	mov	a1,#0
	ldrneb	a1,[v4]               	; Get the Odd Byte
	orrne	a1,a1,#0x2000          	; Set the Flag if Odd Byte
	str	a1,[v3,#DATAR]  		; Write it

	mov	a1,#Bank0
	str	a1,[v3,#BankSel]

	mov	a1,#(PAD_EN | TXENA)	; 0x81- Transmit Enable
	str	a1,[v3,#TCR]
    
	mov	a1,#Bank2
	str	a1,[v3,#BankSel]

	mov	a1,#MMU_ENQUE		; 0x00c0 - Issue the Transmit
	str	a1,[v3,#MMUCR]

	ldmia	sp!,{a1-a4,v3,v4,v6,pc}

;******************************************************
;
;   BLAST
;
;   v2 -> Buffer, v1 = Count
;   Assumes that the Buffer is on a Word Boundry
;
;*******************************************************

blast
	stmdb	sp!, {a1,a2,v1-v4,lr}		; Save Return Address
	movs	v3, v1, lsr #2		; R2 = R4 / 4
	beq	%F02			; If < 4 Remaining, handle it

01	ldr	a2, [a1, #DATAR]		; Get two Bytes
	bic	a2, a2, #0xFF000000	; Clear Upper 8 Bits
	bic	a2, a2, #0x00FF0000	; Clear Next 8 Bits
	mov	v4, v4			; We have to wait 300ns between reads
	ldr	v4, [a1, #DATAR]		; Get next 2 bytes
	orr	v4, a2, v4, lsl #16	; Place the Upper 16 Bits
	str	v4, [v2], #4		; Put it in RAM & Bump Pointer
	subs	v3, v3, #1		; Decrement Count
	bne	%B01			; Loop til done

02	tst	v1, #2			; Remainder >= 2?
	beq	%F03			; If not, then handle it

	ldr	v4, [a1, #DATAR]		; Get the Next 2 Bytes
	strb	v4, [v2], #1		; Place 1st byte & bump pointer
	mov	v4, v4, lsr #8		; Position the next byte
	strb	v4, [v2], #1		; Place next byte & bump pointer

03	tst	v1, #1			; Do we have an odd number of bytes?
	ldrne	v4, [a1, #DATAR]		; Get the Last Byte
	strneb	v4, [v2], #1		; Place it & bump pointer

	ldmia	sp!, {a1,a2,v1-v4,pc}		; & Return

	;
	; Disable e91C92 interrupt
	;
enet_disable
	stmdb	sp!, {a1,v1,lr}		; Save Return Address
	ldr	a1, =ETHER_IOBASE
	mov	v1, #Bank2              ; Prepare for Bank 2
	str	v1, [a1, #BankSel]	; Change to Bank 2

	ldr	v1, [a1, #IST]		; Read the Interrupt Status Register
	bic	v1, v1, #RCV_INTM
	str	v1, [a1, #ACK]		; Issue it
	ldmia	sp!, {a1,v1,pc}		; & Return

	;
	; Enable e91C92 interrupt
	;
enet_enable
	stmdb	sp!, {a1,v1,lr}		; Save Return Address
	ldr	a1, =ETHER_IOBASE
	mov	v1, #Bank2              ; Prepare for Bank 2
	str	v1, [a1, #BankSel]	; Change to Bank 2

	ldr	v1, [a1, #IST]		; Read the Interrupt Status Register
	orr	v1, v1, #RCV_INTM
	str	v1, [a1, #ACK]		; Issue it
	ldmia	sp!, {a1,v1,pc}		; & Return

;****************************************************************************
;*
;*      Delay Subroutines (Delay includes the bl to here)
;*	On the PID3 local IO is 480ns and DRAM sequential is 83ns
;*	if not in the cache and 41ns if in the cache
;****************************************************************************

delay600
	mov	a1, a1	; 83 ns
	mov	a1, a1	; 83 ns
	mov	a1, a1	; 83 ns
	mov	a1, a1	; 83 ns
delay300
	mov	a1, a1	; 83 ns
	mov	pc,r14	; 83 ns


;
;; Routines to control SMC91C9x EtherNet Controller

; Data area
	AREA |C-Demon$$ENET_DATA|, DATA

	EXPORT	rdp_in
	EXPORT	rdp_opened
	EXPORT	rxflag

rdp_in		DCD	0
rdp_opened	DCD	0
rxflag		DCD	0

		;receive buffer
	EXPORT	RecvBuf
	EXPORT	XmitBuf
XmitBuf	EQU	0x1000
	^	XmitBuf
	#	0x100	; should be 0x600
RecvBuf	EQU	0x1100
	^	RecvBuf
	#	0x300	; should be 0x600

        END
