        TTL     C-Demon FP initialisation and support routines      > fp_init.s
        ; ---------------------------------------------------------------------
        ; $Revision: 0.1 $
        ;   $Author: jsmith $
        ;     $Date: 941102 $
        ;
        ; Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
        ; ---------------------------------------------------------------------
        ; Derived from the following ARM Ltd source:
        ;;; Assembler source for FPA support code and emulator-PIE veneer
        ;;; Pre-core part.
        ;;; Copyright (C) Advanced RISC Machines Limited, 1992-3. All rights reserved.

        ;;; RCS $Revision: 1.4 $
        ;;; Checkin $Date: 93/11/01 12:02:44 $
        ;;; Revising $Author: irickard $
        ; ---------------------------------------------------------------------
	; "@(#)fp_init.s	1.2	96/06/07 VLSI"

        INCLUDE global_h.s      ; standard C-Demon definitions
        INCLUDE swis_h.s        ; SWI definitions

        AREA    |FPE$$init$$code|,CODE,READONLY

        IMPORT  FP_image

        IMPORT  FPE_Install
        IMPORT  |FPE$$code$$Base|
        IMPORT  |FPE$$code$$Limit|

        EXPORT  FPE_GenerateError
        EXPORT  FPE_CopyAndInstall

        EXPORT  ReadFPstatus
        EXPORT  ReadFPregister
        EXPORT  WriteFPstatus
        EXPORT  WriteFPregister

        ; The following magic number is where the Floating Point
        ; Emulator expects to be installed:
FPInstalledAddress      EQU     &1400

        ; ---------------------------------------------------------------------

FPE_CopyAndInstall
        STMDB   sp!, {r3-r10}           ;save registers r3 to r10
        LDR     a1, =|FPE$$code$$Base|
        LDR     a2, =|FPE$$code$$Limit|
        MOV     a3, #FPInstalledAddress

01      LDMIA   a1!, {v3-r10}
        STMIA   a3!, {v3-r10}
        CMP     a1, a2
        BLO     %B01
        ; if we're big-endian, error strings at the end of the FPE
        ; image need to be reversed
        MOV     v3, #1                  ;test for endianess
        STR     v3, [a3, #4]
        LDRB    v4, [a3, #4]
        TEQ     v3, v4                  ;if v3 same as v4 then little endian
        SUB     a3, a3, a1              ; amount to relocate ROM addresses by
        BEQ     EndReverse              ;if little endian do not reverse anything
        ADD     a2, a2, a3              ; end of FPE image in RAM

NextWord
        LDR     a1, [a2, #-4]!
        CMP     a1, #-1
        BEQ     EndReverse
        AND     v3, a1, #&ff000000
        CMP     v3, #&80000000
        BEQ     NextWord        ; an error number, not part of a string

        EOR     v3, a1, a1, ROR #16        ; reverse bytes in word
        BIC     v3, v3, #&ff0000
        MOV     a1, a1, ROR #8
        EOR     a1, a1, v3, LSR #8
        STR     a1, [a2]
        B       NextWord

EndReverse
        LDR     a2, =FPE_Install
        ADD     a2, a2, a3
        LDMIA   sp!, {r3-r10}   ;restore registers r3 to r10
        MOV     pc, a2

        ; ---------------------------------------------------------------------

FPE_GenerateError
        SWI     GenerateError

        ; ---------------------------------------------------------------------
        ; ReadFPstatus
        ; in:   void
        ; out:  a1 = FP status
ReadFPstatus
        RFS     a1              ; return FP status reg in a1
        MOV     pc,lr

        ; ---------------------------------------------------------------------
        ; WriteFPstatus
        ; in:   a1 = New FP status
        ; out:  void
WriteFPstatus
        WFS     a1              ; set FP status reg to contents of a1
        MOV     pc,lr

        ; ---------------------------------------------------------------------
        ; ReadFPregister
        ; in:   a1 = index of FP register to read
        ; out:  a1 = FP_image + 0
        ;       a2 = FP_image + 4
        ;       a3 = FP_image + 8
ReadFPregister
        LDR     a4, =FP_image   ; place to temp read FP reg into
        MOV     a2,#0
        STR     a2,[a4,#0]
        STR     a2,[a4,#4]
        STR     a2,[a4,#8]

        CMP     a1,#0           ; get appropriate FP register by a1
        STFEQE  f0,[a4]
        CMP     a1,#1
        STFEQE  f1,[a4]
        CMP     a1,#2
        STFEQE  f2,[a4]
        CMP     a1,#3
        STFEQE  f3,[a4]
        CMP     a1,#4
        STFEQE  f4,[a4]
        CMP     a1,#5
        STFEQE  f5,[a4]
        CMP     a1,#6
        STFEQE  f6,[a4]
        CMP     a1,#7
        STFEQE  f7,[a4]

        LDR     a1,[a4,#0]      ; return FP image in a1-a3
        LDR     a2,[a4,#4]
        LDR     a3,[a4,#8]
        MOV     pc,lr

        ; ---------------------------------------------------------------------
        ; WriteFPregister
        ; in:   a1 = index of FP register to write
        ;       a2 = FP_image + 0
        ;       a3 = FP_image + 4
        ;       a4 = FP_image + 8
        ; out:  void
WriteFPregister
        STMDB   sp!,{v1,lr}     ; save reg v1 and lr
        LDR     v1, =FP_image   ; place to temp write FP reg image
        STR     a2,[v1,#0]      ; place a2-a4 in FP image
        STR     a3,[v1,#4]
        STR     a4,[v1,#8]

        CMP     a1,#0           ; place in appropriate FP register by a1
        LDFEQE  f0,[v1]
        CMP     a1,#1
        LDFEQE  f1,[v1]
        CMP     a1,#2
        LDFEQE  f2,[v1]
        CMP     a1,#3
        LDFEQE  f3,[v1]
        CMP     a1,#4
        LDFEQE  f4,[v1]
        CMP     a1,#5
        LDFEQE  f5,[v1]
        CMP     a1,#6
        LDFEQE  f6,[v1]
        CMP     a1,#7
        LDFEQE  f7,[v1]

        LDMIA   sp!,{v1,pc}     ; restore v1 and return

        ; ---------------------------------------------------------------------
        END     ; EOF fp_init.s
