/*> global.h
 * General include file for all C-Demon targets.
 *
 * $Revision: 0.1 $
 *   $Author: jsmith $
 *     $Date: 941030 $
 *
 * Original author: Geary Chopoff.
 *
 * Copyright (c) 1993-1994, VLSI Technology Inc. All Rights Reserved.
 *
 * "@(#)global.h	1.4	96/06/07 VLSI"
 */
/*---------------------------------------------------------------------------*/

#ifndef __cdemon_global_h
#define __cdemon_global_h

/*---------------------------------------------------------------------------*/
/* general compiler pragma directives */
#pragma no_check_stack
/* NOTE: The above pragma will stop the compiler from generating
 * software stack-limit checking on function entry. This means that if
 * any of the C-Demon code uses more than the available stack then the
 * system will be compromised.
 */

/*---------------------------------------------------------------------------*/
/* general type definitions */
typedef volatile unsigned int    REG ;
typedef unsigned char           *B_ADDR ;
typedef unsigned char            BYTE ;
typedef unsigned int            *W_ADDR ;
typedef unsigned int             WORD ;
typedef int                      BOOLEAN ;

typedef volatile unsigned char *vuptr ; /* for accessing byte wide I/O memory */
typedef volatile unsigned int  *vwptr ; /* for accessing word wide I/O memory */

/* definition of switches */
#define FALSE       0           /* we hold these to be self-evident */
#define TRUE        !FALSE
#define NULL        0

#define ENABLE      1
#define DISABLE     0

/*---------------------------------------------------------------------------*/
/* The following function prototypes define the routines that *MUST*
 * be provided by the target architecture specific driver files. This
 * is the C-Demon hardware support layer.
 */

extern const char   *ResetDriver(void) ;
extern void          DriverSpeed(int baud) ;
extern unsigned char GetByte(void) ;
extern int           AnyByte(void) ;
extern void          PutByte(unsigned char c) ;
extern void          SetLEDs(char LED) ;

extern unsigned int  ReadTimer(void) ;
extern void          SetTimer(unsigned int tick) ;
extern unsigned int  GetWord(void) ;
extern void          PutWord(unsigned int wrd) ;
extern void          PrintString(char *cp) ;

#ifdef SMC91C92
/*
 * These are for ethernet.
 */
extern void ENET_Init(void);
extern void rx_packet(void);
extern void enet_header(unsigned char *tbuf, short len);
extern void ENET_TX(unsigned char *tbuf, short len, unsigned char *addr);
extern unsigned char *bcopy(unsigned char *sa, unsigned char *da, int len);

extern int use_enet, use_serial;
extern int rdp_index;
extern int rdp_in;
extern int rxflag;
extern int rdp_opened;
extern int send_flag;

#define TXADDR	0x1000	/* xmit buffer address */
#define RXADDR	0x1100	/* receive buffer address */
extern unsigned char daddr[];
extern unsigned char Dest_Addr[];
extern unsigned char Src_Addr[];
#endif

/*---------------------------------------------------------------------------*/
/* FuncIndirectZP provides a simple method of indirecting through a
 * memory vector to an attached code routine.
 */
extern void FuncIndirectZP(unsigned int *vector) ;

/*---------------------------------------------------------------------------*/

extern unsigned int   LEDimage ;        /* Soft-copy of the LEDs (if any) */
extern unsigned int   TickVal ;         /* C-Demon soft timer tick */
extern int            TimeZero ;        /* C-Demon start-of-day timer value */
extern int            RAMSize ;         /* Holder for C-Demon calculated RAM size */
extern char           DebuggeeRunning ; /* C-Demon target execution state */
extern int            EndianWord ;      /* Location used when guessing target endianness */
extern int            ErrorP ;          /* Last error pointer */
extern int            CaughtVectors ;   /* Bitmap of ARM vectors to trap back to host */
extern int            ROMEnd ;          /* Variable holding end of ROM address */
extern int            MMUtype ;         /* Variable used to hold MMU information */
extern unsigned char  BP_RDP_STS ;      /* C-Demon last taken breakpoint status */
extern int           *BP_addr ;         /* vector of BreakPoint addresses */
extern int           *BP_inst ;         /* vector of BreakPoint original instructions */

/* The CommandLine is not actually an integer, but this provides a
 * handle onto the symbol so that its address can be taken. It is
 * performed this way as a remnant from earlier C-Demon versions,
 * rather than being provided as a true type definition.
 */
extern int           CommandLine ;

/*---------------------------------------------------------------------------*/
/* External Vectors */
/*                                    num  addr description */
extern unsigned int ResetV ;       /* 0x00 00 - RESET */
extern unsigned int UndefInstrV ;  /* 0x01 04 - Undefined instruction */
extern unsigned int SWIV ;         /* 0x02 08 - SWI */
extern unsigned int PrefAbortV ;   /* 0x03 0C - Abort inst */
extern unsigned int DataAbortV ;   /* 0x04 10 - Abort data */
extern unsigned int AddrExceptV ;  /* 0x05 14 - (reserved) */
extern unsigned int IRQV ;         /* 0x06 18 - IRQ interrupt */
extern unsigned int FIQV ;         /* 0x07 1C - FIQ interrupt */
extern unsigned int ErrorV ;       /* 0x08 20 - Error */
extern unsigned int GetByteV ;     /* 0x09 xx - Get Byte */
extern unsigned int PutByteV ;     /* 0x0A xx - Put Byte */
extern unsigned int GetWordV ;     /* 0x0B xx - Get Word */
extern unsigned int PutWordV ;     /* 0x0C xx - Put Word */
extern unsigned int PrintStringV ; /* 0x0D xx - Print ASCIIZ string to HOST */


extern unsigned int FIQvectors ;   /* 0x10 to 0x17 individual vector per FIQ bits */
extern unsigned int IRQvectors ;   /* 0x20 to 0x27 individual vector per IRQ bits */
#ifdef VY86C650
extern unsigned int IRQ1vectors ;  /* 0x30 to 0x37 individual vector per IRQ1 bits */
#endif /* VY86C650 */

/*---------------------------------------------------------------------------*/
/* These structures are used to pass multiple argument results back
 * from SWI calls, and the FP support routines.
 */

typedef struct ret2_struct {unsigned int p1 ; unsigned int p2 ; } ret2parm ;
typedef struct ret3_struct {unsigned int p1 ; unsigned int p2 ; unsigned int p3 ; } ret3parm ;

/*---------------------------------------------------------------------------*/
/* The following are defined in "fp_init.s" */

extern unsigned int ReadFPstatus(void) ;
extern void         WriteFPstatus(unsigned int sts) ;
extern void         WriteFPregister(int reg,unsigned int e0,unsigned int e1,unsigned int e2) ;
extern __value_in_regs ret3parm ReadFPregister(int reg) ;

/*---------------------------------------------------------------------------*/

#endif /* __cdemon_global_h */

/*---------------------------------------------------------------------------*/
/*> EOF global.h <*/
