        TTL     C-Demon vector handling support                     > handler.s
        ; ---------------------------------------------------------------------
        ; Main C-Demon exception and event handling.
        ;
        ; $Revision: 0.1 $
        ;   $Author: jsmith $
        ;     $Date: 941031 $
        ;
        ; Original author: Geary Chopoff.
        ;
        ; Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
        ; ---------------------------------------------------------------------
	; "@(#)handler.s	1.3	96/06/07 VLSI"

        INCLUDE global_h.s              ; include global variables
        INCLUDE $DRIVER./driver_h.s     ; target specific memory and I/O
        INCLUDE bp_h.s                  ; breakpoint support
        INCLUDE swis_h.s                ; for SWIHandler
        INCLUDE rdi_h.s                 ; for RDP definitions

        KEEP                            ; preserve local (non-exported) symbols

        ; ---------------------------------------------------------------------

        IMPORT  IRQvectors
        IMPORT  FIQvectors

        IMPORT  Handlers
        IMPORT  CaughtVectors
        IMPORT  DebugCPSR
        IMPORT  DebugSP
        IMPORT  DebugLR

        ; The following SWI functions are *NOT* declared in the
        ; "swis_h.s" header file since they should not be called
        ; directly. They are intended to be called via the SWI
        ; vectoring code within this source file.
        ;
        IMPORT  inSWIWriteC             ; 0x00
        IMPORT  inSWIWrite0             ; 0x02
        IMPORT  inSWINewLine            ; 0x03
        IMPORT  inSWIReadC              ; 0x04
        IMPORT  inSWICLI                ; 0x05

        IMPORT  inSWIGetEnv             ; 0x10
        IMPORT  inSWIExit               ; 0x11

        IMPORT  inSWIGetErrno           ; 0x60  ; MinSWI2
        IMPORT  inSWIClock              ; 0x61
        IMPORT  inSWIClockInit          ; 0x62
        IMPORT  inSWITime               ; 0x63
        IMPORT  inSWIRemove             ; 0x64
        IMPORT  inSWIRename             ; 0x65
        IMPORT  inSWIOpen               ; 0x66
        IMPORT  inSWIGetVector          ; 0x67
        IMPORT  inSWIClose              ; 0x68
        IMPORT  inSWIWrite              ; 0x69
        IMPORT  inSWIRead               ; 0x6A
        IMPORT  inSWISeek               ; 0x6B
        IMPORT  inSWIFlen               ; 0x6C

        IMPORT  inSWIIsTTY              ; 0x6E
        IMPORT  inSWITmpNam             ; 0x6F

        IMPORT  inSWIInstallHandler     ; 0x70
        IMPORT  inSWIGenerateError      ; 0x71  ; MaxSWI2

        IMPORT  BP_pc
        IMPORT  BP_r0
        IMPORT  BP_type
        IMPORT  BP_StopInt              ; simulate breakpoint
        IMPORT  BP_CaughtVector         ; simulate breakpoint

        IMPORT  GetByte                 ; get byte from debug channel
        IMPORT  Level1RDI               ; process a RDP message

        ; ---------------------------------------------------------------------
        ; Base ARM vector handlers:

        EXPORT  UndefHandler
        EXPORT  SWIHandler
        EXPORT  PrefetchAbortHandler
        EXPORT  DataAbortHandler
        EXPORT  IRQHandler
        EXPORT  FIQHandler
        EXPORT  IgnoreIRQ
        EXPORT  IgnoreFIQ
        EXPORT  IgnoreVEC
        EXPORT  SerialINT

        ; ---------------------------------------------------------------------

        AREA    |C-Demon$$Handlers|,CODE,READONLY

        ; ---------------------------------------------------------------------

        EXPORT  FuncIndirectZP  ; branch through the referenced vector
FuncIndirectZP
        LDR     pc,[a1,#0x00]   ; provided as assembler to make 'C' source cleaner

        ; ---------------------------------------------------------------------
        ; -- IRQ handling -----------------------------------------------------
        ; ---------------------------------------------------------------------
        ; IRQHandler
        ; in:   IRQ mode; IRQs disabled; FIQs undefined.
        ;       r13 = FD stack
        ;       r14 = (return address + 4)
        ; out:  Processor state at instance of interrupt restored.
        ;
        ; We vector through the IRQvectors table (or RTI if no bits
        ; set, though this would mean that we have not found the IRQ
        ; source, and the system is probably going to loop
        ; infinitely).
        ;
        ; The handler function is called as if it was called
        ; immediately from the IRQ vector. Even though this requires
        ; some more processing here, and may cause problems for
        ; targets where reading the IRQ status flags may clear the
        ; source, it does allow applications more control over the
        ; results of an interrupt. e.g. a multi-threaded system using
        ; C-Demon for the base target support may want to perform
        ; context switches as part of an interrupt handler. If we did
        ; not enter the handler with a clean IRQ state, they
        ; application would need to unwind the C-Demon IRQ state
        ; before saving the context of the thread that was
        ; interrupted.
        ;
IRQHandler
        STMFD   sp!,{a1,a2,pc}          ; save some work registers
        getIRQsource    a1,a2           ; load the IRQ status into "a1"
        TEQ     a1,#0x00000000          ; check that we have at least one bit
        BEQ     IgnoreIRQ_stacked       ; if zero we do not know the source
        ; a1 = currently active IRQ status
        decodeIRQ       a1,a2           ; get a vector number for the sources
        ;
        ; a1 = IRQ vectors slot number of desired handler
        LDR     a2,=IRQvectors
        LDR     a2,[a2,a1,LSL #2]       ; load the handler address
        ; NOTE: The vector table should contain pointers to
        ; "IgnoreIRQ" for those slots that do not have an attached
        ; handler.
        STR     a2,[sp,#(2 * 4)]        ; over-write the stacked pc
        LDMFD   sp!,{a1,a2,pc}          ; and enter the handler

IgnoreIRQ_stacked
        ; in:  working registers stacked from IRQHandler entry
        ; out: exit through the IgnoreIRQ code.
        LDMFD   sp!,{a1,a2}             ; recover work registers
        ADD     sp,sp,#4                ; and ignore "pc" location
IgnoreIRQ
        ; in:  state as at instance of the IRQ vector entry
        ; out: return to interrupted thread
        ;
        ; If we reach here then we do not have a handler for the
        ; interrupt. We could return to the caller with IRQs disabled
        ; (to avoid an infinite loop), but for the moment we assume
        ; that reading the IRQ status above (getIRQsource) will have
        ; cleared the source.
        ;
        SUBS    pc,lr,#4

        ; ---------------------------------------------------------------------
        ; -- FIQ handling -----------------------------------------------------
        ; ---------------------------------------------------------------------
        ; FIQHandler
        ; in:   FIQ mode; IRQs undefined; FIQs disabled.
        ;       r13 = FD stack
        ;       r8-r12 available as private work registers
        ;       r14 = (return address + 4)
        ; out:  Processor state at instance of FIQ restored.
        ;
        ; See the comments for the IRQHandler routine above. NOTE:
        ; This system does not make optimum use of the extra FIQ
        ; banked registers. Since this C-Demon code provides support
        ; for multiple FIQ sources, individual handlers cannot use the
        ; FIQ registers for their own private use without the
        ; co-operation of the other FIQ handler routines.
        ;
        ; So the FIQ handlers for a particular application can use the
        ; banked FIQ registers, this code performs much the same work
        ; as the IRQHandler, and stacks its work registers.
FIQHandler
        STMFD   sp!,{a1,a2,pc}          ; create PC holder on the stack
        getFIQsource    a1,a2           ; load the FIQ status into "a1"
        TEQ     a1,#0x00000000          ; check that we have at least one bit
        BEQ     IgnoreFIQ_stacked       ; if zero we do not know the source
        decodeFIQ       a1,a2,a3        ; get a vector number for the sources
        LDR     a2,=FIQvectors
        LDR     a2,[a2,a1,LSL #2]       ; load the handler address
        STR     a2,[sp,#(2 * 4)]        ; over-write the stacked pc
        LDMFD   sp!,{a1,a2,pc}          ; and enter the handler

IgnoreFIQ_stacked
        ; in:  working registers stacked from FIQHandler entry.
        ; out: exit through the IgnoreFIQ code.
        LDMFD   sp!,{a1,a2}             ; recover stacked work registers
        ADD     sp,sp,#4                ; and dump stack storage space
        ; and fall through to the normal FIQ ignore code:
IgnoreFIQ
        ; in:  state as at instance of the FIQ vector entry
        ; out: return to interrupted thread
        ;
        ; See the comments attached to the "IgnoreFIQ" routine.
        SUBS    pc,lr,#4

        ; ---------------------------------------------------------------------

SerialINT
        ; This is a special C-Demon interrupt handler used to provide
        ; a hook for the host debugger asynchronous interrupt feature.
        ;
        ; in:   interrupt mode; IRQ and FIQ disabled as required;
        ;       r13 = FD stack
        ;       r14 = (return address + 4)
        ;
        ; This interrupt handler does not actually read the data here,
        ; since we leave the message processing to much later. This
        ; means we need to ensure that overrun does not occur whilst
        ; we are performing our pseudo-breakpoint code:
        ;
        STMFD   sp!,{r0-r7,lr}          ; create some work registers
        ;
        dropRTS r0,r1                   ; drop RTS to stop host transmitting
        ;
        ; The "BP_StopInt" call expects to be called with the
        ; following stack state: {r0-r7,lr}
        LDR             pc,=BP_StopInt                  ; and simulate a breakpoint

        ; ---------------------------------------------------------------------
        ; Insert architecture specific interrupt support code into
        ; this file. This avoids having conditional source cluttering
        ; up this file:

        PRIVATEINTERRUPTS

        ; ---------------------------------------------------------------------

        LTORG

        ; ---------------------------------------------------------------------
        ; -- SWI processing ---------------------------------------------------
        ; ---------------------------------------------------------------------
        ; The following SWI number limits describe the shape of the
        ; SWIJumpTable:

MaxSWI1 EQU     0x17    ; last valid value of 1st half of SWIJumpTable
MinSWI2 EQU     0x60    ; start value of 2nd half of SWIJumpTable
MaxSWI2 EQU     0x71    ; last valid value of supported SWI's

AdjSWI  EQU     (MinSWI2 - MaxSWI1 - 1) ; for next location after MaxSWI1

        ; ---------------------------------------------------------------------
        ; SWIHandler
        ; in:   SVC mode; IRQs disabled; FIQs as caller
        ;       a1 to a3 = parameters passed when SWI was invoked
        ; out:  a1 to a4 = undefined
        ;       All other state restored
        ;
        ; NOTE: At the moment this code *RELIES* on SWIs only using a1
        ; to a3 as standard arguments. The SWI code adds the SWI
        ; number as a final argument for most of the SWIs. This is
        ; just a way of ensuring that the SWI handler functions do not
        ; need to know the physical SWI number they are mapped to. If
        ; any SWI handlers are added that need more than 4 arguments
        ; (3 plus the SWI number) then this code will required
        ; extensive changes.
SWIHandler
        ; The FIQ disable state is as the caller of the SWI. The IRQ
        ; disable state has been explicitly set by the SWI vector
        ; entry.

        STMDB   sp!,{a4-ip,lr}  ; save regs a4 to ip and lr (r3-r12,r14)

        stopCTRLC a4,v4         ; ensure no user-interrupts over the SWI code

        LDR     v4,[lr,#-4]             ; get the SWI instruction
        BIC     v4,v4,#0xFF000000       ; extract the SWI number and keep in v4
        MOV     a4,v4                   ; place SWI number in a4
        CMP     v4,#MaxSWI2             ; is number > max value expected?
        BHI     inSWIUnknown            ; if so, tell user

        CMP     v4,#MaxSWI1             ; is number<=max for 1st half of table?
        BLE     SkipAdjSWI              ; yes, skip adjust of vector

        SUB     v4,v4,#AdjSWI           ; otherwise adjust value in v4
        CMP     v4,#MaxSWI1             ; now is it too low?
        BLE     inSWIUnknown            ; if so, it is an error
SkipAdjSWI
        ; Note: This code relies on having valid "sl" (stack-limit)
        ; and "fp" (frame-pointer) registers passed to it. The SWI
        ; handler uses the current SVC stack, and the current "sl",
        ; which may actually reference a different stack. This code
        ; should be updated to generate a valid "sl" value before
        ; calling the APCS-3 routine, or the routine being called
        ; should be compiled/assembled without software stack
        ; checking. Similarly the "fp" being used may reference a
        ; different stack. This should not be a problem in a world
        ; with a linear address space, but WILL cause problems where a
        ; MMU is used to provide multiple overlapping logical address
        ; spaces.
        ;
        ADR     v1,SWIadjust            ; point v1 at base of table
        LDR     pc,[v1,v4,LSL #2]       ; adjust params via v4
        ;
        ; The following code is reached via the table indirection
        ; above. It simply moves the SWI number into the "last"
        ; argument register depending on the number of parameters the
        ; handler function we are about to call has.
        ;
        ; The following actually overwrites one of the non-saved
        ; APCS-3 argument registers if neither Parm_0 or Parm_4 is
        ; specified. We need to ensure that unknown SWIs do *NOT* have
        ; any registers over-written before they call the vector
        ; catching code to ensure that the host debugger is not
        ; presented with mis-leading register information.
        ;
Parm_1
        MOV     a1,a4           ; pass SWI number as only parameter
Parm_2
        MOV     a2,a4           ; pass SWI number as second parameter
Parm_3
        MOV     a3,a4           ; pass SWI number as third parameter
Parm_0                          ; 0 = no params so just continue
Parm_4                          ; SWI number is already the fourth parameter
        ;
        ; NOTE: We use the "Parm_0" case when the SWI function has
        ; parameters, but is not interested in having a copy of the
        ; "a4" SWI number. The functions for those SWIs handled
        ; completely by C-Demon do not need to know their id. Only
        ; when the SWI requires some host debugger support is the SWI
        ; number required. Also we use "Parm_0" for "unknown" SWIs to
        ; ensure that the vector catch code has access to the complete
        ; processor state at the instance of the SWI.
        ;
SWIParmDone
        ; Now that we have the SWI number parameter setup we can call
        ; the required handler function:
        ADR     v1,SWIJumpTable         ; point v1 at base of table
        ADR     lr,SWI_RET              ; fake a BL
        LDR     pc,[v1,v4,LSL #2]       ; goto SWI via v4 (contains SWI number)

        ; The following MACROs are used to encapsulate the standard
        ; SWI exit sequence required by the default SWI return code,
        ; and the SWI handlers provided in assembler. They ensure that
        ; the exit code matches the stacked entry state, without
        ; having to update multiple pieces of source:

        MACRO
$label  doSWIstack
$label
        LDMIA   sp!,{a4-ip,lr}  ; restore a4 to ip and lr
        MEND

        MACRO
$label  doSWIRET    $setcc
$label
        ; This routine is the standard exit point for the SWIs that
        ; perform some RDP communications. Now that the communications
        ; have finished we can re-enable the FIQ that enables
        ; RDP_SignalStop to be processed whilst the target application
        ; is executing.
        ;
        ; NOTE: Since the only SWIs performed (notionally) by DEMON
        ; are during the breakpoint code, we do not need any special
        ; processing to check that we were executing the application
        ; when the SWI occurred. Also, we do not need to worry about a
        ; FIQ occurring that interrupts the execution whilst we are in
        ; the SWI entry or exit sequence, since we will not yet have
        ; started any RDP messages, or will have finished with the
        ; message.
        ;
        allowCTRLC  a4,v1       ; so the user can asynchronously interrupt
        ;
        doSWIstack              ; clear the stack
        ; The "setcc" string is a simple boolean that controls whether
        ; the original PSR state should be restored during the return:
        [   ("$setcc" = "")
        MOVS    pc,lr           ; return from SWI
        |
        MOV     pc,lr
        ]
        MEND

SWI_RET
        ; This code is reached when the SWI handler has completed. We
        ; perform the standard SWI exit processing:
        doSWIRET

        ; ---------------------------------------------------------------------
        ; -- Lo-level SWI handlers --------------------------------------------
        ; ---------------------------------------------------------------------
        ; inSWIEnableINT
        ; in:   a4 = 0x13
        ; out:  void
        ;
        ; Enable both FIQ and IRQ interrupts
        ;
inSWIEnableINT
        MRS     a4,SPSR         ; get the saved PSR
        BIC     a4,a4,#NoInt    ; enable both FIQ and IRQ interrupts
        MSR     SPSR,a4         ; set the saved PSR
        doSWIRET                ; perform standard exit sequence

        ; ---------------------------------------------------------------------
        ; inSWIDisableINT
        ; in:   a4 = 0x14
        ; out:  void
        ;
        ; Disable both FIQ and IRQ interrupts
        ;
inSWIDisableINT
        MRS     a4,SPSR         ; get the saved PSR
        ORR     a4,a4,#NoInt    ; disable both FIQ and IRQ interrupts
        MSR     SPSR,a4         ; set the saved PSR
        doSWIRET                ; perform standard exit sequence

        ; ---------------------------------------------------------------------
        ; inSWIEnterOS
        ; in:   a4 = 0x16
        ; out:  void
        ;
        ; Return to the caller in SVC (supervisor) mode.
inSWIEnterOS
        MRS     a4,SPSR         ; get USER's flags
        BIC     a4,a4,#ModeMask ; return to USER with whatever masks they have
        ORR     a4,a4,#SVCmode  ; already set
        MSR     CPSR,a4
        doSWIRET    no          ; perform standard exit sequence

        ; ---------------------------------------------------------------------
        ; inSWIBreak    software breakpoint
        ; inSWIExiting  terminate foreground application
        ; in:   a1 = num = 0x11 or 0x17
        ; out:  a1 = num = 0x11 or 0x17
        ;       stack cleared and redirects into SWIExit (in swi.c)
        ;
        ; This code is called when no more SWI's are expected. SWI's are
        ; normally called from a child, when a break (or completion) occurs,
        ; we return to the parent process, which may decide to resume via
        ; an RDIExecute.
        ;
inSWIBreak      ; SWI number 0x17
inSWIExiting    ; SWI number 0x11
        LDR     v1,=BP_type
        LDR     v1,[v1,#0x00]   ; get type of breakpoint event
        ; <0 = special
        ;  0 = normal
        ; >0 = breakpoint number
        ;
        CMP     v1,#ASYNCBREAK
        BNE     not_user_interrupt
        ;
        ; Get the first byte of the RDP message:
        BL      GetByte
        ; and perform a single RDP message process:
        BL      Level1RDI
        ; This code will work because the standard C-Demon transfers are
        ; polled across, and the hi-level RDP_SignalStop routine currently
        ; does nothing. The above should eat the user-interrupt RDP
        ; message and then generate a standard break-point reply.
        ;
        ; NOTE: This may not be the best thing to do if the message is
        ; not actually a SignalStop.
not_user_interrupt
        ;
        CMP     v1,#NORMAL      ; is it the NORMAL exit?
        doSWIstack              ; clear the stack
        BNE     NotNormal       ; if not, registers already saved
        ;
        STMFD   sp!,{lr}        ; preserve lr

        LDR     lr,=BP_r0       ; address register save area
        STMIA   lr,{a1-ip}      ; (save r0 to r12)

        LDMFD   sp!,{lr}        ; restore lr

        LDR     v3,=BP_pc
        SUB     lr,lr,#4        ; calculate location of pc from lr
        STR     lr,[v3,#0x00]   ; then set value of BP_pc
NotNormal
        ; The register state has now been preserved.
        ;
        LDR     a2,=DebugCPSR   ; place variable base addr in a2
        LDR     a2,[a2,#0x00]   ; load the debug foreground PSR
        MSR     CPSR,a2         ; switch to desired mode
        ;
        ; Perform the inSWIExit in the stack of the "C-Demon foreground
        ; thread", and then return to that context:
        LDR     a2,=DebugSP
        LDR     sp,[a2,#0x00]   ; get required stack pointer
        LDR     a2,=DebugLR
        LDR     lr,[a2,#0x00]   ; get required return location
        B       inSWIExit       ; finish exiting

        ; ---------------------------------------------------------------------

SWIJumpTable
        DCD     inSWIWriteC             ; 0x00
        DCD     inSWIUnknown            ; 0x01
        DCD     inSWIWrite0             ; 0x02
        DCD     inSWINewLine            ; 0x03
        DCD     inSWIReadC              ; 0x04
        DCD     inSWICLI                ; 0x05
        DCD     inSWIUnknown            ; 0x06
        DCD     inSWIUnknown            ; 0x07
        DCD     inSWIUnknown            ; 0x08
        DCD     inSWIUnknown            ; 0x09
        DCD     inSWIUnknown            ; 0x0A
        DCD     inSWIUnknown            ; 0x0B
        DCD     inSWIUnknown            ; 0x0C
        DCD     inSWIUnknown            ; 0x0D
        DCD     inSWIUnknown            ; 0x0E
        DCD     inSWIUnknown            ; 0x0F
        DCD     inSWIGetEnv             ; 0x10
        DCD     inSWIExiting            ; 0x11
        DCD     inSWIUnknown            ; 0x12
        DCD     inSWIEnableINT          ; 0x13
        DCD     inSWIDisableINT         ; 0x14
        DCD     inSWIUnknown            ; 0x15
        DCD     inSWIEnterOS            ; 0x16
        DCD     inSWIBreak              ; 0x17  ; MaxSWI1

        DCD     inSWIGetErrno           ; 0x60  ; MinSWI2
        DCD     inSWIClock              ; 0x61
        DCD     inSWIClockInit          ; 0x62
        DCD     inSWITime               ; 0x63
        DCD     inSWIRemove             ; 0x64
        DCD     inSWIRename             ; 0x65
        DCD     inSWIOpen               ; 0x66
        DCD     inSWIGetVector          ; 0x67
        DCD     inSWIClose              ; 0x68
        DCD     inSWIWrite              ; 0x69
        DCD     inSWIRead               ; 0x6A
        DCD     inSWISeek               ; 0x6B
        DCD     inSWIFlen               ; 0x6C
        DCD     inSWIUnknown            ; 0x6D
        DCD     inSWIIsTTY              ; 0x6E
        DCD     inSWITmpNam             ; 0x6F
        DCD     inSWIInstallHandler     ; 0x70
        DCD     inSWIGenerateError      ; 0x71  ; MaxSWI2

        ; ---------------------------------------------------------------------
        ; This table matches the SWI handler table, and describes the
        ; number of parameters required by each of the SWI handlers:
SWIadjust
        DCD     Parm_2  ; SWIWriteC         ; 0x00
        DCD     Parm_1  ; SWIUnknown        ; 0x01
        DCD     Parm_2  ; SWIWrite0         ; 0x02
        DCD     Parm_1  ; SWINewLine        ; 0x03
        DCD     Parm_1  ; SWIReadC          ; 0x04
        DCD     Parm_2  ; SWICLI            ; 0x05
        DCD     Parm_1  ; SWIUnknown        ; 0x06
        DCD     Parm_1  ; SWIUnknown        ; 0x07
        DCD     Parm_1  ; SWIUnknown        ; 0x08
        DCD     Parm_1  ; SWIUnknown        ; 0x09
        DCD     Parm_1  ; SWIUnknown        ; 0x0A
        DCD     Parm_1  ; SWIUnknown        ; 0x0B
        DCD     Parm_1  ; SWIUnknown        ; 0x0C
        DCD     Parm_1  ; SWIUnknown        ; 0x0D
        DCD     Parm_1  ; SWIUnknown        ; 0x0E
        DCD     Parm_1  ; SWIUnknown        ; 0x0F
        DCD     Parm_1  ; SWIGetEnv         ; 0x10
        DCD     Parm_0  ; SWIExiting        ; 0x11
        DCD     Parm_1  ; SWIUnknown        ; 0x12
        DCD     Parm_0  ; SWIEnableINT      ; 0x13
        DCD     Parm_0  ; SWIDisableINT     ; 0x14
        DCD     Parm_1  ; SWIUnknown        ; 0x15
        DCD     Parm_0  ; SWIEnterOS        ; 0x16
        DCD     Parm_0  ; SWIBreak          ; 0x17  ; MaxSWI1

        DCD     Parm_1  ; SWIGetErrno       ; 0x60  ; MinSWI2
        DCD     Parm_0  ; SWIClock          ; 0x61
        DCD     Parm_0  ; SWIClockInit      ; 0x62
        DCD     Parm_1  ; SWITime           ; 0x63
        DCD     Parm_2  ; SWIRemove         ; 0x64
        DCD     Parm_3  ; SWIRename         ; 0x65
        DCD     Parm_3  ; SWIOpen           ; 0x66
        DCD     Parm_0  ; SWIGetVector      ; 0x67  ; actually 1 parameter
        DCD     Parm_2  ; SWIClose          ; 0x68
        DCD     Parm_4  ; SWIWrite          ; 0x69
        DCD     Parm_4  ; SWIRead           ; 0x6A
        DCD     Parm_3  ; SWISeek           ; 0x6B
        DCD     Parm_2  ; SWIFlen           ; 0x6C
        DCD     Parm_1  ; SWIUnknown        ; 0x6D
        DCD     Parm_2  ; SWIIsTTY          ; 0x6E
        DCD     Parm_3  ; SWITmpNam         ; 0x6F
        DCD     Parm_0  ; SWIInstallHandler ; 0x70  ; actually 3 paramaters
        DCD     Parm_0  ; SWIGenerateError  ; 0x71  ; actually 2 parameters ; MaxSWI2

        ; ---------------------------------------------------------------------
        ; -- Exception handling -----------------------------------------------
        ; ---------------------------------------------------------------------

        MACRO
$label  callCaughtVector    $reason
$label
        MOV     r10,#$reason
        LDR     pc,=BP_CaughtVector     ; and simulate a breakpoint
        MEND

        MACRO
$label  callHandler         $reason
$label
        ; Check for a handler, and call it with the defined state:
        LDREQ   r11,=Handlers       ; reference the Handlers vector
        ADDEQ   r11,r11,#$reason    ; specific to the abort vector
        LDR     r14,[r11,#0x04]     ; get handler address
        TEQ     r14,#0x00000000     ; check if valid PC
        MOVNE   r14,pc
        LDMNEIA r11,{r11,pc}
        ; If there was no handler address defined then we will just
        ; fall through to the normal raise abort code:
        MEND

        ; ---------------------------------------------------------------------

inSWIUnknown
        ; sp = FD stack containing entry {a4-ip,lr}
        ; NOTE: When this code is entered from the SWIHandler we
        ; *KNOW* that it has not overwritten any of a1, a2 or a3 with
        ; the number of the SWI. We also do not need to keep the SWI
        ; number since the handler code should look at the instruction
        ; directly:
        LDMFD   r13!,{a4-v6}        ; recover most of the stacked registers
        ; The stack now contains {r10,r11,r12,r14}:
        LDR     r14,[r13,#(3 * 4)]  ; load out of the stack
        SUB     r14,r14,#4          ; modify to reference the SWI instruction
        STR     r14,[r13,#(3 * 4)]  ; and put back into the stack
        ; The stack now matches that required for the following
        ; standardised vector catching code:

        ; Even though we KNOW that this is an unknown C-Demon SWI,
        ; there may be a routine attached to the SWIV that can handle
        ; it. However we still check to see if the SWI should be
        ; caught before calling any attached handlers, even though we
        ; know the system SWI routine has been called:

        stopCTRLC r10,r11           ; ensure no user interruptions

        LDR     r10,=CaughtVectors  ; reference CaughtVectors location
        LDR     r10,[r10,#0x00]     ; get current bitmask
        TST     r10,#(1 :SHL: (0x08 :SHR: 2))
        BNE     raiseUnknownSWI
        ; If EQ then check if a handler has been attached:
        callHandler (1 :SHL: (0x08 :SHR: 2))
raiseUnknownSWI
        callCaughtVector    RDIError_SoftwareInterrupt

        ; ---------------------------------------------------------------------

UndefHandler
        ; This is the simplest of the abort handlers, since it is
        ; called before the instruction has been executed there is no
        ; state to unwind:
        SUB     r14,r14,#4          ; reference aborting instruction
        STMFD   r13!,{r10-r12,r14}  ; stack some work registers

        stopCTRLC r10,r11           ; ensure no user interruptions

        LDR     r10,=CaughtVectors  ; reference CaughtVectors location
        LDR     r10,[r10,#0x00]     ; get current bitmask
        TST     r10,#(1 :SHL: (0x04 :SHR: 2))
        BNE     raiseUndefAbort
        ; if EQ then check if a handler has been attached:
        callHandler (1 :SHL: (0x04 :SHR: 2))

raiseUndefAbort
        ; If we reach here then the vector was to be passed to the
        ; host debugger, or there was no handler, or the handler
        ; returned:
        callCaughtVector    RDIError_UndefinedInstruction

        ; ---------------------------------------------------------------------

PrefetchAbortHandler
        ; This is another simple abort handler, with only the PC to
        ; fixup:
        SUB     r14,r14,#4          ; reference aborting instruction
        STMFD   r13!,{r10-r12,r14}  ; stack some work registers

        stopCTRLC r10,r11           ; ensure no user interruptions

        [       (RUBY2 :LAND: HWBREAKPOINT)
        ; This may be an address match, in which case we should check
        ; if we need to treat the abort as a normal breakpoint.
        ; NOTE: The address match may ALSO generate data aborts if
        ; the code attempts to perform a data read from the address
        ; match location (only reads because we explicitly limit the
        ; address match to avoid writes).
        ;
        MOV     r10,#IOBase             ; get hardware address
        LDRB    r11,[r10,#DEBUG]        ; get current debug status
        ; The first job is to ensure that the hardware
        ; single-stepping is disabled.
        BIC     r12,r11,#DEBUG_SINGLE_STEP
        STRB    r12,[r10,#DEBUG]        ; explicitly disable HW single step
        ;
        TST     r11,#2_00000011         ; check for our address read matches
        BEQ     PossibleSingleStep      ; NO - then continue abort checking
        ;
        ; and then we can check for an address match:
        LDR     r12,[r10,#ADDR_MATCH]
        TEQ     r12,r14                 ; check that addresses are the same
        BNE     PossibleSingleStep      ; NO - then continue abort checking
        ;
        ; The following code deals with entering the normal
        ; breakpoint code:
        IMPORT  BP_Hardware             ; the breakpoint handler entry point
        MOV     r10,#RDIError_BreakpointReached
        MOV     r11,#1                  ; single hardware breakpoint
        LDR     pc,=BP_Hardware         ; and simulate a breakpoint

PossibleSingleStep
        ; r10 = IOBase
        ; r11 = entry DEBUG state
        ; r12 = undefined
        TST     r11,#DEBUG_SINGLE_STEP  ; check we were single-stepping
        BEQ     NormalPrefetchAbort
        ;
        MRS     r10,SPSR                ; get aborted thread PSR
        AND     r10,r10,#ModeMask
        TEQ     r10,#ABORTmode          ; and check we were not an ABT thread
        BEQ     NormalPrefetchAbort
        ;
        ; We can now assume that this abort was the result of a single step.
        ;
        ; The RDI_SingleStep function has saved the orginal BP_CPSR
        ; in the BP_SS_CPSR location. The CPSR used has the processor
        ; interrupts explicitly disabled (to avoid every single-step
        ; operation ending up in an interrupt handler). The down-side
        ; is that when single-stepping, the user will see that the
        ; PSR is always returned with IRQ and FIQ disabled.
        ;
        ; However, we cannot simply replace the BP_SS_CPSR I and F
        ; bits, because the instruction we have executed may have
        ; enabled a source (which we can spot, and cope with easily),
        ; or may have disabled a source (which we cannot spot,
        ; without looking at the instruction type and *PREVIOUS*
        ; processor mode).
        ;
        ; TODO: We may want to restore the I and F bits into the
        ; aborted CPSR from the BP_SS_CPSR variable. However, we have
        ; to be aware of the caveats above.
        ;
        MOV     r10,#RDIError_BreakpointReached ; was RDIError_NoError
        MOV     r11,#SSTEPBREAK         ; single-step abort
        LDR     pc,=BP_Hardware         ; and simulate a breakpoint
        ;
NormalPrefetchAbort
        ; We continue here if the prefetch abort was not caused by an
        ; address match:
        ]       ; RUBY2 and HWBREAKPOINT

        LDR     r10,=CaughtVectors  ; reference CaughtVectors location
        LDR     r10,[r10,#0x00]     ; get current bitmask
        TST     r10,#(1 :SHL: (0x0C :SHR: 2))
        BNE     raisePreFetchAbort
        ; if EQ then check if a handler has been attached:
        callHandler (1 :SHL: (0x0C :SHR: 2))
raisePreFetchAbort
        ; If we reach here then the vector was to be passed to the
        ; host debugger, or there was no handler, or the handler
        ; returned:
        callCaughtVector    RDIError_PrefetchAbort

        ; ---------------------------------------------------------------------

DataAbortHandler
        ; This handler is complicated, because we need to fixup the
        ; state to that before the faulting instruction. This might
        ; involve base register writeback, etc.

        SUB     r13,r13,#(15 * 4)   ; allocate register dump space
        STMIA   r13,{r0-r7}         ; store some working registers
        ADD     r0,r13,#(8 * 4)     ; use r0 as register store base

        MRS     r6,SPSR
        MRS     r7,CPSR

        TST     r6,#0x0F            ; check for USR26 or USR32
        STMEQIA r0,{r8-r14}^        ; store remaining USR mode registers
        BEQ     DAH_continue

        ORR     r6,r6,#NoInt        ; ensure all interrupts are disabled
        MSR     CPSR,r6             ; over the store of priviliedged mode regs.

        STMIA   r0,{r8-r14}         ; we are now in the correct mode

        MSR     CPSR,r7             ; return to abort handler mode
DAH_continue
        stopCTRLC r10,r11           ; ensure no user interruptions

        LDR     r0,[r14,#-8]!       ; get aborted instruction
        ; The above also unwinds the pipeline increment on r14.

        MOV     r1,r13              ; reference dumped registers

        AND     r2,r0,#0x000F0000   ; Rn
        AND     r3,r0,#0x0F000000   ; opcode

        CMP     r3,#0x08000000
        CMPNE   r3,#0x09000000
        BEQ     DAH_fixup_LDM_or_STM

        CMP     r3,#0x0C000000
        BEQ     DAH_fixup_CPDT_post

        CMP     r3,#0x0D000000
        BEQ     DAH_fixedup

        TST     r0,#0x00200000      ; CPDT pre
        BEQ     DAH_fixedup

DAH_fixup_CPDT_post
        ; Unlike LDR/STR the writeback still happens on abort:
        CMP     r2,#0x000F0000
        BEQ     DAH_fixedup         ; base = PC
        AND     r3,r0,#0xFF
        B       DAH_update_register

DAH_fixup_LDM_or_STM
        TST     r0,#0x00200000
        CMPNE   r2,#0x000F0000
        BEQ     DAH_fixedup         ; no WB or base = PC

        MOV     r4,r0,LSL #16
        MOVS    r4,r4,LSR #16       ; register bitmask
        MOV     r3,#0               ; count
        BEQ     DAH_fixup_LDM_or_STM_complete
DAH_regloop
        ADD     r3,r3,#1            ; increment register count
        RSB     r5,r4,#0
        AND     r5,r4,r5
        EORS    r4,r5,r4
        BNE     DAH_regloop
        ;
DAH_fixup_LDM_or_STM_complete
        CMP     r3,#0x00
        MOVEQ   r3,#0x10            ; register count
        ; Fall through to:
DAH_update_register
        ; r0 = instruction
        TST     r0,#0x00800000
        LDR     r0,[r1,r2,LSR #(16 - 2)]
        ADDEQ   r0,r0,r3,ASL #2
        SUBNE   r0,r0,r3,ASL #2
        STR     r0,[r1,r2,LSR #(16 - 2)]
        ; The base is now correct in the register dump.
DAH_fixedup
        ANDS    r0,r6,#0x0F         ; check for USR mode
        LDMEQFD r1,{r0-r14}^        ; recover USR-banked registers
        BEQ     DAH_callhandler

        MSR     CPSR,r6             ; interrupts disabled from above
        LDMIA   r1,{r0-r14}         ; recover banked registers
        MSR     CPSR,r7             ; and back to abort mode
DAH_callhandler
        ADD     r13,r13,#(15 * 4)   ; and restore stack
        ; The registers are now as they were when we were entered,
        ; apart from the r14 has been modified to reference the
        ; aborted instruction and the register causing the abort has
        ; been unwound if writeback had occurred:

        STMFD   r13!,{r10-r12,r14}  ; stack some work registers

        [       (RUBY2 :LAND: HWBREAKPOINT)
        ; We need to check if the data abort handler has been entered
        ; as the result of a read from an address where an address match
        ; breakpoint has been set. In reality we would want to treat such
        ; an address match as a NOP, and just turn off the address match;
        ; execute the instruction; and continue. However, we only have a
        ; single address match location and the work involved in allowing
        ; the read operation to execute is too complicated for now. This
        ; code just raises a data abort event as normal.
        ]       ; (RUBY2 and HWBREAKPOINT)

        ; The FIQ interrupt has been stopped above.

        LDR     r10,=CaughtVectors  ; reference CaughtVectors location
        LDR     r10,[r10,#0x00]     ; get current bitmask
        TST     r10,#(1 :SHL: (0x10 :SHR: 2))
        BNE     raiseDataAbort
        ; if EQ then check if a handler has been attached:
        callHandler (1 :SHL: (0x10 :SHR: 2))
raiseDataAbort
        ; If we reach here then the vector was to be passed to the
        ; host debugger, or there was no handler, or the handler
        ; returned:
        callCaughtVector    RDIError_DataAbort

        ; ---------------------------------------------------------------------

IgnoreVEC
        ; This is used within the standard vector routines to ignore
        ; the actual vector. NOTE: If any ARM hardware vectors are
        ; routed here then undefined results will occur, since this
        ; code does not perform any special exception handling. It is
        ; advised to *ALWAYS* route the ARM hardware vectors to
        ; specialised routines for each indivual event.
        MOVS    pc,lr           ; return (restoring PSR)

        ; ---------------------------------------------------------------------
        END     ; EOF handler.s
