        TTL     C-Demon lo-level startup code                        > level0.s
        ; ---------------------------------------------------------------------
        ; This file provides the C-Demon startup code. It deals with
        ; initialising any memory systems, required I/O and the RDP
        ; communications channel.
        ;
        ; $Revision: 0.1 $
        ;   $Author: jsmith $
        ;     $Date: 941031 $
        ;
        ; Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
        ; ---------------------------------------------------------------------
	; "@(#)level0.s	1.4	96/06/07 VLSI"

        INCLUDE global_h.s          ; include global memory, I/O, and variables
        INCLUDE rdi_h.s             ; RDI/RDP manifests
        INCLUDE $DRIVER./driver_h.s ; target architecture specific definitions

        ; ---------------------------------------------------------------------

CDEMONVSN       EQU     0x0238      ; BCD version number for the C-Demon world

        ; ---------------------------------------------------------------------
        ; External symbols required by this source file:

        IMPORT  main_prg    ; void main_prg(void) ;

        IMPORT  SWI_init    ; void SWI_init(void) ;
        IMPORT  BP_init     ; void BP_init(void) ;

        IMPORT  PONimage
        IMPORT  EndianWord
        IMPORT  RAMSize
        IMPORT  ROMEnd      ; end of C-Demon code (for CRC and MMU purposes)
        IMPORT  MMUtype

        IMPORT  ResetDriver ; const char *ResetDriver(void) ;
        IMPORT  PutByte     ; void PutByte(unsigned char byte) ;
        IMPORT  PrintString ; void PrintString(char *str) ;
        IMPORT  GetByte     ; unsigned char GetByte(void) ;
        IMPORT  AnyByte     ; int AnyByte(void) ;

        IMPORT  ReadTimer   ; unsigned int ReadTimer(void) ;
        IMPORT  SetTimer    ; void SetTimer(unsigned int tick) ;
        IMPORT  SetLEDs     ; void SetLEDs(unsigned char ledvalue) ;

        IMPORT  FindRAMSize ; non-APCS3 routine

        IMPORT  FPE_CopyAndInstall,WEAK

        ; ---------------------------------------------------------------------

        EXPORT  ROMReset    ; void ROMReset(void) ;
        ; This is the only symbol exported from this file. It is the
        ; main entry point into the C-Demon world from an ARM hardware
        ; reset. It can also be used as an emergency re-start when
        ; called directly from software control.

        ; ---------------------------------------------------------------------

        KEEP                ; preserve local (non-exported) symbols
        AREA |C-Demon|,CODE,READONLY

        ; ---------------------------------------------------------------------
        ; We are entered here to re-start the C-Demon system. This is
        ; either a result of an ARM power-on-reset,
        ; branch-through-zero, or system watchdog event. We treat the
        ; call to this function as a branch, and will *NOT* return to
        ; the caller.
        ;
        ; If this is a power-on-reset then we will be in SVC mode,
        ; with IRQ and FIQ disabled:
        ;
        ENTRY           ; Entry point to the C-Demon ROM world
ROMStart                ; Symbol for the first location in the ROM
ROMReset                ; our exported entry point
        ; Provide code to check processor mode on startup. There
        ; is a chance that we will be re-entered here on an error, or
        ; if the user has branched through zero. We should force
        ; ourselves into SVC mode, using a SWI if we are currently in
        ; a USR mode.
        ;
        ; We need to ensure that we get into SVC32 mode, with IRQs and
        ; FIQs disabled. The following code corrupts r14/lr.
        ;
        !   0,"ARM6x0 support only at the moment"
        ; On ARM6x0 processors the MMU control register is written as
        ; zero by reset. This means little-endian, early abort, 26bit
        ; data, 26bit program, write buffer off, cache off, alignment
        ; fault off and MMU off world. If we are on an ARM6x0 we need
        ; to configure the world to be 32bit address and data, plus
        ; little-endian operation. If we do not have the control MMU
        ; then we need to live with what we have been given.
        MRS     r14,CPSR                ; get the current processor mode
        TST     r14,#(1 :SHL: 4)        ; 26- or 32-bit mode?
        ; If bit (1 :SHL: 4) is NOT set then we are in a 26-bit mode.
        ; This code assumes that if we are started in 26-bit mode then
        ; we MUST have an MMU.
        MOVEQ   r14,#0x30               ; 32-bit address & data, little-endian
        MCREQ   MMUCP,0,r14,MMUControlReg,c0
        ;
        MRS     r14,CPSR
        ; We know that we are in either 26- or 32-bit SVC mode, so
        ; simply force the bits to explicitly select 32bit mode:
        BIC     r14,r14,#ModeMask
        ORR     r14,r14,#SVCmode
        MSR     CPSR,r14                ; and enter the 32-bit SVC mode
        ; We are now executing in a 32bit mode, with the MMU set
        ; suitably.
        ;
        STARTUPCODE
        ; v4 = PowerOnStatus (zero = hard; non-zero = soft)
        ; v5 = amount of RAM available, or 0x00000000 if not yet known.
        ;
        ; We are now executing in the real ROM location, with RAM
        ; mapped to 0x00000000.
        ;
        ; Perform a check to see if this ARM has a MMU co-processor:
        ADRL    a1,DummyMMUCode         ; reference our simple abort handler
        LDMIA   a1,{a2,a3}              ; load instructions
        MOV     a1,#0x04                ; address of undefined instruction handler
        STMIA   a1,{a2,a3}              ; store instructions from vector onwards
        ;
        ; Now that we have a simple undefined instruction handler in
        ; place, we can perform a MMU co-processor instruction:
        MRC     MMUCP,0,v3,MMUIDReg,c0  ; get the MMU ID register
        ; The above instruction will have aborted if there is no MMU,
        ; and our simple undefined instruction handler will have
        ; returned to this point.
        ;
        TEQ     v3,#0x00        ; ensure MMU and cache disabled, if present
        MOVNE   a1,#0x30               ; 32-bit address & data, little-endian
        MCRNE   MMUCP,0,a1,MMUControlReg,c0
        ; NOTE: This may have duplicated the 26bit RESET check above,
        ; but it doesn't really matter.
        ;
        ; We leave the ID register in v3 for the code to use after the
        ; RAM has been initialised.
        ;
        IMPORT  |Image$$RO$$Limit|  ; end of ROM code
        IMPORT  |Image$$RW$$Base|   ; final address of initialised RW
        IMPORT  |Image$$RW$$Limit|  ; can be used to size above
        ;
        ; NOTE: At the moment the linker places the BSS into the final
        ; ROM image. It also sets the Image$$RW$$Base and Limit values
        ; to cover the initialised and BSS data. This is not a problem
        ; at the moment, but may be an issue when a large ROM
        ; application, with a large BSS requirement needs to fit into
        ; a small ROM.
        ;
        ; Copy the data to the final RAM address:
        LDR     a2,=|Image$$RO$$Limit|  ; source address
        LDR     a3,=|Image$$RW$$Base|   ; destination address
        LDR     a4,=|Image$$RW$$Limit|  ; destination end address
        ;
        [       (RUBY2)
        ; NOTE: The RUBY2 has some PCMCIA attribute memory mapped
        ; into the low RAM address space. The following is a simple
        ; solution to avoiding writing to those special RAM
        ; locations, whilst keeping the simple (single) data area
        ; scheme for building the ROM.
        ;
        IMPORT  RUBY2_CCR_Start
        IMPORT  RUBY2_CCR_End
        ;
        LDR             v1,=RUBY2_CCR_Start             ; start of RAM CCR
        LDR             v2,=RUBY2_CCR_End               ; end of RAM CCR
        ;
01      LDR     a1,[a2],#4              ; get from ROM
        STR     a1,[a3],#4              ; store to RAM
        CMP     a3,v1                   ; check for end condition
        BLO     %BT01                   ; if (a3 < v1) we have more to transfer
        ;
        ADD             a2,a2,#AttrSize                 ; step over the ROM data space
        ;
02      LDR     a1,[a2],#4              ; get from ROM
        STR     a1,[v2],#4              ; store to RAM
        CMP     v2,a4                   ; check for end condition
        BLO     %BT02                   ; if (v2 < a4) we have more to transfer
        ;
        |       ; else normal RAM systems
01      LDR     a1,[a2],#4              ; get from ROM
        STR     a1,[a3],#4              ; store to RAM
        CMP     a3,a4                   ; check for end condition
        BLO     %BT01                   ; if (a3 < a4) we have more to transfer
        ]       ; (RUBY2)
        ; NOTE: We could optimise the loop above to use more
        ; registers. This would improve the initialisation latency of
        ; ROM systems.
        ; "a2" is the ROM end address.
        ; We have just initialised the RAM, so we can now use those variables:
        LDR     a1,=ROMEnd
        STR     a2,[a1,#0x00]

        ; The MMU identification word is held in v3 from above. A
        ; value of zero is used when no MMU is present.
        LDR     a1,=MMUtype
        STR     v3,[a1,#0x00]

        ; ---------------------------------------------------------------------

        [   (RUBY2)
        B       ContinueROMStartup      ; branch over RUBY2 CIS space
        LTORG                       ; force literals to be added here
        ;
        ; This code is part of the AREA forced to start at offset in the ROM.
        ;
        ; The StarRuby2 board has part of the PCMCIA interface mapped
        ; over the ROM. The CIS structure is addressable from:
        ;   (ROMBase + CISBase)
        ; upto:
        ;   (ROMBase + CISBase + CISSize).
        ;
        ; Since we know this code will be placed at the start of the
        ; ROM we can force the space for the ROM CIS.
        ;
        ASSERT  (({PC} - ROMStart) <= CISBase)  ; Too much entry code
        ; If the above ASSERT fails, then this complete conditional block
        ; should be moved to earlier in the startup code. This just ensures
        ; that the ROM CIS information appears at the correct offset in the
        ; ROM image.
        ;
        !   0,"NOTE: 0x" :CC: (:STR: (CISBase - ({PC} - ROMStart))) :CC: " bytes of padding to reach PCMCIA ROM CIS"
        %       ((CISBase - ({PC} - ROMStart))) ; pad upto CISBase
        ;
romCIS
        INCLUDE $DRIVER./romcis.s ; target specific definitions
        ;
        ; To keep things tidy we pad up to the end of the ROM CIS
        ; allocation. This is not necessary if the CIS data is
        ; terminated, since we could then just continue normal ROM
        ; data from this point. However, ensuring there is a CIS size
        ; hole in the ROM code/data means that the CIS could be poked
        ; into the final ROM image if required, rather than being
        ; built at assembly time.
        %       (CISSize - ({PC} - romCIS)) ; more padding
        ;
ContinueROMStartup
        ; After ROM CIS allocation:
        ]   ; RUBY2

        ; ---------------------------------------------------------------------
        ; Initialise the stack pointers. We do this by switching into
        ; all the other modes (ensuring interrupts stay disabled),
        ; setting up the default stack top addresses:

        LDR     sp,=RAMBase
        ADD     sp,sp,#SVCStackBase     ; supervisor mode stack

        MOV     v1,#(NoInt | FIQmode)
        MSR     CPSR,v1
        LDR     sp,=RAMBase
        ADD     sp,sp,#FIQStackBase     ; FIQ mode stack

        MOV     v1,#(NoInt | IRQmode)
        MSR     CPSR,v1
        LDR     sp,=RAMBase
        ADD     sp,sp,#IRQStackBase     ; IRQ mode stack

        MOV     v1,#(NoInt | UNDEFmode)
        MSR     CPSR,v1
        LDR     sp,=RAMBase
        ADD     sp,sp,#UndefStackBase   ; UND mode stack

        MOV     v1,#(NoInt | ABORTmode)
        MSR     CPSR,v1
        LDR     sp,=RAMBase
        ADD     sp,sp,#AbortStackBase   ; ABT mode stack

        MOV     v1,#(NoInt | SVCmode)
        MSR     CPSR,v1
        ; We are now back in SVC mode with IRQs and FIQs still
        ; disabled.
        ;
        ; Before we call any APCS conforming functions we need to
        ; ensure that the register world is suitably defined:
        MOV     fp,#0x00000000  ; no frame-pointer chain
        ; NOTE: This code assumes that the APCS routines called from
        ; this code are constructed without software stack-limit
        ; checking. If software stack-limit checking has been enabled
        ; then suitable "sl" register initialisation is required.
        ;
        MOV     a1,#2_0001              ; ROM reset complete
        BL      SetLEDs
        ;
        ; The memory size may already have been calculated as part of
        ; one of the earlier startup sequences:
        TEQ     v5,#0x00000000  ; check if we have the RAM size
        MOVNE   a1,v5           ; we already know the RAM size
        BLEQ    FindRAMSize     ; not known, so size the RAM
        LDR     v1,=RAMSize     ; global variable
        STR     a1,[v1,#0x00]   ; store RAM size
        ; We must ensure that the memory is sized *BEFORE* the cache
        ; (if present) is enabled.
        ;
        LDR     a1,=MMUtype     ; reference holding variable
        LDR     a1,[a1,#0x00]   ; and load the MMU id register
        TEQ     a1,#0x00        ; a zero value indicates no MMU present
        BEQ     noMMUinit       ; so we can ignore the MMU initialisation code
        ;
        INITMMU   a1            ; perform MMU initialisation if required
noMMUinit
        ;
        INITTIMER a1,v1         ; and any special timer initialisation
        ;
        ; For systems that cannot tell the type of reset that
        ; occurred, the STARTUPCODE will have set v4 to pretend that
        ; it is a PowerOnReset:
        LDR     v1,=PONimage    ; global variable
        STR     v4,[v1,#0x00]   ; store PowerOnReset status
        ;
        BL      ResetDriver     ; initialise the RDP communications channel
        MOV     v3,a1           ; system identification string
        ;
        MOV     a1,#0           ; set soft timer to initial value (0)
        BL      SetTimer
        ;
        ; We only let the timer interrupt run when there is an
        ; application running. When C-Demon (RDP) has control we do
        ; not allow any interrupts, so that we do not lose state if
        ; debugging an application level interrupt handler, and also
        ; so the application does not get confused by
        ; multiple-interrupts happening whilst its foreground is not
        ; executing. This means we cannot use interrupts to introduce
        ; pauses into the startup sequence. If we needed to perform
        ; some form of real-time delay we should simply perform a
        ; polling loop checking the timer ticks.
        ;       
        ; ---------------------------------------------------------------------
        ; Initialise the (armsd assumed) command-line buffer:
        MOV     a1,#0x00
        LDR     a2,=CommandLine ; address of fixed CommandLine buffer
        STR     a1,[a2,#0x00]   ; ensure a NULL string for startup
        ;
        ; ---------------------------------------------------------------------
        ;
        MOV     a1,#2_0010      ; About to start ROM checksum calculation
        BL      SetLEDs

        MOV     v4,#ROMBase     ; checksum the ROM from its base to its end
        LDR     v5,=ROMEnd      ; get address of ROMEnd variable
        LDR     v5,[v5,#0x00]   ; load the ROM end address

        MOV     v1,#0           ; checksum initial value
CheckSumLoop
        LDR     v6,[v4],#4      ; get next word to calc checksum with
        MOV     a1,v6           ; checksum is calculated a byte at a time
        BL      CheckSum
        MOV     a1,v6,LSR #8
        BL      CheckSum
        MOV     a1,v6,LSR #16
        BL      CheckSum
        MOV     a1,v6,LSR #24
        BL      CheckSum
        CMP     v4,v5                   ; see if at last address
        BNE     CheckSumLoop            ; loop until whole ROM has been counted
        ;
        LDR     a1,[v4,#0x00]           ; get the stored checksum value
        BIC     a1,a1,#0x00FF0000       ; convert from xxxxcdab to xxxxabcd
        BIC     a1,a1,#0xFF000000       ; also mask upper
        ORR     a1,a1,a1,ROR #16        ; xx00cdab -> yyabcdab
        BIC     a1,a1,#0xFF000000       ; -> 00abcdab
        MOV     v5,a1,LSR #8            ; -> 0000abcd
        ;
        MOV     v4,v1,LSR #16           ; get calculated checksum into v4
        ; v4 = calculated checksum
        ; v5 = checksum stored in ROM
        ; These registers are preserved over the following code to be
        ; used when transmitting the C-Demon banner information.
        
        ; ---------------------------------------------------------------------
        ; NOTE: The reason that a large number of 7F's were sent is to
        ; try and reset the target end in-case the protocol broke down
        ; in the middle of a large message. Unfortunately it is
        ; possible that a message larger than 256bytes was being sent
        ; when the protocol failed. What is really needed is a fixed
        ; RDP protocol that copes with re-starting at any transmission
        ; point, by either end of the communication channel.
        ;
        [   {TRUE}              ; At the moment we only send two:
        MOV     a1,#RDP_Reset
        BL      PutByte
        MOV     a1,#RDP_Reset
        BL      PutByte
        |                       ; old 256 byte transmission:
        MOV     v2,#256         ; number of bytes to send
        MOV     v1,#RDP_Reset   ; protocol reset value
01      MOV     a1,v1           ; copy into argument register
        BL      PutByte         ; transmit the byte
        SUBS    v2,v2,#1        ; decrement count
        BNE     %BT01           ; go around the loop again
        ]
        ;
        ; We now wait until the host debugger is ready to communicate:
TryAgain
        MOV     a1,#2_0100      ; Checksum done - attempting host communication
        BL      SetLEDs
        ;
        BL      AnyByte         ; is there a byte waiting for us?
        ORRS    a1,a1,a1        ; if 0 then no byte waiting
        BEQ     NoByte
        MOV     a1,#2_1000      ; Send prompt out communications channel
        BL      SetLEDs
WasteByte
        BL      GetByte         ; throw away the byte
        BL      AnyByte         ; any more?
        ORRS    a1,a1,a1        ; if 0 then no byte waiting
        BNE     WasteByte       ; otherwise get rid of it
        B       TryAgain        ; and loop around...
        ;
NoByte  ; We are now ready to start communicating with the host
        ; debugger. The first thing we do is transmit the banner:
        ADR     a1,ARMBanner
        BL      PrintString
        ;
        ; It is possible to do a lot of the processor identification
        ; in software. Unfortunately some ARM variants (e.g. '650) do
        ; not have any easily spotted difference.
        MOV     a1,#ARMVSN      ; depends on target architecture
        BL      PrintWord

        ADR     a1,DemonBanner
        BL      PrintString

        ; The "v3" register should be preserved from the ResetDriver
        ; call above. It is used to pass a system specific description
        ; string to this banner printing code, without the need for
        ; lots of target specific conditionals in this source.
        MOVS    a1,v3           ; print the system description string
        BLNE    PrintString     ; if it is non-NULL

        ADR     a1,CopyrightBanner
        BL      PrintString

        LDR     v1,=RAMSize
        LDR     a1,[v1,#0x00]   ; get RAM size
        BL      PrintHexWord    ; output the memory size

        ; We don't bother printing out the checksum if it is invalid,
        ; since we can always examine the ROM during a post-mortem to
        ; find out why the checksum failed:
        CMP     v4,v5           ; see if calculated checksum matches ROM
        ADRNE   a1,CRCFailBanner
        ADREQ   a1,CRCOKBanner
        BL      PrintString

        MOV     a1,#0xFF
        LDR     v1,=EndianWord
        STR     a1,[v1,#0x00]
        LDRB    a1,[v1,#0x00]
        CMP     a1,#0xFF
        LDREQ   a1,=LitEndBanner
        LDRNE   a1,=BigEndBanner
        BL      PrintString

        LDR     a1,=FPE_CopyAndInstall  ; has the FPE been installed
        CMP     a1,#0x00                ; if LABEL is 0 then not installed
        BEQ     EndBanner               ; if no FPE, skip install and message

        BL      FPE_CopyAndInstall      ; install the FPE

        ADR     a1,FPBanner             ; and add FPE to Banner
        BL      PrintString
EndBanner
        MOV     a1,#'.'                 ; terminate the string
        BL      PutByte
        MOV     a1,#0x0A                ; newline
        BL      PutByte

        ; ---------------------------------------------------------------------

        MOV     a1,#2_0101              ; Performing level 1 initialisation
        BL      SetLEDs

        BL      SWI_init                ; initialise SWI module
        BL      BP_init                 ; initialise BreakPoint module

        ; We can now notify the host debugger that this target is
        ; ready to accept RDP communications:
        MOV     a1,#0x00                ; special notification byte
        BL      PutByte

        MOV     a1,#2_0110              ; Start byte has been sent
        BL      SetLEDs

        ; Since we use polling to implement the C-Demon RDP
        ; communications we need to ensure that the host does not
        ; cause an overflow whilst we are not listening. Ideally
        ; RTS/CTS flow control should be used, though at the moment
        ; there is no guarantee that the host "armsd" will provide
        ; such support. The 0x00 byte we have just sent tells the host
        ; system to start sending its normal RDP requests. The current
        ; "armsd" program will transmit *ALL* of the bytes at
        ; once. The GetByte routine provided as part of the target
        ; specific support should perform RTS control to limit the
        ; transmissions of the host program.
        ;
        ; We are still in SVC mode (with IRQs and FIQs enabled) at
        ; this point. Original the assembler Demon would simply
        ; perform a branch-to-self, waiting for an interrupt started
        ; RDP communication. This has been changed to use a foreground
        ; polling loop. This is reasonable since the target foreground
        ; is *NOT* executing when C-Demon has control. This is to
        ; ensure that the host debugger context is valid at all
        ; times. With this limitation it is actually faster to poll
        ; the communications, rather than performing interrupt
        ; handling and foreground notification.
        ;
        ; Perform the polling in a priviledged mode other than SVC
        ; means that we should not suffer problems if the RDP
        ; processing code needs to perform SWIs etc.
        MRS     a1,CPSR             ; get current PSR
        BIC     a1,a1,#ModeMask     ; clear mode flags
        ORR     a1,a1,#ABORTmode    ; ABORT mode
        MSR     CPSR,a1             ; with interrupts disabled
        B       main_prg            ; standard RDP polling routine

        ; ---------------------------------------------------------------------
        ; The following code fragment is the simple undefined
        ; instruction handler we use when checking for the presence of
        ; a MMU co-processor.
DummyMMUCode
        MOV     v3,#0x00            ; no MMU present if we have aborted
        MOVS    pc,lr               ; return to just after faulting instruction

        ; ---------------------------------------------------------------------
        ; ASCII messages sent to the host debugger:

ARMBanner
        DCB     "ARM",0x00
DemonBanner
        DCB     " " :CC: "$BOARD" :CC: " C-Demon v"
        [       ((((:STR: CDEMONVSN) :RIGHT: 4) :LEFT: 1) <> "0")
        DCB     (((:STR: CDEMONVSN) :RIGHT: 4) :LEFT: 1)
        ]
	[	(SMC91C92)
        DCB     (((:STR: CDEMONVSN) :RIGHT: 3) :LEFT: 1) :CC: "." :CC: ((:STR: CDEMONVSN) :RIGHT: 2),"E",0x00
	|
        DCB     (((:STR: CDEMONVSN) :RIGHT: 3) :LEFT: 1) :CC: "." :CC: ((:STR: CDEMONVSN) :RIGHT: 2),0x00
	]
CopyrightBanner
        DCB     ", Copyright (c) 1994 VLSI.",0x0A,0x00
CRCOKBanner
        DCB     " bytes RAM, ROM CRC OK",0x00
CRCFailBanner
        DCB     " bytes RAM, ROM CRC Fail",0x00
LitEndBanner
        DCB     ", little endian",0x00
BigEndBanner
        DCB     ", BIG ENDIAN",0x00
FPBanner
        DCB     ", FPE",0x00
        ALIGN

        ; ---------------------------------------------------------------------
        ; -- Level0 support code ----------------------------------------------
        ; ---------------------------------------------------------------------
        ; void PrintHexWord(unsigned int value) ;
        ; Send the given value as a 8-nybble hex string with the
        ; standard hexadecimal prefix.
PrintHexWord
        STMFD   sp!,{v1,v2,v3,lr}
        MOV     v1,a1           ; the desired value
        ADRL    a1,HexPrefix    ; standard prefix string
        BL      PrintString     ; send to the debugger host
        B       PrintWord2

        ; ---------------------------------------------------------------------
        ; void PrintWord2(unsigned int value) ;
        ; Send the given value as a 8-nybble hex string.
PrintWord
        STMFD   sp!,{v1,v2,v3,lr}
        MOV     v1,a1           ; value to be output
PrintWord2
        ADRL    v3,HexString    ; ASCII characters
        MOV     v2,#(32 - 4)    ; start with leftmost nibble
01      MOV     a1,v1,LSR v2    ; shift nybble down
        ANDS    a1,a1,#0x0F     ; and mask
        BNE     %F02            ; the first non-zero
        SUBS    v2,v2,#4        ; step onto next nybble
        BNE     %B01            ; all but the last digit
        AND     a1,v1,#0x0F     ; get the last digit
02      LDRB    a1,[v3,a1]      ; load the required character
        BL      PutByte         ; and send to the debugger
        SUBS    v2,v2,#4        ; step onto next nybble
        MOV     a1,v1,LSR v2    ; shift correct nybble down
        AND     a1,a1,#0x0F     ; just the nybble we are interested in
        BPL     %B02            ; loop around
        LDMIA   sp!,{v1,v2,v3,pc}

HexString
        DCB     "0123456789ABCDEF"  ; no need for terminator
HexPrefix
        DCB     "0x",0x00
        ALIGN

        ; ---------------------------------------------------------------------
        ; CheckSum
        ;   in:  a1 = byte to calculate new CRC with
        ;        v1 = current CRC value (top 16bits)
        ;   out: v1 = updated CRC value
        ; NOTE: This function DOES not conform to the APCS-3 specification.
CheckSum
        STMDB   sp!,{v2-v4,lr}
        LDR     v2,=0x80050000  ; poly = 0x80050000
        MOV     v3,#8           ; bit = 8
01      MOV     v4,a1,LSL #24   ; local = ch << 24
        EORS    v4,v4,v1        ; local = crc ^ (ch << 24)
        MOV     v1,v1,LSL #1    ; crc <<= 1
        MOV     a1,a1,LSL #1    ; ch  <<= 1
        EORMI   v1,v1,v2        ; if (local < 0) crc ^= poly
        SUBS    v3,v3,#1        ; bit--
        BNE     %BT01           ; for (bit = 8; (bit != 0); bit--)
        LDMIA   sp!,{v2-v4,pc}  ; return to caller

        ; ---------------------------------------------------------------------

        EXPORT  branchaddr
branchaddr
        ; in: a1 = function address
        MOV     pc,a1

        ; ---------------------------------------------------------------------
        END     ; EOF level0.s
