/*> mmu.c <*/
/* This file provides the interface and support code for the standard
 * ARM6x0 MMU.
 *
 * The code expects a default MMU table to have been appended to the
 * ROM image, and addressable via the ROMEnd symbol. The "makemmu.c"
 * program deals with constructing and adding the necessary tables.
 *
 * NOTE: The current VY86PID version of C-Demon does *NOT* require
 * this file, since the MMU initialisation is performed within the
 * INITMMU macro provided in "vy86pid/driver_h.s".
 *
 * $Revision: 0.1 $
 *   $Author: jsmith $
 *     $Date: 941103 $
 *
 * Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.  */
/*---------------------------------------------------------------------------*/

#include    "global.h"
#include    "mmu.h"

/*---------------------------------------------------------------------------*/
/* BannerMMU
 * =========
 *
 * ENTER:   void
 * EXIT:    void
 */
void BannerMMU(void)
{
 if ((MMUtype & 0xFFFF) >= 0x100) /* only if recognized MMU */
  {
   PrintString("\nMMU enabled, Cache enabled, Write buffer enabled.\0") ;
  }
 return ;
}

/*---------------------------------------------------------------------------*/
/* MMU_init
 * ========
 *
 * ENTER:
 *
 * EXIT:
 *
 * Set up the ARM6x0 MMU with a simple flat page table
 */
void MMU_init(void)
{
 int MMUstart ;
 
 putMMU(MMUFlushTLBReg,0) ;              /* Flush the TLB */
 putMMU(MMUFlushIDCReg,0) ;              /* Flush the cache */
 putMMU(MMUDomainAccessReg,0x55555555) ; /* set domains 0 through 15 to CLIENT */

#if 1 /* new ROMEnd variable (rather than address symbol) */
 MMUstart = (int)ROMEnd ; /* get address of end of program */
#else
 MMUstart  = (int)&ROMEnd ; /* get addr of end of program */
#endif
 MMUstart &= ~0x3FFF ; /* mask */
 MMUstart += 0x4000 ;  /* now have start of MMU table */
    
 putMMU(MMUPageTableBaseReg,MMUstart) ; /* First level page table address */
 putMMU(MMUControlReg,EnableMMU32) ;

 MMUtype = getMMU(MMUIDReg) ; /* set global variable to MMU's ID */
 return ;
}

/*---------------------------------------------------------------------------*/
/*> mmu.c <*/
