/*> rdi.c <*/
/* This file provides the basic RDP handling for the C-Demon world.
 *
 * $Revision: 0.1 $
 *   $Author: jsmith $
 *     $Date: 941030 $
 *
 * Original author: Geary Chopoff.
 *
 * Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
 *
 * "@(#)rdi.c	1.3	96/06/07 VLSI"
 */
/*---------------------------------------------------------------------------*/

#include "global.h" /* general system manifests */
#include "driver.h" /* target architecture description */
#include "rdi.h"    /* RDI protocol definitions */
#include "bp.h"     /* C-Demon breakpoint support */
#include "swi.h"    /* structure parameter return definitions */

/*---------------------------------------------------------------------------*/
/* RDIUnknown
 * ==========
 *
 * ENTER:       num = invalid RDI request
 *
 * EXIT:        void
 */
static void RDIUnknown(int num)
{
 /* Do nothing for the moment */

	return ;
}

/*---------------------------------------------------------------------------*/
/* RDIUnimplemented
 * ================
 *
 * ENTER:       num = valid but unimplemented RDI request
 */
static void RDIUnimplemented(int num)
{
 PutByte(RDP_Return) ;
 PutByte(RDIError_UnimplementedMessage) ;
 return ;
}

/*---------------------------------------------------------------------------*/
/* RDIOpen
 * =======
 *
 * ENTER:       void
 *
 * EXIT:        void
 */
static void RDIOpen(void)
{
 unsigned char  ub ;
 unsigned char  ub2 ;
 unsigned char  c ;
 unsigned char *cp ;
 int            memsz ;

#ifdef SMC91C92
	if (use_enet) {
		/*
		 * Ethernet connection has attempted.
		 * Set the destination address and flag.
		 */
		bcopy(Src_Addr, &daddr[0], 6);
	}
	rdp_opened = 1;
#endif

 ub = GetByte() ; /* get type of OPEN from HOST */
 memsz = GetWord() ; /* get the initial memory size from HOST */

 if ((ub & RDPOpen_ResetLine) != 0) /* reset channel? */
  ub2 = GetByte() ; /* get the line speed */

 PutByte(RDP_Return) ;
        
 EndianWord = RDPOpen_SexBig ;
 cp = (unsigned char *)&EndianWord ;
 c  = *cp ;
        
 if ((ub & RDPOpen_SexDontCare) == 0)
  {
   if ((ub & RDPOpen_SexBig) == c)
    PutByte(RDIError_WrongByteSex);
   else
    PutByte(RDIError_NoError);
  }
 else /* WhatSex */
  {
   if (c == RDPOpen_SexBig)
    PutByte(RDIError_LittleEndian) ;
   else
    PutByte(RDIError_BigEndian) ;
  }

 if ((ub & RDPOpen_ResetLine) != 0) /* reset channel? */
  DriverSpeed(ub2) ; /* change the baud rate to ub2 */

 return ;
}

/*---------------------------------------------------------------------------*/
/* RDIClose
 * ========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDIClose(void)
{
	PutByte(RDP_Return) ;
	PutByte(RDIError_NoError) ;

#ifdef SMC91C92
	/*
	 * Reset flag.
	 */
	use_serial = 1;
	use_enet = 1;
	rdp_opened = 0;
#endif

#if 0 /* removed */
	DriverSpeed(1) ; /* change the baud rate to 9600 */
#endif

	return ;
}

/*---------------------------------------------------------------------------*/
/* RDIRead
 * =======
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDIRead(void)
{
 unsigned char *addr ;
 unsigned int   len ;
 unsigned int   j ;
  
 addr = (unsigned char *)GetWord() ; /* get address from HOST */
 len = GetWord() ; /* get length from HOST */

 PutByte(RDP_Return) ; /* ACK the HOST */

#if 0 /* support for aborts not yet included */
 ExpectedAbort = RDIRead_Fail ; /* set failure vector to RDIRead_Fail */
#endif

#ifdef VY86C650
 if (addr < (unsigned char *)0x04000000)
  {
   for (j=0; (j < len); j++)
    PutByte(*addr++);
  }
 else
  {
   for (j=0; (j < len); j++)
    {
     uip = (unsigned int *)addr ;
     uip = (unsigned int *)((int)uip & ~3) ;
     ui = *uip ;
     PutByte(ui >> (((int)addr & 3) ^ 3) * 8) ;
     addr++ ;
    }
  }
#else
 for (j=0; (j < len); j++)
  PutByte(*addr++) ;
#endif
        
#if 0 /* support for aborts not yet included */
 ExpectedAbort = 0 ; /* clear failure vector */
#endif

 PutByte(RDIError_NoError) ; /* OK the HOST */
 return ;
}

/* This is the original assembler Demon source for dealing with memory
 * aborts during read operations:
 */
/*
RDIRead_Fail
        SUB     v4,v4,v3        ;failure entry, calc remaining bytes
        MOV     a1,#0

RDIRead_FailLoop
        BL      PutByte         ;if FAIL write remaining bytes as 0x00's
        SUBS    v3,v3,#1
        BNE     RDIRead_FailLoop

        MOV     a1,#RDIError_DataAbort
        BL      PutByte         ;notify caller of the error
        MOV     a1,v4           ;and number of bytes we had left to send
        BL      PutWord

        MOV     v1,#VB1         ;place variable base addr in v1
        MOV     a1,#0           ;clear away failure vector
        STR     a1,[v1,#ExpectedAbort-VB1]
*/

/*---------------------------------------------------------------------------*/
/* RDIWrite
 * ========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDIWrite(void)
{
 unsigned char *addr ;
 unsigned int   len ;
 unsigned int   j ;
 
 addr = (unsigned char *)GetWord() ; /* get address from HOST */
 len = GetWord() ; /* get length from HOST */
 
#if 0 /* support for aborts not yet included */
 ExpectedAbort = RDIWrite_Fail ; /* set failure vector to RDIWrite_Fail */
#endif

#if defined(VY86C650)
 if (addr < (unsigned char *)0x04000000)
  {
   for (j=0; (j < len); j++)
    *addr++ = GetByte() ;
  }
 else
  {
   for (j=0; (j < len); j++)
    {
     uc = GetByte() ;
     uip = (unsigned int *)addr ;
     uip = (unsigned int *)((int)uip & ~3) ;
     ui = *uip ;
     ls = ((((int)addr & 3) ^ 3) * 8) ;
     ui = (ui & ~(0xFF << ls)) ;
     ui = (ui | (uc << ls)) ;
     *uip = ui ;
     addr++ ;
    }
  }
#else
#if defined(RUBY2)
 /* Care must be taken if the memory is *NOT* byte addressable. Byte
  * writes to non-byte addressable memory will result in certain
  * locations being overwritten. i.e. 32bit addressable memory will
  * have location 0 corrupted by byte writes to locations 1, 2 or 3.
  *
  * To handle this the following code attempts to write the memory as
  * complete 32bit words. NOTE: There are two build-time options in
  * this code. The first will perform word reads as necessary to
  * construct any non-word aligned start and end of transfer
  * bytes. The alternate version just simply writes the bytes (with
  * the possible result being corruption of lower address bytes as
  * described above).
  */
#define BYTEWRITE (0) /* 1 - to perform byte writes; 0 to perform word reads, and word writes */
 {
  unsigned int start_dribble = 0 ;
  unsigned int end_dribble ;
 
  /* Calculate number of (non-word) bytes at the start of the transfer */
  start_dribble = ((4 -((unsigned int)addr & 3)) & 3) ;
 
  end_dribble = ((len - start_dribble) & (4 - 1)) ;
 
  /* Deal with non-word data at the start of the transfer */
#if (BYTEWRITE == 1) /* perform byte transfers */
  while (start_dribble != 0)
   {
    *addr++ = GetByte() ;
    start_dribble-- ;
    len-- ;
   }  
#else /* perform word transfers */
  if (start_dribble != 0)
   {
    unsigned int *taddr = (unsigned int *)((unsigned int)addr & ~(4 - 1)) ;
    unsigned int  tdata ;
      
    tdata = *taddr ;
 
    len -= start_dribble ;
 
    while (start_dribble != 0)
     {
      /* At the start the last byte goes into byte offset 3 */
      tdata &= ~(0xFF << (8 * (((unsigned int)addr & 3)))) ;
      tdata |= ((GetByte() & 0xFF) << (8 * (((unsigned int)addr & 3)))) ;
      start_dribble-- ;
      addr++;
     }
      
    *taddr = tdata ;
   }
#endif
 
  /* Address and length should now be on 32bit word boundaries */
  while (len > 3)
   {
    *(unsigned int *)addr = GetWord() ;
    addr += 4 ;
    len -= 4 ;
   }  
      
  /* Deal with non-word data at the end of the transfer */
#if (BYTEWRITE == 1) /* perform byte transfers */
  while (end_dribble != 0)
   {
    *addr++ = GetByte() ;
    end_dribble-- ;
   }  
#else /* perform word transfers */
  if (end_dribble != 0)
   {
    unsigned int *taddr = (unsigned int *)((unsigned int)addr & ~(4 - 1)) ;
    unsigned int tdata ;
      
    while (end_dribble != 0)
     {
      /* At the end the 1st byte goes into byte offset 0 */
      tdata &= ~(0xFF << (8 * ((unsigned int)addr & 3))) ;
      tdata |= ((GetByte() & 0xFF) << (8 * ((unsigned int)addr & 3))) ;
      end_dribble-- ;
      addr++;
     }
      
    *taddr = tdata ;
   }
#endif
 }
 j = j ; /* to stop compiler warning */
#else /* normal byte-addressable memory */
 for(j=0; (j < len); j++)
  *addr++ = GetByte() ;
#endif /* RUBY2 */
#endif /* VY86C650 */
 
#if 0 /* support for aborts not yet included */
 ExpectedAbort = 0 ; /* clear failure vector */
#endif
 
 PutByte(RDP_Return) ; /* ACK the HOST */
 PutByte(RDIError_NoError) ; /* OK the HOST */
 return ;
}

/* This is the original assembler Demon source for dealing with memory
 * aborts during write operations:
 */
/*
RDIWrite_Fail
        SUB     v4,v4,v3        ;failure entry, calc remaining bytes
RDIWrite_FailLoop
        BL      GetByte         ;get a byte and bit bucket it (throw it away)
        SUBS    v3,v3,#1        ;any more?
        BNE     RDIWrite_FailLoop       ;loop until all bytes gotten
        MOV     a1,#RDIError_DataAbort  ;tell caller of failure
        BL      PutByte
        MOV     a1,v4           ;and how many bytes did not get written
        BL      PutWord

        MOV     v1,#VB1         ;place variable base addr in v1
        MOV     a1,#0           ;clear away failure vector
        STR     a1,[v1,#ExpectedAbort-VB1]
*/

/*---------------------------------------------------------------------------*/
/* Support routine to actually perform the "intelligence" in selecting
 * the correct register to be used depending on the processor mode and
 * bitmask. A simple boolean is used to control whether the registers
 * are being read or written.
 */
static void moveReg(unsigned char mode,unsigned int mask,int write)
{
 unsigned int  regbit ;
 unsigned int *uip ;

 regbit = 0 ;
 while (mask != 0)
  {
   if (mask & (1 << regbit))
    {
     switch (regbit)
      {
       case  0 : case  1 : case  2 : case  3 :
       case  4 : case  5 : case  6 : case  7 :
       case  8 : case  9 : case 10 : case 11 :
       case 12 : case 13 : case 14 :
        uip = (unsigned int *)((int)&BP_r0) ;
        uip += regbit ; /* reference the correct USR register */
        switch (mode)
         {
          case 0x11 : /* FIQ */
           if (regbit > 8)
            {
             uip = (unsigned int *)((int)&BP_FIQ_r8) ;
             uip += (regbit - 8) ;
            }
           /* else use the USR register */
           break ;

          case 0x12 : /* IRQ */
           switch (regbit)
            {
             case 13 :
              uip = (unsigned int *)((int)&BP_IRQ_sp) ;
              break ;

             case 14 :
              uip = (unsigned int *)((int)&BP_IRQ_lr) ;
              break ;

             default : /* use the USR register */
              break ;
            }
           break ;

          case 0x13 : /* SVC */
           switch (regbit)
            {
             case 13 :
              uip = (unsigned int *)((int)&BP_SVC_sp) ;
              break ;

             case 14 :
              uip = (unsigned int *)((int)&BP_SVC_lr) ;
              break ;

             default : /* use the USR register */
              break ;
            }
           break ;

          case 0x17 : /* Abort */
           switch (regbit)
            {
             case 13 :
              uip = (unsigned int *)((int)&BP_ABT_sp) ;
              break ;

             case 14 :
              uip = (unsigned int *)((int)&BP_ABT_lr) ;
              break ;

             default : /* use the USR register */
              break ;
            }
           break ;

          case 0x1B : /* Undefined */
           switch (regbit)
            {
             case 13 :
              uip = (unsigned int *)((int)&BP_UND_sp) ;
              break ;

             case 14 :
              uip = (unsigned int *)((int)&BP_UND_lr) ;
              break ;

             default : /* use the USR register */
              break ;
            }
           break ;

          default : /* return USR register */
           break ;
         }
        if (write == TRUE)
         *uip = GetWord() ;
        else
         PutWord(*uip) ;
        break ;

       case 15 : /* raw PC */
       case 16 : /* PC (without PSR state in 26bit modes) */
        uip = (unsigned int *)((int)&BP_pc) ;
        if (write == TRUE)
         *uip = GetWord() ;
        else
         PutWord(*uip) ;
        break ;

       case 17 : /* CPSR */
        uip = (unsigned int *)((int)&BP_CPSR) ;
        if (write == TRUE)
         *uip = GetWord() ;
        else
         PutWord(*uip) ;
        break ;

       case 18 : /* SPSR */
        switch (mode)
         {
          case 0x11 : /* FIQ */
           uip = (unsigned int *)((int)&BP_FIQ_SPSR) ;
           break ;

          case 0x12 : /* IRQ */
           uip = (unsigned int *)((int)&BP_IRQ_SPSR) ;
           break ;

          case 0x13 : /* SVC */
           uip = (unsigned int *)((int)&BP_SVC_SPSR) ;
           break ;

          case 0x17 : /* Abort */
           uip = (unsigned int *)((int)&BP_ABT_SPSR) ;
           break ;

          case 0x1B : /* Undefined */
           uip = (unsigned int *)((int)&BP_UND_SPSR) ;
           break ;

          default : /* use the CPSR */
           uip = (unsigned int *)((int)&BP_CPSR) ;
           break ;
         }
        if (write == TRUE)
         *uip = GetWord() ;
        else
         PutWord(*uip) ;
        break ;

       default :
        /* Do nothing for unrecognised requests */
        break ;
      }

     mask &= ~(1 << regbit) ;
    }

   regbit++ ; /* next register bit */
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/* RDICPURead
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 *
 * The RDP message contains a mode number (standard ARM mode number or
 * 255 for the current mode) and a bitmask of the registers to be
 * returned.
 *
 * The bitmask bits 0..14 return the required register.
 * Bit 15 returns the PC, with the PSR flags if a 26bit mode.
 * Bit 16 returns the PC, without the PSR flags if a 26bit mode.
 * Bit 17 returns the CPSR.
 * Bit 18 returns the SPSR, for non-USR 32bit modes, CPSR for USR modes.
 *
 * At the moment this version of C-Demon does not support any 26bit
 * modes, so if the mode is a 26bit one then the corresponding 32bit
 * mode registers are returned. For this to be fixed would require
 * some assembler stubs to interrogate the required processor mode
 * registers, or for the breakpoint code to be extended to store *ALL*
 * of the ARM registers into the ZP memory.
 */
static void RDICPURead(void)
{
 unsigned char mode ; /* 255 = current, otherwise is CPU mode (User,SVC,etc.) */
 unsigned int  mask ; /* register mask of registers to return */

 mode = GetByte() ; /* get mode to read registers from */
 mask = GetWord() ; /* get bit mask of which registers to return */
        
 PutByte(RDP_Return) ; /* ACK the HOST */

 if (mode == 255)
  {
   unsigned int *uip ;
   /* We want to interrogate the current mode: */
   uip = (unsigned int *)((int)&BP_CPSR) ;
   mode = (*uip & 0x1F) ; /* get current mode number */
  }

#if 1 /* Cheat for the moment and pretend 26bit modes are the 32bit ones */
 /* See the comments in the function header about 26bit mode support */
 mode |= 0x10 ; /* turn 26bit mode number into 32bit mode number */
#endif

 moveReg(mode,mask,FALSE) ;

 PutByte(RDIError_NoError) ; /* OK the HOST */
 return ;
}

/*---------------------------------------------------------------------------*/
/* RDICPUWrite
 * ===========
 *
 * ENTER:       void
 * EXIT:        void
 *
 * See the comments attached to RDICPURead() above.
 */
static void RDICPUWrite(void)
{
 unsigned char mode ; /* 255 = current, otherwise is CPU mode (User,SVC,etc.) */
 unsigned int  mask ; /* register mask of registers to return */

 mode = GetByte() ; /* get mode to write registers to */
 mask = GetWord() ; /* get bit mask of which registers to set */

 if (mode == 255)
  {
   unsigned int *uip ;
   /* We want to interrogate the current mode: */
   uip = (unsigned int *)((int)&BP_CPSR) ;
   mode = (*uip & 0x1F) ; /* get current mode number */
  }

#if 1 /* Cheat for the moment and pretend 26bit modes are the 32bit ones */
 /* See the comments in the function header about 26bit mode support */
 mode |= 0x10 ; /* turn 26bit mode number into 32bit mode number */
#endif

 moveReg(mode,mask,TRUE) ;

 PutByte(RDP_Return) ;       /* ACK the HOST */
 PutByte(RDIError_NoError) ; /* O.K. the HOST */
 return ;
}

/*---------------------------------------------------------------------------*/
/* FindFPE
 * =======
 *
 * ENTER:       void
 *
 * EXIT:        contents of the fpsr if there is an FPE
 *                      -1 if no FPE
 */
static int FindFPE(void)
{
 int fp_sts ;
 unsigned char copro_num ;
 
 copro_num = GetByte() ; /* get co-pro number from HOST */
        
 if ((copro_num == 1) || (copro_num == 2))
  fp_sts = ReadFPstatus() ; /* read FP status into fp_sts */
 else
  fp_sts = -1 ;

 return(fp_sts) ;
}

/*---------------------------------------------------------------------------*/
/* RDICPRead
 * =========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDICPRead(void)
{
 int fp_sts ;
 unsigned int mask ;
 unsigned int bit_tst ;
 int j ;
 ret3parm rp ;

 fp_sts = FindFPE() ; /* get FP status */

 mask = GetWord() ; /* get FP register mask */

 PutByte(RDP_Return) ; /* ACK the HOST */

 bit_tst = (mask & 0xFF) ; /* mask to check registers */

 if (fp_sts == -1)
  {
   j = 0 ;
   while (bit_tst != 0)
    {
     if ((bit_tst & 1) == 1)
      {
       PutWord(0) ; /* send ZEROES */
       PutWord(0) ;
       PutWord(0) ;
       PutWord(0xa0000000) ; /* Extended format */
      }
     j++ ;
     bit_tst = (bit_tst >> 1) ;
    }
   if (mask & 0x00000100)
    PutWord(fp_sts) ; /* FPSR */

   if (mask & 0x00000200)
    PutWord(0) ; /* FPCR */

   PutByte(RDIError_BadCoProState) ; /* tell HOST status (we are done)*/
  }
 else
  {
   WriteFPstatus(0) ; /* clear exception masks (else storing a trapping NAN will cause trouble) */
   j = 0 ;
   while (bit_tst != 0)
    {
     if ((bit_tst & 1) == 1)
      {
       rp = ReadFPregister(j) ;
       PutWord(rp.p1) ;
       PutWord(rp.p2) ;
       PutWord(rp.p3) ;
       PutWord(0xa0000000) ; /* Extended format */
      }
     j++ ;
     bit_tst = (bit_tst >> 1) ;
    }
   WriteFPstatus(fp_sts) ; /* restore FP status */
                        
   if (mask & 0x00000100)
    PutWord(fp_sts) ; /* FPSR */

   if (mask & 0x00000200)
    PutWord(0) ; /* FPCR */

   PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/* RDICPWrite
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDICPWrite(void)
{
 int fp_sts ;
 unsigned int mask ;
 unsigned int bit_tst ;
 int j ;
 unsigned int p1 ;
 unsigned int p2 ;
 unsigned int p3 ;
 
 fp_sts = FindFPE() ; /* get FP status */

 mask = GetWord() ; /* get FP register mask */

 PutByte(RDP_Return) ; /* ACK the HOST */

 bit_tst = (mask & 0xFF) ; /* mask to check registers */

 if (fp_sts == -1)
  {
   j = 0 ;
   while (bit_tst != 0)
    {
     if ((bit_tst & 1) == 1)
      {
       (void)GetWord() ; /* get to the bit-bucket */
       (void)GetWord() ;
       (void)GetWord() ;
       (void)GetWord() ; /* Extended format */
      }
     j++ ;
     bit_tst = (bit_tst >> 1) ;
    }

   if (mask & 0x00000100)
    (void)GetWord() ; /* FPSR */

   if (mask & 0x00000200)
    (void)GetWord() ; /* FPCR */

   PutByte(RDIError_BadCoProState) ; /* tell HOST status (we are done) */
  }
 else
  {
   WriteFPstatus(0) ; /* clear exception masks (else storing a trapping NAN will cause trouble) */
   j = 0 ;
   while (bit_tst != 0)
    {
     if ((bit_tst & 1) == 1)
      {
       p1 = GetWord() ;
       p2 = GetWord() ;
       p3 = GetWord() ;
       (void)GetWord() ; /* Extended format to bit-bucket */
       WriteFPregister(j,p1,p2,p3) ;
      }
     j++ ;
     bit_tst = (bit_tst >> 1) ;
    }

   if (mask & 0x00000100)
    fp_sts = GetWord() ; /* FPSR */

   WriteFPstatus(fp_sts) ; /* restore (or set) FP status */
                
   if (mask & 0x00000200)
    (void)GetWord() ; /* FPCR to the bit-bucket */

   PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/* RDISetBreak
 * ===========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDISetBreak(void)
{
 int addr ;
 int offset ;
 unsigned char typ ;
 unsigned char tell_HOST ;
 
 addr = GetWord() ; /* place the address to set breakpoint in addr */
 typ  = GetByte() ; /* place type of breakpoint in typ */

 PutByte(RDP_Return) ; /* ACK the HOST */

 if (typ == 0) /* is it type ZERO ? */
  {
   tell_HOST = RDIError_NoError ; /* Assume we will set breakpoint */
        
   if (!BP_Exist(addr)) /* does the breakpoint NOT exist? */
    {
     offset = BP_Offset(0) ; /* get the next available breakpoint */
     if (offset == -1) /* is a slot available ? */
      tell_HOST = RDIError_NoMorePoints ; /* No, so tell HOST */
     else
      BP_Set(offset,addr) ;     /* otherwise set the breakpoint */
    }
  }
 else
  tell_HOST = RDIError_UnimplementedType ; /* if NOT, tell HOST */

 PutByte(tell_HOST) ; /* send HOST status and that we are done */
 return ;
}

/*---------------------------------------------------------------------------*/
/* RDIClearBreak
 * =============
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDIClearBreak(void)
{
 int addr ;
 int offset ;
 
 addr = GetWord() ; /* get breakpoint addr to clear */
 PutByte(RDP_Return) ; /* ACK the HOST */
        
 offset = BP_Offset(addr) ;

 if (offset == -1) /* do we have an offset? */
  PutByte(RDIError_NoSuchPoint) ; /* if NOT then tell HOST */
 else
  {
   BP_Clear(offset) ; /* otherwise clear the breakpoint */
   PutByte(RDIError_NoError) ; /* and O.K. the HOST */
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/* RDIExecute
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 *
 * This routine starts the routine pointed at by DeBuggeePC
 * it is stopped or exited via SWIBreak or SWIExit (see swi.s)
 *
 */
#if 1 /* 950407 */
extern void branchaddr(void (*fn)(void)) ;
#endif

static void RDIExecute(void)
{
 BP_type = NORMAL ; /* set default exit type to NORMAL */
 DebuggeeRunning = GetByte() ; /* store type of execute (SYNC or ASYNC) */
 /* If requested to execute asynchronously, then return status immediately */
 if (DebuggeeRunning & RDP_ExecAsync)
  {
   PutByte(RDP_Return) ;
   PutByte(RDIError_NoError) ;
  }

#if 1 /* 950407 */
 /* It was noticed that the ARM Ltd linker would not complain even if
  * the destination address for a symbol is further than 26bits away.
  * The following is a hack to ensure that the actual symbol address
  * is loaded into the PC.
  */
 branchaddr(RDI_Go) ;
#else
 RDI_Go() ;
#endif

 return ; /* never reached */
}

/*---------------------------------------------------------------------------*/
/* InfoTarget
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void InfoTarget(void)
{
 const char IDString[] = SYSID ; /* Must be exactly 4 bytes */

 PutByte(RDP_Return) ; /* ACK the HOST */
 PutWord(0x17) ; /* send 23 to HOST (why?) */
 PutWord((int)IDString); /* send ID string to HOST as word */
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */

 return ;
}

/*---------------------------------------------------------------------------*/
/* InfoPoints
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void InfoPoints(void)
{
 PutByte(RDP_Return) ; /* ACK the HOST */
 PutWord(0) ; /* send 0 to HOST */
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* InfoStep
 * ========
 *
 * This function is used to return information about the
 * single-stepping capabilities of this target.
 *
 * ENTER:       void
 * EXIT:        void
 */
static void InfoStep(void)
{
 PutByte(RDP_Return) ; /* ACK the HOST */
#if defined(RUBY2) && defined(HWBREAKPOINT)
 PutWord(1 << 2) ; /* we support hardware single-stepping */
#else
 PutWord(0) ; /* send 0 to HOST */
#endif /* RUBY2 and HWBREAKPOINT */
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* InfoErrorP
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void InfoErrorP(void)
{
 PutByte(RDP_Return) ; /* ACK the HOST */
 PutWord(ErrorP) ; /* send ErrorP to HOST */
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* SignalStop
 * ==========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void SignalStop(void)
{
 /* Do nothing for the moment */
 return ;
}

/* The following is the original assembler Demon source for this
 * function. The C-Demon world operates differently, so this code has not
 * been required.
 */
/*
SignalStop
        MOV     v4,#VB1         ;place variable base addr in v4
        LDR     a1,[v4,#DebuggeeRunning-VB1]
        TEQ     a1,#0
        BEQ     SignalStop_NotRunning

        LDR     a1,[v4,#r14*4]
 ****STR        a1,[v4,#DebuggeePC-VB1]
        LDR     a1,[v4,#15*4]
 ****STR        a1,[v4,#DebuggeeCPSR-VB1]
        LDR     v1,[v4,#DebuggeeRunning-VB1]
        MOV     a1,#0
        STR     a1,[v4,#DebuggeeRunning-VB1]

        TST     v1,#RDP_ExecAsync
        MOVEQ   a1,#RDP_Return
        MOVNE   a1,#RDP_Stopped
        BL      PutByte

        MOV     a1,#RDIError_UserInterrupt
        BL      PutByte

        B       RDIInfoDone             ;return through a common exit

 * if we were stopped waiting for the response to a SWI,
 * the debugger will send its 'go' to resume from the SWI
 * before it gets the response to the stop.  So we must
 * arrange to ignore that 'go'.

SignalStop_NotRunning

        LDRB    a1,[v4,#SWIState-VB1]
        TST     a1,#SWIState_Active
        BEQ     %F01

        ORRNE   a1,a1,#SWIState_IgnoreGo
        STRB    a1,[v4,#SWIState-VB1]

        MOV     a1,#RDP_Return
        BL      PutByte

        MOV     a1,#RDIError_UserInterrupt
        BL      PutByte
01
        B       RDIInfoDone             ;return through a common exit
*/

/*---------------------------------------------------------------------------*/
/* VectorCatch
 * ===========
 *
 * ENTER:       void
 * EXIT:        void
 */
static void VectorCatch(void)
{
 CaughtVectors = GetWord() ; /* save which vectors to catch */
 PutByte(RDP_Return) ; /* ACK the HOST */
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* Cycles
 * ======
 *
 * ENTER:       void
 * EXIT:        void
 */
static void Cycles(void)
{
 PutByte(RDP_Return) ; /* ACK the HOST */
 /* send 6 ZEROES to HOST */
 PutWord(0) ;
 PutWord(0) ;
 PutWord(0) ;
 PutWord(0) ;
 PutWord(0) ;
 PutWord(0) ;
 PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* CmdLine
 * =======
 *
 * Get the command line from the HOST and place it in the CommandLine buffer
 *
 * ENTER:       void
 * EXIT:        void
 */
static void CmdLine(void)
{
 unsigned char *ubp ;
 
 PutByte(RDP_Return) ; /* ACK the HOST */
        
 ubp = (unsigned char *)&CommandLine;
 do
  {
   if (((int)ubp - (int)&CommandLine) >= 256)
    ubp = (unsigned char *)&CommandLine ; /* wrap the buffer */
   *ubp = GetByte() ; /* get byte from the HOST */
  } while (*ubp++ != NULL) ;

 PutByte(RDIError_NoError) ; /* O.K. the HOST (we are done) */
 return ;
}

/*---------------------------------------------------------------------------*/
/* RDIOSOpReply
 * ============
 *
 * ENTER:       void
 *
 * EXIT:        NULL (or void) if GetByte is NULL       ( 0 )
 *                      BYTE if GetByte is a_byte       ( 1 )
 *                      WORD otherwise          ( x )
 */
static int RDIOSOpReply(void)
{
#define a_byte (1) /* part of OS Request */

 unsigned char ub ;
 int rv ;

 ub = GetByte() ; /* get byte from HOST */

 switch(ub)
  {
   case NULL : /* if NULL then nothing to get from HOST */
    rv = NULL ;
    break ;
                        
   case a_byte :
    rv = (int)GetByte() ; /* return byte from HOST */
    break ;
                        
   default :
    rv = GetWord() ; /* return word from HOST */
  }

 return(rv) ;
}

/*---------------------------------------------------------------------------*/

#if defined(RUBY2) && defined(HWBREAKPOINT)
static void RDISingleStep(void)
{
 unsigned int ninstr ;
 char dostate ;

 BP_type = NORMAL ; /* in-case we hit a SWI Exit */
 
 /* NOTE: If bit7 of the "return" state is set then we need to return a
  * PointHandle.
  */
 dostate = GetByte() ; /* store type of execute (SYNC or ASYNC) */
 ninstr = (unsigned int)GetWord() ; /* and get the number of instructions */

 if (ninstr != 1)
  RDIUnimplemented(0x11) ; /* only allow single instructions to be stepped */
 else
  {
   DebuggeeRunning = dostate ;
   /* With the current stored CPU state, allow a single
    * instruction to be executed.
    */

   /* Respond immediately */
   if (DebuggeeRunning & RDP_ExecAsync)
    {
     PutByte(RDP_Return) ;
     PutByte(RDIError_NoError) ;
    }

   RDI_SingleStep() ; /* perform the necessary low-level work */
  }

 return ;
}
#endif /* RUBY2 and HWBREAKPOINT */

/*---------------------------------------------------------------------------*/
/* RDIInfo
 * =======
 *
 * ENTER:       void
 * EXIT:        void
 */
static void RDIInfo(void)
{
 int typ ;
 
 typ = GetWord() ; /* get type of info HOST wants */
        
 switch (typ)
  {
   case RDIInfo_Target :
    InfoTarget() ;
    break ;
                        
   case RDIInfo_Points :
    InfoPoints() ;
    break ;
                        
   case RDIInfo_Step :
    InfoStep() ;
    break ;
                        
   case RDISignal_Stop :
    SignalStop() ;
    break ;
                        
   case RDIVector_Catch :
    VectorCatch() ;
    break ;
                        
   case RDIInfo_Cycles :
    Cycles() ;
    break ;
                        
   case RDIInfo_ErrorP :
/*
        SUB     v1,a1,#RDIInfo_ErrorP :AND: :NOT: &ff

        CMP     v1,#RDIInfo_ErrorP :AND: &ff
        BEQ     InfoErrorP
*/
    InfoErrorP() ;
    break ;
                        
   case RDISet_Cmdline :
    CmdLine() ;
    break ;

   case RDIInfo_MMU : 
    {
     const char MMUID[] = SYSID ; /* Must be exactly 4bytes */ 
     PutByte(RDP_Return) ; /* ACK the HOST */
     PutWord((int)MMUID) ; /* return 4byte ID string */
     PutByte(RDIError_NoError) ; /* OK the HOST (we are done) */
    }
    break ;
                        
   default :
    PutByte(RDP_Return) ; /* ACK the HOST */
    PutByte(RDIError_UnimplementedMessage) ; /* tell HOST cannot do */
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/* Level1RDI
 * =========
 *
 * ENTER:       num = RDI function to invoke
 *
 * EXIT:                void
 *
 * This is the start of the RDI code. It is now called from the foreground.
 * Thereby lr should be saved if a call to a different module is made.
 */
void Level1RDI(int num)
{
 switch (num)
  {
   case 0x00 :
    RDIOpen() ;             /* 0x00 - Start Session */
    break ;

   case 0x01 :
    RDIClose() ;            /* 0x01 - End Session */
    break ;

   case 0x02 :
    RDIRead() ;             /* 0x02 - Read Memory Address */
    break ;

   case 0x03 :
    RDIWrite() ;            /* 0x03 - Write Memory Address */
    break ;

   case 0x04 :
    RDICPURead() ;          /* 0x04 - Read CPU state */
    break ;

   case 0x05 :
    RDICPUWrite() ;         /* 0x05 - Write CPU state */
    break ;

   case 0x06 :
    RDICPRead() ;           /* 0x06 - Read CoProcessor state */
    break;

   case 0x07 :
    RDICPWrite() ;          /* 0x07 - Write CoProcessor state */
    break ;

   case 0x0A :
    RDISetBreak() ;         /* 0x0A - Set a Break Point */
    break;

   case 0x0B :
    RDIClearBreak() ;       /* 0x0B - Clear a Break Point */
    break;

   case 0x0C :
    RDIUnimplemented(num) ; /* 0x0C - (Set Watch Point) */
    break;

   case 0x0D :
    RDIUnimplemented(num) ; /* 0x0D - (Clear Watch Point) */
    break ;

   case 0x10 :
    RDIExecute() ;          /* 0x10 - Execute */
    break;

   case 0x11 :
#if defined(RUBY2) && defined(HWBREAKPOINT)
    RDISingleStep() ;       /* 0x11 - Multiple Step */
#else
    RDIUnimplemented(num) ; /* 0x11 - (Multiple Step) */
#endif /* RUBY2 and HWBREAKPOINT */
    break ;

   case 0x12 :
    RDIInfo() ;             /* 0x12 - Miscellaneous Info */
    break ;

   case 0x13 :
    RDIOSOpReply() ;        /* 0x13 - reply to OS Op Request */
    break;
                
   default :
    RDIUnknown(num) ;       /* unknown RDI request */
    break ;
  }

 return ;
}

/*---------------------------------------------------------------------------*/
/*> EOF rdi.c <*/
