/*> rdi.h
 *
 * $Revision: 0.1 $
 *   $Author: jsmith $
 *     $Date: 941030 $
 *
 * Original author: Geary Chopoff.
 *
 * Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
 *
 * "@(#)rdi.h	1.2	96/06/07 VLSI"
 */
/*---------------------------------------------------------------------------*/

#ifndef __cdemon_rdi_h
#define __cdemon_rdi_h

/*---------------------------------------------------------------------------*/
/*
 * E r r o r   R e t u r n   V a l u e s
 */
#define RDIError_NoError                0   /* Good return */
#define RDIError_Reset                  1
#define RDIError_UndefinedInstruction   2
#define RDIError_SoftwareInterrupt      3
#define RDIError_PrefetchAbort          4
#define RDIError_DataAbort              5
#define RDIError_AddressException       6
#define RDIError_IRQ                    7
#define RDIError_FIQ                    8
#define RDIError_Error                  9
#define RDIError_BranchThrough0         10

#define RDIError_NotInitialised         128
#define RDIError_UnableToInitialise     129
#define RDIError_WrongByteSex           130
#define RDIError_UnableToTerminate      131
#define RDIError_BadInstruction         132
#define RDIError_IllegalInstruction     133
#define RDIError_BadCPUStateSetting     134
#define RDIError_UnknownCoPro           135
#define RDIError_UnknownCoProState      136
#define RDIError_BadCoProState          137
#define RDIError_BadPointType           138
#define RDIError_UnimplementedType      139
#define RDIError_BadPointSize           140
#define RDIError_UnimplementedSize      141
#define RDIError_NoMorePoints           142
#define RDIError_BreakpointReached      143
#define RDIError_WatchpointAccessed     144
#define RDIError_NoSuchPoint            145
#define RDIError_ProgramFinishedInStep  146
#define RDIError_UserInterrupt          147
#define RDIError_NoPCValue              148

#define RDIError_LittleEndian           240
#define RDIError_BigEndian              241

#define RDIError_InsufficientPrivilege  253
#define RDIError_UnimplementedMessage   254
#define RDIError_UndefinedMessage       255

/*
 * R D P    M e s s a g e   V a l u e s
 */
#define RDP_Open            0x00
#define RDP_Close           0x01
#define RDP_Read            0x02
#define RDP_Write           0x03
#define RDP_CPURead         0x04
#define RDP_CPUWrite        0x05
#define RDP_CPRead          0x06
#define RDP_CPWrite         0x07
#define RDP_SetBreak        0x0A
#define RDP_ClearBreak      0x0B
#define RDP_SetWatch        0x0C
#define RDP_ClearWatch      0x0D
#define RDP_Execute         0x10
#define RDP_ExecSync        0       /* sync  = send ACK when done */
#define RDP_ExecAsync       1       /* async = immediate ACK */
#define RDP_Step            0x11
#define RDP_Info            0x12
#define RDP_OSOpReply       0x13
#define RDP_LastRequest     0x13
#define RDP_Stopped         0x20
#define RDP_OSOp            0x21

#define RDP_Fatal           0x5E
#define RDP_Return          0x5F

#define RDP_Reset           0x7F

/*
 * M i s c e l l a n e o u s   R D I   V a l u e s
 */
#define RDISex_Little       0       /* the byte sex of the debuggee */
#define RDISex_Big          1
#define RDISex_DontCare     2

#define RDIPoint_EQ         0       /* the different types of break/watchpoints */
#define RDIPoint_GT         1
#define RDIPoint_GE         2
#define RDIPoint_LT         3
#define RDIPoint_LE         4

#define RDIPoint_IN         5
#define RDIPoint_OUT        6

#define RDIWatch_ByteRead   1       /* types of data accesses to watch for */
#define RDIWatch_HalfRead   2
#define RDIWatch_WordRead   4

#define RDIWatch_ByteWrite  8
#define RDIWatch_HalfWrite  16
#define RDIWatch_WordWrite  32

#define RDIReg_R15          (1 << 15)   /* mask values for CPU */
#define RDIReg_PC           (1 << 16)
#define RDIReg_CPSR         (1 << 17)
#define RDIReg_SPSR         (1 << 18)

#define RDINum_CPURegs      19
#define RDINum_CPRegs       10          /* current maximum */

#define RDIMode_Curr        255

#define RDIInfo_Target      0
#define RDIInfo_Points      1
#define RDIInfo_Step        2
#define RDIInfo_MMU         3

#define RDISignal_Stop      0x100
#define RDIVector_Catch     0x180
#define RDIInfo_Cycles      0x200
#define RDIInfo_CyclesSize  48

#define RDIInfo_ErrorP      0x201
#define RDISet_Cmdline      0x300

#define RDPOpen_Start       1
#define RDPOpen_ResetLine   2
#define RDPOpen_SexBig      4
#define RDPOpen_SexDontCare 8

/*---------------------------------------------------------------------------*/

#endif /* __cdemon_rdi_h */

/*---------------------------------------------------------------------------*/
/*> EOF rdi.h <*/
