        TTL     C-Demon vector handling                             > vectors.s
	; "@(#)vectors.s	1.2	96/06/07 VLSI"
        ; ---------------------------------------------------------------------
        ; This source file contains the data and code used to
        ; initialise the low (RAM) memory of the C-Demon system. The
        ; area between LowMemoryStart and LowMemoryEnd are copied from
        ; the ROM image to the base of RAM during the C-Demon
        ; initialisation. This area includes the ARM vectors, aswell
        ; as C-Demon workspace and breakpoint support routines.
        ;
        ; $Revision: 0.1 $
        ;   $Author: jsmith $
        ;     $Date: 941031 $
        ;
        ; Copyright (c) 1994, VLSI Technology Inc. All Rights Reserved.
        ; ---------------------------------------------------------------------

        INCLUDE global_h.s          ; include global memory, I/O, and variables
        INCLUDE $DRIVER./driver_h.s ; target specific definitions
        INCLUDE bp_h.s              ; break point support
        INCLUDE rdi_h.s             ; most breaks require to set BP_RDP_STS
        INCLUDE swis_h.s            ; support for SWI calls

        ; ---------------------------------------------------------------------
        ; Symbols required by this source file:

        IMPORT  ROMReset                ; 00 - Reset
        IMPORT  UndefHandler            ; 04 - Undefined instruction
        IMPORT  SWIHandler              ; 08 - SWI handler
        IMPORT  PrefetchAbortHandler    ; 0C - Abort inst handler
        IMPORT  DataAbortHandler        ; 10 - Abort data handler
                                        ; 14 - (reserved) AddrException handler
        IMPORT  IRQHandler              ; 18 - IRQ interrupt handler
        IMPORT  FIQHandler              ; 1C - FIQ interrupt handler
                                        ; 20 - Error handler

        IMPORT  IgnoreIRQ               ; standard IRQ ignoring handler
        IMPORT  IgnoreFIQ               ; standard FIQ ignoring handler
        IMPORT  IgnoreVEC               ; used when no handler exists
                                        ; for a system vector

        IMPORT  GetByte                 ; from driver.c
        IMPORT  PutByte                 ; from driver.c
        IMPORT  GetWord                 ; from driver.c
        IMPORT  PutWord                 ; from driver.c
        IMPORT  PrintString             ; from driver.c
        IMPORT  SetLEDs                 ; from driver.c

        ; ---------------------------------------------------------------------
        ; Symbols provided by this source file to the rest of the
        ; C-Demon system:

        EXPORT  ResetV  
        EXPORT  UndefInstrV     
        EXPORT  SWIV    
        EXPORT  PrefAbortV      
        EXPORT  DataAbortV      
        EXPORT  AddrExceptV     
        EXPORT  IRQV    
        EXPORT  FIQV    
        EXPORT  ErrorV

        EXPORT  GetByteV
        EXPORT  PutByteV
        EXPORT  GetWordV
        EXPORT  PutWordV
        EXPORT  PrintStringV
        EXPORT  SetLEDV

        EXPORT  IRQvectors
        EXPORT  FIQvectors

        ; ---------------------------------------------------------------------

        KEEP    ; local symbols

        ; ---------------------------------------------------------------------
        ; The following vectors *MUST* start at 0x00000000 (an ARM
        ; processor requirement), and must remain in sequence. The
        ; following vectors indirect through a RAM based "SoftVectors"
        ; table. This allows for new handlers to be attached easily,
        ; replacing those defined during the C-Demon initialisation.
        ;
        ; Normally some form of ROM is present at 0x00000000 during
        ; reset, with the startup code performing some target specific
        ; magic to map RAM to address zero (for the ARM hardware
        ; vectors).
        ;
        ; The following area should be placed at zero by the
        ; linker.

        AREA    |LowMemoryCode|,CODE    ; NOTE: Read/Write AREA
LowMemoryStart
        ; This is the area used to initialise the RAM.
VectorSTART
        ; First we have the ARM processor vectors (which should end up
        ; at zero):
        LDR     pc,ResetV       ; 00 - RESET
        LDR     pc,UndefInstrV  ; 04 - Undefined instruction
        LDR     pc,SWIV         ; 08 - SWI
        LDR     pc,PrefAbortV   ; 0C - Aborted instruction fetch
        LDR     pc,DataAbortV   ; 10 - Aborted data (memory) access
        LDR     pc,AddrExceptV  ; 14 - (reserved)
        LDR     pc,IRQV         ; 18 - IRQ interrupt
        LDR     pc,FIQV         ; 1C - FIQ interrupt
        ; Then we have some C-Demon system support vectors:
        LDR     pc,ErrorV       ; 20 - Error
        ;
        ; ---------------------------------------------------------------------

        [   (:LNOT: (RUBY2 :LAND: HWBREAKPOINT))
        ; Breakpoints are now handled with a "MOV pc,#BREAKxx"
        ; instruction (limiting the breakpoint support code to living
        ; at a 8bit shifted constant value). Currently a maximum of 99
        ; breakpoints are allowed, though the current default is less
        ; (at least 16 is recommended).
        ;
        ; The rest of the code expects these "vectoring" branches to
        ; start at 0x24:
        ASSERT  (({PC} - LowMemoryStart) = 0x24) ; check offset is correct

        ; Globals used when consrtructing breakpoint stubs:
        GBLA    BPv
        GBLS    BPs
        ;
        ; The BPVEC macro is used to generate the fixed address branch
        ; vectors.
        MACRO
        BPVEC   $loc
        B       $loc            ; 24+4*(xx-1) : Break Point #xx Vector
        MEND
        ;
BPv     SETA    1
        WHILE   (BPv <= NumBP)  ; loop creating the necessary vector structures
BPs     SETS    "BreakPt" :CC: (("0123456789" :LEFT: ((BPv / 10) + 1)) :RIGHT: 1) :CC: (("0123456789" :LEFT: ((BPv :MOD: 10) + 1)) :RIGHT: 1)
        BPVEC   $BPs
BPv     SETA    (BPv + 1)
        WEND
        ]   ; (NOT RUBY2 and HWBREAKPOINT)
        ;
        ; ---------------------------------------------------------------------
        ; The "SoftVectors" array is the main indirection scheme
        ; within the C-Demon code. The vectors are initially held in
        ; ROM, but are copied to RAM during the system
        ; initialisation. Having the vectors RAM based allows the
        ; handlers for certain system events to be dynamically changed
        ; at run-time.
SoftVectors
        ; The first set of vectors in the table are the standard ARM
        ; exception vectors. These are the table slots referenced by
        ; the ARM processor vector table:
                                              ; vecnum
ResetV          DCD     ROMReset              ; 0x40 00 - RESET
UndefInstrV     DCD     UndefHandler          ; 0x41 04 - Undefined instruction
SWIV            DCD     SWIHandler            ; 0x42 08 - SWI
PrefAbortV      DCD     PrefetchAbortHandler  ; 0x43 0C - Abort inst
DataAbortV      DCD     DataAbortHandler      ; 0x44 10 - Abort data
AddrExceptV     DCD     IgnoreVEC             ; 0x45 14 - (reserved)
IRQV            DCD     IRQHandler            ; 0x46 18 - IRQ interrupt
FIQV            DCD     FIQHandler            ; 0x47 1C - FIQ interrupt

        ; The following table slots are for C-Demon system support
        ; vectors.
                                              ; vecnum
ErrorV          DCD     IgnoreVEC             ; 0x48 20 - Error Handler
GetByteV        DCD     GetByte               ; 0x49 xx - Get Byte from HOST
PutByteV        DCD     PutByte               ; 0x4A xx - Put Byte to HOST
GetWordV        DCD     GetWord               ; 0x4B xx - Get WORD from HOST
PutWordV        DCD     PutWord               ; 0x4C xx - Put WORD to HOST
PrintStringV    DCD     PrintString           ; 0x4D xx - Print NULL terminated String to HOST
SetLEDV         DCD     SetLEDs               ; 0x4E xx - SetLED
                DCD     IgnoreVEC             ; 0x4F xx - (reserved)

        ; The contents and size of the following vector tables depend
        ; on the target architecture:
IRQvectors      IRQVECTORS
FIQvectors      FIQVECTORS

                LTORG   ; place any literals generated by the code above
VectorEND
        [       (RUBY2) ; leave space for RAM based CCR memory
        ASSERT  (({PC} - LowMemoryStart) <= AttrBase) ; ensure will fit in
        !       0,"NOTE: 0x" :CC: (:STR: (AttrBase - ({PC} - LowMemoryStart))) :CC: " bytes of padding to reach PCMCIA RAM CCR"
        %       (AttrBase - ({PC} - LowMemoryStart)) ; pad upto AttrBase
RUBY2_CCR_Start
        %       AttrSize                             ; zero attribute memory
RUBY2_CCR_End
                ;
                EXPORT  RUBY2_CCR_Start
                EXPORT  RUBY2_CCR_End
        ]       ; (RUBY2)

        ; ---------------------------------------------------------------------
        ; GLOBAL variables defined in the following C-Demon workspace:

        EXPORT  BP_RDP_STS
        EXPORT  BP_CPSR
        [       (RUBY2 :LAND: HWBREAKPOINT)
        EXPORT  BP_SS_CPSR
        ]       ; RUBY2 and HWBREAKPOINT
        EXPORT  BP_pc
        EXPORT  BP_r0
        EXPORT  BP_r1
        EXPORT  BP_r2
        EXPORT  BP_r3
        EXPORT  BP_r4
        EXPORT  BP_r5
        EXPORT  BP_r6
        EXPORT  BP_r7
        EXPORT  BP_r8
        EXPORT  BP_r9
        EXPORT  BP_r10
        EXPORT  BP_r11
        EXPORT  BP_r12
        EXPORT  BP_FIQ_r8
        EXPORT  BP_FIQ_r9
        EXPORT  BP_FIQ_r10
        EXPORT  BP_FIQ_r11
        EXPORT  BP_FIQ_r12
        EXPORT  BP_USR_sp
        EXPORT  BP_USR_lr
        EXPORT  BP_FIQ_sp
        EXPORT  BP_FIQ_lr
        EXPORT  BP_FIQ_SPSR
        EXPORT  BP_SVC_sp
        EXPORT  BP_SVC_lr
        EXPORT  BP_SVC_SPSR
        EXPORT  BP_ABT_sp
        EXPORT  BP_ABT_lr
        EXPORT  BP_ABT_SPSR
        EXPORT  BP_IRQ_sp
        EXPORT  BP_IRQ_lr
        EXPORT  BP_IRQ_SPSR
        EXPORT  BP_UND_sp
        EXPORT  BP_UND_lr
        EXPORT  BP_UND_SPSR
        EXPORT  BP_type

        EXPORT  BP_addr
        EXPORT  BP_inst

        EXPORT  BP_Stop         ; stop executing, save BP_ values
        EXPORT  BP_StopInt      ; stop executing from interrupt
        EXPORT  BP_CaughtVector ; stop executing after exception
        EXPORT  BP_Go           ; start executing with BP_ values
        EXPORT  RDI_Go          ; machine dependent part of RDIExecute
        [       (RUBY2 :LAND: HWBREAKPOINT)
        EXPORT  RDI_SingleStep  ; machine dependent part of RDISingleStep
        ]       ; RUBY2 and HWBREAKPOINT

        EXPORT      TickVal
        EXPORT      Handlers
        EXPORT      HandlersEnd
        EXPORT      CaughtVectors
        EXPORT      ErrorP
        EXPORT      ExpectedAbort
        EXPORT      TimeZero
        EXPORT      FPRegsAddr
        EXPORT      FPAbortPossible
        EXPORT      SWIState
        EXPORT      DebuggeeRunning
        EXPORT      DebugLR
        EXPORT      DebugSP
        EXPORT      DebugCPSR
        EXPORT      RAMBranchThrough0
        EXPORT      ResetVectorCopy
        EXPORT      EndianWord
        EXPORT      RAMSize
        EXPORT      ROMEnd
        EXPORT      RDIreply
        EXPORT      MMUtype
        EXPORT      LEDimage
        EXPORT      PONimage
        EXPORT      FP_image
        EXPORT      WorkingDump
        EXPORT      WorkingDumpEnd
        EXPORT      FPDumpEnd

        ; ---------------------------------------------------------------------

        [   (:LNOT: (RUBY2 :LAND: HWBREAKPOINT))
        ; BreakPtxx
        ; =========
        ; The following while loop constructs NumBP breakpoint
        ; vectors. These stubs are referenced from the fixed address
        ; entry points constructed earlier (from offset 0x24):
        ;
        ; The SAVEBP macro is used mainly to define a label, but code
        ; has been inserted there instead of the while loop for a more
        ; readable structure:
        MACRO
$lab    SAVEBP  $where,$value,$cnt
$lab.$cnt
        STR     r0,$where       ;need a free register to use
        MOV     r0,#$value
        B       Breaking        ;continue with code common to break points 1 to NumBP
        MEND
        ;
BPv     SETA    1
        ;
        WHILE   (BPv <= NumBP)
BPs     SETS    (("0123456789":LEFT:((BPv/10)+1)):RIGHT:1):CC:(("0123456789":LEFT:((BPv:MOD:10)+1)):RIGHT:1)
BreakPt SAVEBP  BP_r0,BPv,$BPs
BPv     SETA    BPv + 1
        WEND
 
        ; ---------------------------------------------------------------------
        ; Breaking
        ; ========
        ; This is the start of the common breakpoint handling code.
        ;
        ; in:   r0 = value fo BP_type (r0 already stored in BP_r0).
        ; out:  BP_RDP_STS is set to indicate that breakpoint was tripped.
        ;
Breaking
        STR     r0,BP_type      ; save which breakpoint caused break
        MOV     r0,#RDIError_BreakpointReached
        STRB    r0,BP_RDP_STS
        B       BP_Stop2        ; continue with common break point code
        ]   ; (NOT RUBY2 and HWBREAKPOINT)

        ; ---------------------------------------------------------------------

        [   (RUBY2 :LAND: HWBREAKPOINT)
        EXPORT  BP_Hardware
BP_Hardware
        ; We are entered here when a RUBY2 prefetch abort has been
        ; caused by an address match.
        ; in:   abort mode; IRQs disabled; FIQs undefined.
        ;       r10 = type of hardware breakpoint
        ;       r11 = breakpoint number (0=normal;1=address match;<0=special)
        ;       r12 = work register
        ;       r13 = FD stack containing original {r10-r12,r14}
        ;       r14 = work register
        ;
        STR     r0,BP_r0        ; preserve entry r0

        MRS     r0,CPSR         ; get current PSR status
        ORR     r0,r0,#NoInt    ; set all the interrupt flags
        MSR     CPSR,r0         ; and disable ALL interrupts

        STR     r11,BP_type     ; depends on the type of ABORT
        STRB    r10,BP_RDP_STS  ; store the type of breakpoint

        LDMIA   sp!,{r10-r12,r14}   ; recover work registers
        ; NOTE: The r14 has already been modified by the prefetch
        ; abort handler before calling this code.
        STR     r14,BP_pc       ; preserve "PC" for later
        MRS     r0,SPSR         ; get CPSR of aborted thread
        STR     r0,BP_CPSR      ; store for later
        B       BP_NOT_User     ; continue with the breakpoint code
        ]   ; RUBY2 and HWBREAKPOINT

        ; ---------------------------------------------------------------------

BP_CaughtVector
        ; We are entered here from the Demon abort vector handlers.
        ; in:   abort mode; IRQs disabled; FIQs undefined.
        ;       r10 = abort reason code
        ;       r11 = work register
        ;       r12 = work register
        ;       r13 = FD stack containing original {r10-r12,r14}
        ;       r14 = work register
        ;
        STR     r0,BP_r0        ; preserve entry r0

        MRS     r0,CPSR         ; get current PSR status
        ORR     r0,r0,#NoInt    ; set all the interrupt flags
        MSR     CPSR,r0         ; and disable ALL interrupts

        ; We have a special break identifier, so that the later code
        ; does not overwrite reg.
        MOV     r0,#ABORTBREAK
        STR     r0,BP_type

        ; Store the abort description value for later:
        STR     r10,BP_RDP_STS

        LDMIA   sp!,{r10-r12,r14}
        ;
        ; NOTE: The r14 has already been modified by the specific
        ; abort handler before calling this code.
        STR     r14,BP_pc       ; preserve "PC" for later
        MRS     r0,SPSR         ; get CPSR of aborted thread
        STR     r0,BP_CPSR      ; store for later
        B       BP_NOT_User     ; continue with the breakpoint code

        ; ---------------------------------------------------------------------

BP_StopInt
        ; We are entered here from the C-Demon interrupt handler when
        ; a message has been received whilst the target "OS" is
        ; executing. For the following code to work correctly, and for
        ; the correct information to be given to the host armsd, we
        ; need to pretend the interrupt did not happen, so that the
        ; user will simply see the interruption occurring within the
        ; active thread rather than always seeing the interruption
        ; occurring in the interrupt handler.
        ;
        ; in:   sp = FD stack containing {r0-r7,lr}
        ;       interrupt mode; with relevant interrupt disabled
        ;
        LDMIA   sp!,{r0}        ; recover entry r0
        STR     r0,BP_r0

        MRS     r0,CPSR         ; get current PSR status
        ; The BP_CPSR is actually set to the SPSR value later in this
        ; code, so we do not need to preserve the current PSR:
        ORR     r0,r0,#NoInt    ; set the relevant PSR flags
        MSR     CPSR,r0         ; and disable interrupts

        MOV     r0,#ASYNCBREAK  ; user interrupt break
        STR     r0,BP_type

        MOV     r0,#RDIError_UserInterrupt
        STRB    r0,BP_RDP_STS

        LDMIA   sp!,{r1-r7,lr}  ; restore r1 to r7 and lr
        ; The interrupt mode stack pointer is now back to its entry
        ; state. The only information that has been changed at this
        ; point is the interrupt mode r14 has been overwritten, and we
        ; have updated the CPSR to disable all interrupt sources.

        SUB     lr,lr,#4        ; modify lr to correct address
        STR     lr,BP_pc        ; and preserve for later

        ; We use the interrupt mode SPSR to pretend that the
        ; "breakpoint" occurred in the original thread:
        MRS     r0,SPSR         ; get CPSR of interrupted thread
        STR     r0,BP_CPSR      ; store for later

        B       BP_NOT_User     ; continue storing the thread state

        ; ---------------------------------------------------------------------
        ; BP_Stop
        ; =======
        ; Perform a soft breakpoint operation, saving the complete
        ; register state for later interrogation. The BP_type variable
        ; is updated with the number of the breakpoint that was tripped.
        ;
BP_Stop
        STR     r0,BP_r0        ; need a free register to use as a pointer
        MOV     r0,#0           ; mark normal break
        STR     r0,BP_type
        ; Fall through to BP_Stop2...
        ;
        ; BP_Stop2
        ; ========
        ; in:   r0 has been saved in BP_r0
        ;       BP_type has been set to identify which breakpoint was tripped
BP_Stop2
        MRS     r0,CPSR         ; determine the mode
        STR     r0,BP_CPSR      ; (save psr too)
        ORR     r0,r0,#NoInt    ; try to disable interrupts
        MSR     CPSR,r0         ; this will NOT work if it is USER mode
        AND     r0,r0,#ModeMask ; now check what mode
        CMP     r0,#USRmode     ; are we in USER mode?
        BNE     BP_NOT_User     ; handle that differently
        ;
        ; only way for USER to switch to another mode is by using a SWI:
        SWI     EnterOS         ; switch to SVC mode
        MOV     r0,#(SVCmode | NoInt)
        MSR     CPSR,r0         ; now interrupts are really disabled
        ;
        ; Both FIQ and IRQ interrupts are disabled. We are in some
        ; mode, but as long as it is NOT USER we can get at the state
        ; of the registers by switching modes.
BP_NOT_User
        ; Save common registers and USER registers (r13 and r14):
        ADR     r0,BP_r1        ; so use r0
        STMIA   r0,{r1-r14}^    ; to store the common and USER registers (r13 and r14)
        ;
        ; save FIQ registers:
        MOV     r0,#(FIQmode | NoInt)
        MSR     CPSR,r0         ; in FIQ mode
        ADR     r0,BP_FIQ_r8    ; point r0 at FIQ r8 to r12
        STMIA   r0,{r8-lr}      ; store FIQ registers r8 to r14
        MRS     r0,SPSR
        STR     r0,BP_FIQ_SPSR  ; save SPSR of FIQ mode
        ;
        ; save IRQ registers:
        MOV     r0,#(IRQmode | NoInt)
        MSR     CPSR,r0         ; in IRQ mode
        ADR     r0,BP_IRQ_sp    ; point r0 at IRQ sp
        STMIA   r0,{sp-lr}      ; store IRQ registers sp and lr
        MRS     r0,SPSR
        STR     r0,BP_IRQ_SPSR  ; save SPSR of IRQ mode
        ;
        ; save UNDefined registers:
        MOV     r0,#(UNDEFmode | NoInt)
        MSR     CPSR,r0         ; in UND mode
        ADR     r0,BP_UND_sp    ; point r0 at UND sp
        STMIA   r0,{sp-lr}      ; store UND registers sp and lr
        MRS     r0,SPSR
        STR     r0,BP_UND_SPSR  ; save SPSR of UND mode
        ;
        ; save ABT (Abort) registers:
        MOV     r0,#(ABORTmode | NoInt)
        MSR     CPSR,r0         ; in ABT mode
        ADR     r0,BP_ABT_sp    ; point r0 at ABT sp
        STMIA   r0,{sp-lr}      ; store ABT registers sp and lr
        MRS     r0,SPSR
        STR     r0,BP_ABT_SPSR  ; save SPSR of ABT mode
        ;
        ; save SVC (Supervisor) registers:
        MOV     r0,#(SVCmode | NoInt)
        MSR     CPSR,r0         ; in SVC mode
        ADR     r0,BP_SVC_sp    ; point r0 at SVC sp
        STMIA   r0,{sp-lr}      ; store SVC registers sp and lr
        MRS     r0,SPSR
        STR     r0,BP_SVC_SPSR  ; save SPSR of SVC mode
        ;
BP_Set_PC
        LDR     r0,BP_type      ; check type of breakpoint
        SUBS    r0,r0,#1        ; reduce by 1
        BMI     BP_Done         ; if normal, skip setting of BP_pc
        ;
        ADR     r1,BP_addr      ; place pointer to BP_addr array in r1
        LDR     r0,[r1,r0,LSL #2] ;get addr from BP_addr[r0]
        STR     r0,BP_pc        ; and set BP_pc with gotten addr
BP_Done
        LDR     a2,=DebugCPSR
        LDR     a1,[a2,#0x00]   ; PSR when target execution started
        ;
        ; NOTE: The SWI entry code performs any necessary Ctrl-C
        ; interrupt masking support and RTS flow control, so we do not
        ; need to do it here.
        ;
        ; This disables ALL the system interrupts whilst in the
        ; debugging handling code. This means that the RDP
        ; communications *MUST* be polled, otherwise the system will
        ; just hang.
        ORR     a1,a1,#NoInt    ; disable IRQ and FIQ
        STR     a1,[a2,#0x00]   ; still referencing DebugCPSR
        ;
        SWI     Exit            ; return to HOST control

        ; ---------------------------------------------------------------------

        [       (RUBY2 :LAND: HWBREAKPOINT)
        ; RDI_SingleStep
        ; ==============
        ; Execute the next instruction using the stored CPU state.
RDI_SingleStep
        ; in: no conditions
        ; out : no conditions - exits via abort breakpoint handler
        ;
        ; The following ensures that the exit code will return to the
        ; caller of RDI_SingleStep with the correct stack and
        ; processor state:
        LDR     a2,=VB1                 ; variable base address in a2
        STR     lr,[a2,#DebugLR-VB1]    ; return location
        STR     sp,[a2,#DebugSP-VB1]    ; stack pointer
        MRS     a1,CPSR                 ; current processor mode
        STR     a1,[a2,#DebugCPSR-VB1]
        ;
        ; The RUBY2 SingleStep mode operates when the processor is
        ; *NOT* in ABT mode. NOTE: This means that when using the
        ; hardware single-stepping features of the RUBY2, care
        ; must be taken when debugging ABT mode code.
        ;
        ; First load *MOST* of the processor state. This is a close
        ; copy of the BP_Go code used by RDI_Execute. It would be
        ; worth spending some time merging the code to try and save
        ; some code space.
        ;
        ; restore SVC (Supervisor) registers:
        MOV     r0,#(SVCmode | NoInt)
        MSR     CPSR,r0         ; in SVC mode, with interrupts disabled
        ADR     r0,BP_SVC_sp    ; point r0 at SVC sp
        LDMIA   r0,{sp-lr}      ; restore SVC registers sp and lr
        LDR     r0,BP_SVC_SPSR  ; restore SPSR of SVC mode
        MSR     SPSR,r0
        ;
        ; restore UNDefined registers:
        MOV     r0,#(UNDEFmode | NoInt)
        MSR     CPSR,r0         ; in UND mode, with interrupts disabled
        ADR     r0,BP_UND_sp    ; point r0 at UND sp
        LDMIA   r0,{sp-lr}      ; restore UND registers sp and lr
        LDR     r0,BP_UND_SPSR  ; restore SPSR of UND mode
        MSR     SPSR,r0
        ;
        ; restore IRQ registers:
        MOV     r0,#(IRQmode | NoInt)
        MSR     CPSR,r0         ; in IRQ mode, with interrupts disabled
        ADR     r0,BP_IRQ_sp    ; point r0 at IRQ sp
        LDMIA   r0,{sp-lr}      ; restore IRQ registers sp and lr
        LDR     r0,BP_IRQ_SPSR  ; restore SPSR of IRQ mode
        MSR     SPSR,r0
        ;
        ; restore FIQ registers:
        MOV     r0,#(FIQmode | NoInt)
        MSR     CPSR,r0         ; in FIQ mode, with interrupts disabled
        ADR     r0,BP_FIQ_r8    ; point r0 at FIQ r8
        LDMIA   r0,{r8-lr}      ; restore FIQ registers r8 to r14
        LDR     r0,BP_FIQ_SPSR  ; restore SPSR of FIQ mode
        MSR     SPSR,r0
        ;
        ; enter ABT mode for special single step support:
        MOV     r0,#(ABORTmode | NoInt)
        MSR     CPSR,r0         ; in ABT mode, with interrupts disabled
        ;
        ; Since we are about to execute the target "application" we
        ; need to be able to accept user-interrupt requests. We do
        ; this by allowing serial interrupts, which can be used
        ; received asynchronously, without the requirement for the
        ; executing code to poll DEMON.
        allowCTRLC  r0,r1
        ;
        ; restore USR registers:
        ; NOTE: We keep r0 around as a work register, to avoid losing
        ; our ABT mode stack pointer.
        ADR     r0,BP_r1        ; reference the stored USR registers
        LDMIA   r0,{r1-lr}^     ; restore most of the USR state
        NOP
        ;
        ; Whilst in ABT mode we can enable the hardware single-stepping:
        MOV     r0,#IOBase                      ; reference the I/O hardware
        LDRB    lr,[r0,#DEBUG]                  ; load the current DEBUG control
        ORR     lr,lr,#DEBUG_SINGLE_STEP        ; enable the hardware single step
        STRB    lr,[r0,#DEBUG]                  ; and update the DEBUG control
        ;
        ; NOTE: The normal C-Demon operation is for interrupts to be
        ; disabled whilst the host has control, and for
        ; communications to be polled. This means that if any
        ; interrupt sources are enabled they may have triggered and
        ; have a pending interrupt (i.e. especially timers).  This
        ; means that as soon as we single-step we will probably enter
        ; the IRQ handler code.
        ;
        ; Since the RUBY2 single-step support is not a TRUE
        ; single-step engine, and mostly users will want to be
        ; stepping through their foreground code, we explicitly
        ; disable interrupts whilst performing single-stepping. If a
        ; user wants to single step through an IRQ or FIQ handler,
        ; they should set a breakpoint on the relevant vector and
        ; execute their foreground (or sit in a waiting loop) until
        ; the event occurs. They can then single step through this
        ; (now in the debug foreground) code.
        ;
        ; However this may cause strange effects when single-stepping
        ; through instructions that enable processor interrupts.
        ;
        LDR     r0,BP_r0        ; restore USR r0
        LDR     lr,BP_CPSR      ; desired PSR
        ;
        STR     lr,BP_SS_CPSR   ; copy of pre-singlestep PSR
        ORR     lr,lr,#NoInt    ; disable processor interrupts
        ;
        MSR     SPSR,lr         ; and put into the ABT SPSR
        LDR     lr,BP_pc        ; recover the required PC
        MOVS    pc,lr           ; and re-enter the code
        ;
        ; NOTE: This code loses the ABT lr and SPSR registers.
        ; This means that RUBY2 hardware single stepping cannot be
        ; used to debug ABT mode code.
        ]       ; RUBY2 and HWBREAKPOINT

        ; ---------------------------------------------------------------------
        ; RDI_Go
        ; ======
RDI_Go
        ; The following ensures that the SWIExit or breakpoint code
        ; will return to the caller of RDI_Go, with the correct stack
        ; and processor state:
        LDR     a2, =VB1                ;place variable base addr in a2
        STR     lr,[a2,#DebugLR-VB1]    ;the return location
        STR     sp,[a2,#DebugSP-VB1]    ;save our stack pointer
        MRS     a1,CPSR                 ;get current mode
        STR     a1,[a2,#DebugCPSR-VB1]  ;save our current mode
        ; Fall through to BP_Go code...
        ;
        ; BP_Go
        ; =====
        ; Start executing from BP_pc, with the appropriate environment
        ; restored. This routine is normally entered with a GOTO as
        ; ALL the registers and the environment itself is taken from the
        ; values stored in the "BP_" registers. NOTE: you must be in
        ; SVC or some other priviledged mode (NOT User) in order to
        ; make the following work.
        ;
BP_Go
        ; restore SVC (Supervisor) registers:
        MOV     r0,#(SVCmode | NoInt)
        MSR     CPSR,r0         ; in SVC mode, with interrupts disabled
        ADR     r0,BP_SVC_sp    ; point r0 at SVC sp
        LDMIA   r0,{sp-lr}      ; restore SVC registers sp and lr
        LDR     r0,BP_SVC_SPSR  ; restore SPSR of SVC mode
        MSR     SPSR,r0
        ;
        ; restore ABT (Abort) registers:
        MOV     r0,#(ABORTmode | NoInt)
        MSR     CPSR,r0         ; in ABT mode, with interrupts disabled
        ADR     r0,BP_ABT_sp    ; point r0 at ABT sp
        LDMIA   r0,{sp-lr}      ; restore ABT registers sp and lr
        LDR     r0,BP_ABT_SPSR  ; restore SPSR of ABT mode
        MSR     SPSR,r0
        ;
        ; restore UNDefined registers:
        MOV     r0,#(UNDEFmode | NoInt)
        MSR     CPSR,r0         ; in UND mode, with interrupts disabled
        ADR     r0,BP_UND_sp    ; point r0 at UND sp
        LDMIA   r0,{sp-lr}      ; restore UND registers sp and lr
        LDR     r0,BP_UND_SPSR  ; restore SPSR of UND mode
        MSR     SPSR,r0
        ;
        ; restore IRQ registers:
        MOV     r0,#(IRQmode | NoInt)
        MSR     CPSR,r0         ; in IRQ mode, with interrupts disabled
        ADR     r0,BP_IRQ_sp    ; point r0 at IRQ sp
        LDMIA   r0,{sp-lr}      ; restore IRQ registers sp and lr
        LDR     r0,BP_IRQ_SPSR  ; restore SPSR of IRQ mode
        MSR     SPSR,r0
        ;
        ; restore FIQ registers:
        MOV     r0,#(FIQmode | NoInt)
        MSR     CPSR,r0         ; in FIQ mode, with interrupts disabled
        ADR     r0,BP_FIQ_r8    ; point r0 at FIQ r8 !!! bug fixed - was pointing at sp
        LDMIA   r0,{r8-lr}      ; restore FIQ registers r8 to r14
        LDR     r0,BP_FIQ_SPSR  ; restore SPSR of FIQ mode
        MSR     SPSR,r0
        ;
        ; Since we are about to execute the target "application" we
        ; need to be able to accept user-interrupt requests. We do
        ; this by allowing serial interrupts, which can be used
        ; received asynchronously, without the requirement for the
        ; executing code to poll DEMON.
        allowCTRLC  r0,r1
        ;
        ; load common registers and USER registers (r13 and r14):
        ADR     r0,BP_r1        ; do NOT use LDR r0,=BP_r1 (it produces ABSOLUTE code)
        LDMIA   r0,{r1-lr}^     ; We want relocatable code
        ;
        ; now set mode we are to start up in:
        LDR     r0,BP_CPSR      ; set the desired CPSR
        MSR     CPSR,r0
        ;
        LDR     r0,BP_r0        ; finally get r0
        LDR     pc,BP_pc        ; and then goto PC

        ; ---------------------------------------------------------------------
        ; Register Images
        ; ===============
        ;
BP_RDP_STS      DCD     0                       ; Status to return about type of break

BP_type         DCD     0                       ; Which breakpoint

BP_CPSR         DCD     USRmode                 ; All modes share the CPSR

                [       (RUBY2 :LAND: HWBREAKPOINT)
BP_SS_CPSR      DCD     0       ; special HW single-stepping support
                ; See the comments in the RDI_SingleStep code above for the
                ; reasons behind this variable.
                ]       ; (RUBY2 and HWBREAKPOINT)

BP_pc           DCD     0                       ; All modes share the PC

BP_r0           DCD     0                       ; All modes share r0 to r7
BP_r1           DCD     0
BP_r2           DCD     0
BP_r3           DCD     0
BP_r4           DCD     0
BP_r5           DCD     0
BP_r6           DCD     0
BP_r7           DCD     0

BP_r8           DCD     0                       ; All modes except FIQ share r8 to r12 as well
BP_r9           DCD     0
BP_r10          DCD     0
BP_r11          DCD     0
BP_r12          DCD     0

BP_USR_sp       DCD     UserStackBase           ; sp (r13) and lr (r14) for USR Mode
BP_USR_lr       DCD     0

BP_FIQ_r8       DCD     0                       ; FIQ r8 to r12
BP_FIQ_r9       DCD     0
BP_FIQ_r10      DCD     0
BP_FIQ_r11      DCD     0
BP_FIQ_r12      DCD     0
BP_FIQ_sp       DCD     FIQStackBase            ; sp (r13) and lr (r14) for FIQ Mode
BP_FIQ_lr       DCD     0
BP_FIQ_SPSR     DCD     (USRmode | NoInt)       ; SPSR for FIQ

BP_SVC_sp       DCD     SVCStackBase            ; sp (r13) and lr (r14) for Supervisor Mode
BP_SVC_lr       DCD     0
BP_SVC_SPSR     DCD     (USRmode | NoInt)       ; SPSR for Supervisor Mode


BP_ABT_sp       DCD     AbortStackBase          ; sp (r13) and lr (r14) for Abort Mode
BP_ABT_lr       DCD     0
BP_ABT_SPSR     DCD     (USRmode | NoInt)       ; SPSR for Abort Mode


BP_IRQ_sp       DCD     IRQStackBase            ; sp (r13) and lr (r14) for IRQ Mode
BP_IRQ_lr       DCD     0
BP_IRQ_SPSR     DCD     (USRmode | NoInt)       ; SPSR for IRQ Mode


BP_UND_sp       DCD     UndefStackBase          ; sp (r13) and lr (r14) for Undefined Mode
BP_UND_lr       DCD     0
BP_UND_SPSR     DCD     (USRmode | NoInt)       ; SPSR for Undefined Mode

        ; ---------------------------------------------------------------------

                LTORG                   ; allow for symbol storage
BP_addr         %       (NumBP * 4)     ; location of respective BP
BP_inst         %       (NumBP * 4)     ; instruction at respective BP

        ; ---------------------------------------------------------------------
        ; Other C-Demon variable requirements:

DriverRAM       %       0           ; simple accounting definition

TickVal         %       4
                %       4           ; padding (what is this for?)
                ; The "Handlers" vector *MUST* be initialised to zero, since
                ; PC values of NULL are used to denote that a handler has
                ; not been attached.
Handlers        %       (4 * 8 * 2) ; Vector of event handlers
HandlersEnd     %       0

CaughtVectors   %       4
ErrorP          %       4
ExpectedAbort   %       24

DriverRAMend    %       0           ; simple accounting definition

        ; ---------------------------------------------------------------------

VB1                     %       0       ; variable base for old level 1 code

TimeZero                %       4
FPRegsAddr              %       4
FPAbortPossible         %       4       ; a byte
SWIState                %       4       ; a byte

DebuggeeRunning         %       4       ; 0 => not running
DebugLR                 %       4
DebugSP                 %       4
DebugCPSR               %       4

RAMBranchThrough0       %       (8 * 4) ; 0x18 ;** 8 off an 8-bit constant

ResetVectorCopy         %       4
EndianWord              %       4
RAMSize                 %       4
ROMEnd                  %       4
RDIreply                %       4

MMUtype                 %       4
LEDimage                %       4
PONimage                %       4       ; actually only 1 bit is used
                        %       4
FP_image                %       12
                        %       12      ; spare
                        %       4       ; what is this word for?
                        %       4       ; what is this word for?
WorkingDump             %       (16 * 4)        ; 0x40 bytes
WorkingDumpEnd          %       ((8 * 12) + 4)  ; 0x64 bytes
FPDumpEnd               %       12

Level1RAMend            %       0

        ; ---------------------------------------------------------------------

LowMemoryEnd
        ; The following ASSERT is a bit nasty at the moment. The FPE code
        ; assumes it can be placed from 0x1400 onwards. The C-Demon
        ; also starts placing the ARM exception mode stacks from
        ; ARMSystemStacks onwards. This check assumes that the data
        ; area we have just created will fit beneath the fixed offsets.
        ;
        ASSERT ((LowMemoryEnd - LowMemoryStart) < ARMSystemStacks)

        ; ---------------------------------------------------------------------
        END     ; EOF vectors.s
